package EV::Telegram::TDLib::Schema;

use strict;
use warnings;

our $VERSION = '0.03';

# Generated by author/gen-schema.pl from the td_api.h that Alien::TDLib
# ships. Do not edit; regenerate instead.

our $TDLIB_VERSION = '1.8.66';

our %FUNCTIONS = (
    acceptCall                                   => 'call_id protocol',
    acceptOauthRequest                           => 'url match_code allow_write_access allow_phone_number_access',
    acceptTermsOfService                         => 'terms_of_service_id',
    activateStoryStealthMode                     => '',
    addBotMediaPreview                           => 'bot_user_id language_code content',
    addChatFolderByInviteLink                    => 'invite_link chat_ids',
    addChatMember                                => 'chat_id user_id forward_limit',
    addChatMembers                               => 'chat_id user_ids',
    addChatToList                                => 'chat_id chat_list',
    addChecklistTasks                            => 'chat_id message_id tasks',
    addContact                                   => 'user_id contact share_phone_number',
    addCustomServerLanguagePack                  => 'language_pack_id',
    addFavoriteSticker                           => 'sticker',
    addFileToDownloads                           => 'file_id chat_id message_id priority',
    addGiftCollectionGifts                       => 'owner_id collection_id received_gift_ids',
    addLocalMessage                              => 'chat_id sender_id reply_to disable_notification input_message_content',
    addLogMessage                                => 'verbosity_level text',
    addLoginPasskey                              => 'client_data attestation_object',
    addMessageReaction                           => 'chat_id message_id reaction_type is_big update_recent_reactions',
    addNetworkStatistics                         => 'entry',
    addOffer                                     => 'chat_id message_id options',
    addPendingLiveStoryReaction                  => 'group_call_id star_count',
    addPendingPaidMessageReaction                => 'chat_id message_id star_count type',
    addPollOption                                => 'chat_id message_id option',
    addProfileAudio                              => 'audio',
    addProxy                                     => 'proxy enable comment',
    addQuickReplyShortcutInlineQueryResultMessage => 'shortcut_name reply_to_message_id query_id result_id hide_via_bot',
    addQuickReplyShortcutMessage                 => 'shortcut_name reply_to_message_id input_message_content',
    addQuickReplyShortcutMessageAlbum            => 'shortcut_name reply_to_message_id input_message_contents',
    addRecentSticker                             => 'is_attached sticker',
    addRecentlyFoundChat                         => 'chat_id',
    addSavedAnimation                            => 'animation',
    addSavedNotificationSound                    => 'sound',
    addStickerToSet                              => 'user_id name sticker',
    addStoryAlbumStories                         => 'chat_id story_album_id story_ids',
    addTextCompositionStyle                      => 'name',
    addWebBrowserSettingsException               => 'open_external_browser url',
    allowBotToSendMessages                       => 'bot_user_id',
    allowUnpaidMessagesFromUser                  => 'user_id refund_payments',
    answerCallbackQuery                          => 'callback_query_id text show_alert url cache_time',
    answerChatJoinRequestQuery                   => 'query_id result url',
    answerCustomQuery                            => 'custom_query_id data',
    answerGuestQuery                             => 'guest_query_id result',
    answerInlineQuery                            => 'inline_query_id is_personal button results cache_time next_offset',
    answerPreCheckoutQuery                       => 'pre_checkout_query_id error_message',
    answerShippingQuery                          => 'shipping_query_id shipping_options error_message',
    answerWebAppQuery                            => 'web_app_query_id result',
    applyPremiumGiftCode                         => 'code',
    approveSuggestedPost                         => 'chat_id message_id send_date',
    assignStoreTransaction                       => 'transaction purpose',
    banChatMember                                => 'chat_id member_id banned_until_date revoke_messages',
    banGroupCallParticipants                     => 'group_call_id user_ids',
    blockMessageSenderFromReplies                => 'message_id delete_message delete_all_messages report_spam',
    boostChat                                    => 'chat_id slot_ids',
    buyGiftUpgrade                               => 'owner_id prepaid_upgrade_hash star_count',
    canBotSendMessages                           => 'bot_user_id',
    canPostStory                                 => 'chat_id',
    canPurchaseFromStore                         => 'purpose',
    canSendGift                                  => 'gift_id',
    canSendMessageToUser                         => 'user_id only_local',
    canTransferOwnership                         => '',
    cancelDownloadFile                           => 'file_id only_if_pending',
    cancelPasswordReset                          => '',
    cancelPreliminaryUploadFile                  => 'file_id',
    cancelRecoveryEmailAddressVerification       => '',
    changeImportedContacts                       => 'contacts',
    changeStickerSet                             => 'set_id is_installed is_archived',
    changeWebBrowserSettings                     => 'open_external_browser display_close_button',
    checkAuthenticationBotToken                  => 'token',
    checkAuthenticationCode                      => 'code',
    checkAuthenticationEmailCode                 => 'code',
    checkAuthenticationPasskey                   => 'credential_id client_data authenticator_data signature user_handle',
    checkAuthenticationPassword                  => 'password',
    checkAuthenticationPasswordRecoveryCode      => 'recovery_code',
    checkAuthenticationPremiumPurchase           => 'premium_day_count currency amount',
    checkAuthenticationWebToken                  => 'token dc_id',
    checkBotUsername                             => 'username',
    checkChatFolderInviteLink                    => 'invite_link',
    checkChatInviteLink                          => 'invite_link',
    checkChatUsername                            => 'chat_id username',
    checkCreatedPublicChatsLimit                 => 'type',
    checkEmailAddressVerificationCode            => 'code',
    checkLoginEmailAddressCode                   => 'code',
    checkOauthRequestMatchCode                   => 'url match_code',
    checkPasswordRecoveryCode                    => 'recovery_code',
    checkPhoneNumberCode                         => 'code',
    checkPremiumGiftCode                         => 'code',
    checkQuickReplyShortcutName                  => 'name',
    checkRecoveryEmailAddressCode                => 'code',
    checkStickerSetName                          => 'name',
    checkWebAppFileDownload                      => 'bot_user_id file_name url',
    cleanFileName                                => 'file_name',
    clearAllDraftMessages                        => 'exclude_secret_chats',
    clearAutosaveSettingsExceptions              => '',
    clearImportedContacts                        => '',
    clearRecentEmojiStatuses                     => '',
    clearRecentReactions                         => '',
    clearRecentStickers                          => 'is_attached',
    clearRecentlyFoundChats                      => '',
    clearSearchedForTags                         => 'clear_cashtags',
    clickAnimatedEmojiMessage                    => 'chat_id message_id',
    clickChatSponsoredMessage                    => 'chat_id message_id is_media_click from_fullscreen',
    clickPremiumSubscriptionButton               => '',
    clickVideoMessageAdvertisement               => 'advertisement_unique_id',
    close                                        => '',
    closeChat                                    => 'chat_id',
    closeGiftAuction                             => 'gift_id',
    closeSecretChat                              => 'secret_chat_id',
    closeStory                                   => 'story_poster_chat_id story_id',
    closeWebApp                                  => 'web_app_launch_id',
    commitPendingLiveStoryReactions              => 'group_call_id',
    commitPendingPaidMessageReactions            => 'chat_id message_id',
    composeRichMessageWithAi                     => 'message translate_to_language_code style_name custom_prompt add_emojis',
    composeTextWithAi                            => 'text translate_to_language_code style_name add_emojis',
    confirmBusinessConnectedBot                  => 'bot_user_id',
    confirmQrCodeAuthentication                  => 'link',
    confirmSession                               => 'session_id',
    connectAffiliateProgram                      => 'affiliate bot_user_id',
    craftGift                                    => 'received_gift_ids',
    createBasicGroupChat                         => 'basic_group_id force',
    createBot                                    => 'manager_bot_user_id name username via_link',
    createBusinessChatLink                       => 'link_info',
    createCall                                   => 'user_id protocol is_video',
    createChatFolder                             => 'folder',
    createChatFolderInviteLink                   => 'chat_folder_id name chat_ids',
    createChatInviteLink                         => 'chat_id name expiration_date member_limit creates_join_request',
    createChatSubscriptionInviteLink             => 'chat_id name subscription_pricing',
    createForumTopic                             => 'chat_id name is_name_implicit icon',
    createGiftCollection                         => 'owner_id name received_gift_ids',
    createGroupCall                              => 'join_parameters',
    createInvoiceLink                            => 'business_connection_id invoice',
    createNewBasicGroupChat                      => 'user_ids title message_auto_delete_time',
    createNewSecretChat                          => 'user_id',
    createNewStickerSet                          => 'user_id title name sticker_type needs_repainting stickers source',
    createNewSupergroupChat                      => 'title is_forum is_channel description location message_auto_delete_time for_import',
    createPrivateChat                            => 'user_id force',
    createRichMessageWithAi                      => 'prompt language_code add_emojis',
    createSecretChat                             => 'secret_chat_id',
    createStoryAlbum                             => 'story_poster_chat_id name story_ids',
    createSupergroupChat                         => 'supergroup_id force',
    createTemporaryPassword                      => 'password valid_for',
    createTextCompositionStyle                   => 'title custom_emoji_id prompt show_creator',
    createVideoChat                              => 'chat_id title start_date is_rtmp_stream',
    declineGroupCallInvitation                   => 'chat_id message_id',
    declineOauthRequest                          => 'url',
    declineSuggestedPost                         => 'chat_id message_id comment',
    decryptGroupCallData                         => 'group_call_id participant_id data_channel data',
    deleteAccount                                => 'reason password',
    deleteAllCallMessages                        => 'revoke',
    deleteAllRecentMessageReactionsFromSender    => 'chat_id sender_id',
    deleteAllRevokedChatInviteLinks              => 'chat_id creator_user_id',
    deleteBotMediaPreviews                       => 'bot_user_id language_code file_ids',
    deleteBusinessChatLink                       => 'link',
    deleteBusinessConnectedBot                   => 'bot_user_id',
    deleteBusinessMessages                       => 'business_connection_id message_ids',
    deleteBusinessStory                          => 'business_connection_id story_id',
    deleteChat                                   => 'chat_id',
    deleteChatBackground                         => 'chat_id restore_previous',
    deleteChatFolder                             => 'chat_folder_id leave_chat_ids',
    deleteChatFolderInviteLink                   => 'chat_folder_id invite_link',
    deleteChatHistory                            => 'chat_id remove_from_chat_list revoke',
    deleteChatMessagesByDate                     => 'chat_id min_date max_date revoke',
    deleteChatMessagesBySender                   => 'chat_id sender_id',
    deleteChatReplyMarkup                        => 'chat_id message_id',
    deleteCommands                               => 'scope language_code',
    deleteDefaultBackground                      => 'for_dark_theme',
    deleteDirectMessagesChatTopicHistory         => 'chat_id topic_id',
    deleteDirectMessagesChatTopicMessagesByDate  => 'chat_id topic_id min_date max_date',
    deleteEphemeralMessage                       => 'chat_id receiver_user_id ephemeral_message_id',
    deleteFile                                   => 'file_id',
    deleteForumTopic                             => 'chat_id forum_topic_id',
    deleteGiftCollection                         => 'owner_id collection_id',
    deleteGroupCallMessages                      => 'group_call_id message_ids report_spam',
    deleteGroupCallMessagesBySender              => 'group_call_id sender_id report_spam',
    deleteLanguagePack                           => 'language_pack_id',
    deleteMessageReactionsFromSender             => 'chat_id message_id sender_id',
    deleteMessages                               => 'chat_id message_ids revoke',
    deletePassportElement                        => 'type',
    deletePollOption                             => 'chat_id message_id option_id',
    deleteProfilePhoto                           => 'profile_photo_id',
    deleteQuickReplyShortcut                     => 'shortcut_id',
    deleteQuickReplyShortcutMessages             => 'shortcut_id message_ids',
    deleteRevokedChatInviteLink                  => 'chat_id invite_link',
    deleteSavedCredentials                       => '',
    deleteSavedMessagesTopicHistory              => 'saved_messages_topic_id',
    deleteSavedMessagesTopicMessagesByDate       => 'saved_messages_topic_id min_date max_date',
    deleteSavedOrderInfo                         => '',
    deleteStickerSet                             => 'name',
    deleteStory                                  => 'story_poster_chat_id story_id',
    deleteStoryAlbum                             => 'chat_id story_album_id',
    deleteTextCompositionStyle                   => 'name',
    destroy                                      => '',
    disableAllSupergroupUsernames                => 'supergroup_id',
    disableProxy                                 => '',
    discardCall                                  => 'call_id is_disconnected invite_link duration is_video connection_id',
    disconnectAffiliateProgram                   => 'affiliate url',
    disconnectAllWebsites                        => '',
    disconnectWebsite                            => 'website_id',
    downloadFile                                 => 'file_id priority offset limit synchronous',
    dropGiftOriginalDetails                      => 'received_gift_id star_count',
    editBotMediaPreview                          => 'bot_user_id language_code file_id content',
    editBusinessChatLink                         => 'link link_info',
    editBusinessMessageCaption                   => 'business_connection_id chat_id message_id reply_markup caption show_caption_above_media',
    editBusinessMessageChecklist                 => 'business_connection_id chat_id message_id reply_markup checklist',
    editBusinessMessageLiveLocation              => 'business_connection_id chat_id message_id reply_markup location',
    editBusinessMessageMedia                     => 'business_connection_id chat_id message_id reply_markup input_message_content',
    editBusinessMessageReplyMarkup               => 'business_connection_id chat_id message_id reply_markup',
    editBusinessMessageText                      => 'business_connection_id chat_id message_id reply_markup input_message_content',
    editBusinessStory                            => 'story_poster_chat_id story_id content areas caption privacy_settings',
    editChatFolder                               => 'chat_folder_id folder',
    editChatFolderInviteLink                     => 'chat_folder_id invite_link name chat_ids',
    editChatInviteLink                           => 'chat_id invite_link name expiration_date member_limit creates_join_request',
    editChatSubscriptionInviteLink               => 'chat_id invite_link name',
    editCustomLanguagePackInfo                   => 'info',
    editEphemeralMessage                         => 'chat_id receiver_user_id ephemeral_message_id reply_markup input_message_content',
    editForumTopic                               => 'chat_id forum_topic_id name edit_icon_custom_emoji icon_custom_emoji_id',
    editInlineMessageCaption                     => 'inline_message_id reply_markup caption show_caption_above_media',
    editInlineMessageLiveLocation                => 'inline_message_id reply_markup location',
    editInlineMessageMedia                       => 'inline_message_id reply_markup input_message_content',
    editInlineMessageReplyMarkup                 => 'inline_message_id reply_markup',
    editInlineMessageText                        => 'inline_message_id reply_markup input_message_content',
    editMessageCaption                           => 'chat_id message_id reply_markup caption show_caption_above_media',
    editMessageChecklist                         => 'chat_id message_id reply_markup checklist',
    editMessageLiveLocation                      => 'chat_id message_id reply_markup location',
    editMessageMedia                             => 'chat_id message_id reply_markup input_message_content',
    editMessageReplyMarkup                       => 'chat_id message_id reply_markup',
    editMessageSchedulingState                   => 'chat_id message_id scheduling_state',
    editMessageText                              => 'chat_id message_id reply_markup input_message_content',
    editProxy                                    => 'proxy_id proxy enable comment',
    editQuickReplyMessage                        => 'shortcut_id message_id input_message_content',
    editStarSubscription                         => 'subscription_id is_canceled',
    editStory                                    => 'story_poster_chat_id story_id content areas caption',
    editStoryCover                               => 'story_poster_chat_id story_id cover_frame_timestamp',
    editTextCompositionStyle                     => 'name title custom_emoji_id prompt show_creator',
    editUserStarSubscription                     => 'user_id telegram_payment_charge_id is_canceled',
    enableProxy                                  => 'proxy_id',
    encryptGroupCallData                         => 'group_call_id data_channel data unencrypted_prefix_size',
    endGroupCall                                 => 'group_call_id',
    endGroupCallRecording                        => 'group_call_id',
    endGroupCallScreenSharing                    => 'group_call_id',
    finishFileGeneration                         => 'generation_id error',
    fixRichMessageWithAi                         => 'message',
    fixTextWithAi                                => 'text',
    forwardMessages                              => 'chat_id topic_id from_chat_id message_ids options send_copy remove_caption',
    getAccountTtl                                => '',
    getActiveSessions                            => '',
    getAllPassportElements                       => 'password',
    getAllStickerEmojis                          => 'sticker_type query chat_id return_only_main_emoji',
    getAnimatedEmoji                             => 'emoji',
    getApplicationConfig                         => '',
    getApplicationDownloadLink                   => '',
    getArchiveChatListSettings                   => '',
    getArchivedStickerSets                       => 'sticker_type offset_sticker_set_id limit',
    getAttachedStickerSets                       => 'file_id',
    getAttachmentMenuBot                         => 'bot_user_id',
    getAuthenticationPasskeyParameters           => '',
    getAuthorizationState                        => '',
    getAutoDownloadSettingsPresets               => '',
    getAutosaveSettings                          => '',
    getAvailableChatBoostSlots                   => '',
    getAvailableGifts                            => '',
    getBackgroundUrl                             => 'name type',
    getBankCardInfo                              => 'bank_card_number',
    getBasicGroup                                => 'basic_group_id',
    getBasicGroupFullInfo                        => 'basic_group_id',
    getBlockedMessageSenders                     => 'block_list offset limit',
    getBotInfoDescription                        => 'bot_user_id language_code',
    getBotInfoShortDescription                   => 'bot_user_id language_code',
    getBotMediaPreviewInfo                       => 'bot_user_id language_code',
    getBotMediaPreviews                          => 'bot_user_id',
    getBotName                                   => 'bot_user_id language_code',
    getBotSimilarBotCount                        => 'bot_user_id return_local',
    getBotSimilarBots                            => 'bot_user_id',
    getBusinessAccountStarAmount                 => 'business_connection_id',
    getBusinessChatLinkInfo                      => 'link_name',
    getBusinessChatLinks                         => '',
    getBusinessConnectedBot                      => '',
    getBusinessConnection                        => 'connection_id',
    getBusinessFeatures                          => 'source',
    getCallbackQueryAnswer                       => 'chat_id message_id payload',
    getCallbackQueryMessage                      => 'chat_id message_id callback_query_id',
    getChat                                      => 'chat_id',
    getChatActiveStories                         => 'chat_id',
    getChatAdministrators                        => 'chat_id',
    getChatArchivedStories                       => 'chat_id from_story_id limit',
    getChatAvailableMessageSenders               => 'chat_id',
    getChatAvailablePaidMessageReactionSenders   => 'chat_id',
    getChatBoostFeatures                         => 'is_channel',
    getChatBoostLevelFeatures                    => 'is_channel level',
    getChatBoostLink                             => 'chat_id',
    getChatBoostLinkInfo                         => 'url',
    getChatBoostStatus                           => 'chat_id',
    getChatBoosts                                => 'chat_id only_gift_codes offset limit',
    getChatEventLog                              => 'chat_id query from_event_id limit filters user_ids',
    getChatFolder                                => 'chat_folder_id',
    getChatFolderChatCount                       => 'folder',
    getChatFolderChatsToLeave                    => 'chat_folder_id',
    getChatFolderDefaultIconName                 => 'folder',
    getChatFolderInviteLinks                     => 'chat_folder_id',
    getChatFolderNewChats                        => 'chat_folder_id',
    getChatHistory                               => 'chat_id from_message_id offset limit only_local',
    getChatInviteLink                            => 'chat_id invite_link',
    getChatInviteLinkCounts                      => 'chat_id',
    getChatInviteLinkMembers                     => 'chat_id invite_link only_with_expired_subscription offset_member limit',
    getChatInviteLinks                           => 'chat_id creator_user_id is_revoked offset_date offset_invite_link limit',
    getChatJoinRequests                          => 'chat_id invite_link query offset_request limit',
    getChatListsToAddChat                        => 'chat_id',
    getChatMember                                => 'chat_id member_id',
    getChatMessageByDate                         => 'chat_id date',
    getChatMessageCalendar                       => 'chat_id topic_id filter from_message_id',
    getChatMessageCount                          => 'chat_id topic_id filter return_local',
    getChatMessagePosition                       => 'chat_id topic_id filter message_id',
    getChatNotificationSettingsExceptions        => 'scope compare_sound',
    getChatOwnerAfterLeaving                     => 'chat_id',
    getChatPinnedMessage                         => 'chat_id',
    getChatPostedToChatPageStories               => 'chat_id from_story_id limit',
    getChatRevenueStatistics                     => 'chat_id is_dark',
    getChatRevenueTransactions                   => 'chat_id offset limit',
    getChatRevenueWithdrawalUrl                  => 'chat_id password',
    getChatScheduledMessages                     => 'chat_id',
    getChatSimilarChatCount                      => 'chat_id return_local',
    getChatSimilarChats                          => 'chat_id',
    getChatSparseMessagePositions                => 'chat_id filter from_message_id limit saved_messages_topic_id',
    getChatSponsoredMessages                     => 'chat_id',
    getChatStatistics                            => 'chat_id is_dark',
    getChatStoryAlbums                           => 'chat_id',
    getChatStoryInteractions                     => 'story_poster_chat_id story_id reaction_type prefer_forwards offset limit',
    getChats                                     => 'chat_list limit',
    getChatsForChatFolderInviteLink              => 'chat_folder_id',
    getChatsToPostStories                        => '',
    getCloseFriends                              => '',
    getCollectibleItemInfo                       => 'type',
    getCommands                                  => 'scope language_code',
    getConnectedAffiliateProgram                 => 'affiliate bot_user_id',
    getConnectedAffiliatePrograms                => 'affiliate offset limit',
    getConnectedWebsites                         => '',
    getContacts                                  => '',
    getCountries                                 => '',
    getCountry                                   => 'country_code',
    getCountryCode                               => '',
    getCountryFlagEmoji                          => 'country_code',
    getCreatedPublicChats                        => 'type',
    getCurrentState                              => '',
    getCurrentWeather                            => 'location',
    getCustomEmojiReactionAnimations             => '',
    getCustomEmojiStickers                       => 'custom_emoji_ids',
    getDatabaseStatistics                        => '',
    getDeepLinkInfo                              => 'link',
    getDefaultBackgroundCustomEmojiStickers      => '',
    getDefaultChatEmojiStatuses                  => '',
    getDefaultChatPhotoCustomEmojiStickers       => '',
    getDefaultEmojiStatuses                      => '',
    getDefaultMessageAutoDeleteTime              => '',
    getDefaultProfilePhotoCustomEmojiStickers    => '',
    getDirectMessagesChatTopic                   => 'chat_id topic_id',
    getDirectMessagesChatTopicHistory            => 'chat_id topic_id from_message_id offset limit',
    getDirectMessagesChatTopicMessageByDate      => 'chat_id topic_id date',
    getDirectMessagesChatTopicRevenue            => 'chat_id topic_id',
    getDisallowedChatEmojiStatuses               => '',
    getEmojiCategories                           => 'type',
    getEmojiReaction                             => 'emoji',
    getEmojiSuggestionsUrl                       => 'language_code',
    getExternalLink                              => 'link allow_write_access',
    getExternalLinkInfo                          => 'link',
    getFavoriteStickers                          => '',
    getFile                                      => 'file_id',
    getFileDownloadedPrefixSize                  => 'file_id offset',
    getFileExtension                             => 'mime_type',
    getFileMimeType                              => 'file_name',
    getForumTopic                                => 'chat_id forum_topic_id',
    getForumTopicDefaultIcons                    => '',
    getForumTopicHistory                         => 'chat_id forum_topic_id from_message_id offset limit',
    getForumTopicLink                            => 'chat_id forum_topic_id',
    getForumTopics                               => 'chat_id query offset_date offset_message_id offset_forum_topic_id limit',
    getFullRichMessage                           => 'chat_id message_id',
    getGameHighScores                            => 'chat_id message_id user_id',
    getGiftAuctionAcquiredGifts                  => 'gift_id',
    getGiftAuctionState                          => 'auction_id',
    getGiftChatThemes                            => 'offset limit',
    getGiftCollections                           => 'owner_id',
    getGiftUpgradePreview                        => 'regular_gift_id',
    getGiftsForCrafting                          => 'regular_gift_id offset limit',
    getGiveawayInfo                              => 'chat_id message_id',
    getGramRevenueStatistics                     => 'is_dark',
    getGramWithdrawalUrl                         => 'password',
    getGreetingStickers                          => '',
    getGrossingWebAppBots                        => 'offset limit',
    getGroupCall                                 => 'group_call_id',
    getGroupCallParticipants                     => 'input_group_call limit',
    getGroupCallStreamSegment                    => 'group_call_id time_offset scale channel_id video_quality',
    getGroupCallStreams                          => 'group_call_id',
    getGroupsInCommon                            => 'user_id offset_chat_id limit',
    getGuardBotWebAppUrl                         => 'query_id parameters',
    getImportedContactCount                      => '',
    getInactiveSupergroupChats                   => '',
    getInlineGameHighScores                      => 'inline_message_id user_id',
    getInlineQueryResults                        => 'bot_user_id chat_id user_location query offset',
    getInstalledBackgrounds                      => 'for_dark_theme',
    getInstalledStickerSets                      => 'sticker_type',
    getInternalLink                              => 'type is_http',
    getInternalLinkType                          => 'link',
    getJsonString                                => 'json_value',
    getJsonValue                                 => 'json',
    getKeywordEmojis                             => 'text input_language_codes',
    getLanguagePackInfo                          => 'language_pack_id',
    getLanguagePackString                        => 'language_pack_database_path localization_target language_pack_id key',
    getLanguagePackStrings                       => 'language_pack_id keys',
    getLinkPreview                               => 'text link_preview_options',
    getLinkWebBrowserType                        => 'link',
    getLiveStoryAvailableMessageSenders          => 'group_call_id',
    getLiveStoryRtmpUrl                          => 'chat_id',
    getLiveStoryStreamer                         => 'group_call_id',
    getLiveStoryTopDonors                        => 'group_call_id',
    getLocalizationTargetInfo                    => 'only_local',
    getLogStream                                 => '',
    getLogTagVerbosityLevel                      => 'tag',
    getLogTags                                   => '',
    getLogVerbosityLevel                         => '',
    getLoginPasskeys                             => '',
    getLoginUrl                                  => 'chat_id message_id button_id allow_write_access',
    getLoginUrlInfo                              => 'chat_id message_id button_id',
    getMainWebApp                                => 'chat_id bot_user_id start_parameter parameters',
    getManagedBotAccessSettings                  => 'bot_user_id',
    getManagedBotToken                           => 'bot_user_id revoke',
    getMapThumbnailFile                          => 'location zoom width height scale chat_id',
    getMarkdownText                              => 'text',
    getMe                                        => '',
    getMenuButton                                => 'user_id',
    getMessage                                   => 'chat_id message_id',
    getMessageAddedReactions                     => 'chat_id message_id reaction_type offset limit',
    getMessageAuthor                             => 'chat_id message_id',
    getMessageAvailableReactions                 => 'chat_id message_id row_size',
    getMessageEffect                             => 'effect_id',
    getMessageEmbeddingCode                      => 'chat_id message_id for_album',
    getMessageFileType                           => 'message_file_head',
    getMessageImportConfirmationText             => 'chat_id',
    getMessageLink                               => 'chat_id message_id media_timestamp checklist_task_id poll_option_id for_album in_message_thread',
    getMessageLinkInfo                           => 'url',
    getMessageLocally                            => 'chat_id message_id',
    getMessageProperties                         => 'chat_id message_id',
    getMessagePublicForwards                     => 'chat_id message_id offset limit',
    getMessageReadDate                           => 'chat_id message_id',
    getMessageStatistics                         => 'chat_id message_id is_dark',
    getMessageThread                             => 'chat_id message_id',
    getMessageThreadHistory                      => 'chat_id message_id from_message_id offset limit',
    getMessageViewers                            => 'chat_id message_id',
    getMessages                                  => 'chat_id message_ids',
    getNetworkStatistics                         => 'only_current',
    getNewChatPrivacySettings                    => '',
    getOauthLinkInfo                             => 'url in_app_origin',
    getOption                                    => 'name',
    getOwnedBots                                 => '',
    getOwnedStickerSets                          => 'offset_sticker_set_id limit',
    getPaidMessageRevenue                        => 'user_id',
    getPasskeyParameters                         => '',
    getPassportAuthorizationForm                 => 'bot_user_id scope public_key nonce',
    getPassportAuthorizationFormAvailableElements => 'authorization_form_id password',
    getPassportElement                           => 'type password',
    getPasswordState                             => '',
    getPaymentForm                               => 'input_invoice theme',
    getPaymentReceipt                            => 'chat_id message_id',
    getPersonalChatHistory                       => 'user_id limit',
    getPhoneNumberInfo                           => 'phone_number_prefix',
    getPhoneNumberInfoSync                       => 'language_code phone_number_prefix',
    getPollOptionProperties                      => 'chat_id message_id poll_option_id',
    getPollVoteStatistics                        => 'chat_id message_id is_dark',
    getPollVoters                                => 'chat_id message_id option_id offset limit',
    getPreferredCountryLanguage                  => 'country_code',
    getPremiumFeatures                           => 'source',
    getPremiumGiftPaymentOptions                 => '',
    getPremiumGiveawayPaymentOptions             => 'boosted_chat_id',
    getPremiumInfoSticker                        => 'month_count',
    getPremiumLimit                              => 'limit_type',
    getPremiumState                              => '',
    getPremiumStickerExamples                    => '',
    getPremiumStickers                           => 'limit',
    getPreparedInlineMessage                     => 'bot_user_id prepared_message_id',
    getPreparedKeyboardButton                    => 'bot_user_id prepared_button_id',
    getProxies                                   => '',
    getPublicPostSearchLimits                    => 'query',
    getPushReceiverId                            => 'payload',
    getReadDatePrivacySettings                   => '',
    getReceivedGift                              => 'received_gift_id',
    getReceivedGifts                             => 'business_connection_id owner_id collection_id exclude_unsaved exclude_saved exclude_unlimited exclude_upgradable exclude_non_upgradable exclude_upgraded exclude_without_colors exclude_hosted sort_by_price offset limit',
    getRecentEmojiStatuses                       => '',
    getRecentInlineBots                          => '',
    getRecentStickers                            => 'is_attached',
    getRecentlyOpenedChats                       => 'limit',
    getRecentlyVisitedTMeUrls                    => 'referrer',
    getRecommendedChatFolders                    => '',
    getRecommendedChats                          => '',
    getRecoveryEmailAddress                      => 'password',
    getRemoteFile                                => 'remote_file_id file_type',
    getRepliedMessage                            => 'chat_id message_id',
    getSavedAnimations                           => '',
    getSavedMessagesTags                         => 'saved_messages_topic_id',
    getSavedMessagesTopicHistory                 => 'saved_messages_topic_id from_message_id offset limit',
    getSavedMessagesTopicMessageByDate           => 'saved_messages_topic_id date',
    getSavedNotificationSound                    => 'notification_sound_id',
    getSavedNotificationSounds                   => '',
    getSavedOrderInfo                            => '',
    getScopeNotificationSettings                 => 'scope',
    getSearchSponsoredChats                      => 'query',
    getSearchedForTags                           => 'tag_prefix limit',
    getSecretChat                                => 'secret_chat_id',
    getStakeDiceState                            => '',
    getStarAdAccountUrl                          => 'owner_id',
    getStarGiftPaymentOptions                    => 'user_id',
    getStarGiveawayPaymentOptions                => '',
    getStarPaymentOptions                        => '',
    getStarRevenueStatistics                     => 'owner_id is_dark',
    getStarSubscriptions                         => 'only_expiring offset',
    getStarTransactions                          => 'owner_id subscription_id direction offset limit',
    getStarWithdrawalUrl                         => 'owner_id star_count password',
    getStatisticalGraph                          => 'chat_id token x',
    getStickerEmojis                             => 'sticker',
    getStickerOutline                            => 'sticker_file_id for_animated_emoji for_clicked_animated_emoji_message',
    getStickerOutlineSvgPath                     => 'sticker_file_id for_animated_emoji for_clicked_animated_emoji_message',
    getStickerSet                                => 'set_id',
    getStickerSetName                            => 'set_id',
    getStickers                                  => 'sticker_type query limit chat_id',
    getStorageStatistics                         => 'chat_limit',
    getStorageStatisticsFast                     => '',
    getStory                                     => 'story_poster_chat_id story_id only_local',
    getStoryAlbumStories                         => 'chat_id story_album_id offset limit',
    getStoryAvailableReactions                   => 'row_size',
    getStoryInteractions                         => 'story_id query only_contacts prefer_forwards prefer_with_reaction offset limit',
    getStoryNotificationSettingsExceptions       => '',
    getStoryPublicForwards                       => 'story_poster_chat_id story_id offset limit',
    getStoryStatistics                           => 'chat_id story_id is_dark',
    getSuggestedFileName                         => 'file_id directory',
    getSuggestedStickerSetName                   => 'title',
    getSuitableDiscussionChats                   => '',
    getSuitablePersonalChats                     => '',
    getSupergroup                                => 'supergroup_id',
    getSupergroupFullInfo                        => 'supergroup_id',
    getSupergroupMembers                         => 'supergroup_id filter offset limit',
    getSupportName                               => '',
    getSupportUser                               => '',
    getTemporaryPasswordState                    => '',
    getTextCompositionStyleExample               => 'name example_number',
    getTextEntities                              => 'text',
    getThemeParametersJsonString                 => 'theme',
    getThemedChatEmojiStatuses                   => '',
    getThemedEmojiStatuses                       => '',
    getTimeZones                                 => '',
    getTonTransactions                           => 'direction offset limit',
    getTopChats                                  => 'category limit',
    getTrendingStickerSets                       => 'sticker_type offset limit',
    getUpgradedGift                              => 'name',
    getUpgradedGiftEmojiStatuses                 => '',
    getUpgradedGiftValueInfo                     => 'name',
    getUpgradedGiftVariants                      => 'regular_gift_id return_upgrade_models return_craft_models',
    getUpgradedGiftWithdrawalUrl                 => 'received_gift_id password',
    getUpgradedGiftsPromotionalAnimation         => '',
    getUser                                      => 'user_id',
    getUserChatBoosts                            => 'chat_id user_id',
    getUserFullInfo                              => 'user_id',
    getUserLink                                  => '',
    getUserPrivacySettingRules                   => 'setting',
    getUserProfileAudios                         => 'user_id offset limit',
    getUserProfilePhotos                         => 'user_id offset limit',
    getUserSupportInfo                           => 'user_id',
    getVideoChatAvailableParticipants            => 'chat_id',
    getVideoChatInviteLink                       => 'group_call_id can_self_unmute',
    getVideoChatRtmpUrl                          => 'chat_id',
    getVideoMessageAdvertisements                => 'chat_id message_id',
    getWebAppLinkUrl                             => 'chat_id bot_user_id web_app_short_name start_parameter allow_write_access parameters',
    getWebAppPlaceholder                         => 'bot_user_id',
    getWebAppUrl                                 => 'bot_user_id url parameters',
    getWebPageInstantView                        => 'url only_local',
    giftPremiumWithStars                         => 'user_id star_count month_count text',
    hideContactCloseBirthdays                    => '',
    hideSuggestedAction                          => 'action',
    importContacts                               => 'contacts',
    importMessages                               => 'chat_id message_file attached_files',
    increaseGiftAuctionBid                       => 'gift_id star_count',
    inviteGroupCallParticipant                   => 'group_call_id user_id is_video',
    inviteVideoChatParticipants                  => 'group_call_id user_ids',
    isLoginEmailAddressRequired                  => '',
    isProfileAudio                               => 'file_id',
    joinChat                                     => 'chat_id',
    joinChatByInviteLink                         => 'invite_link',
    joinGroupCall                                => 'input_group_call join_parameters',
    joinLiveStory                                => 'group_call_id join_parameters',
    joinVideoChat                                => 'group_call_id participant_id join_parameters invite_hash',
    launchPrepaidGiveaway                        => 'giveaway_id parameters winner_count star_count',
    leaveChat                                    => 'chat_id',
    leaveGroupCall                               => 'group_call_id',
    listenToAudio                                => 'audio_file_id duration',
    loadActiveStories                            => 'story_list',
    loadChats                                    => 'chat_list limit',
    loadDirectMessagesChatTopics                 => 'chat_id limit',
    loadGroupCallParticipants                    => 'group_call_id limit',
    loadQuickReplyShortcutMessages               => 'shortcut_id',
    loadQuickReplyShortcuts                      => '',
    loadSavedMessagesTopics                      => 'limit',
    logOut                                       => '',
    markChecklistTasksAsDone                     => 'chat_id message_id marked_as_done_task_ids marked_as_not_done_task_ids',
    openBotSimilarBot                            => 'bot_user_id opened_bot_user_id',
    openChat                                     => 'chat_id',
    openChatSimilarChat                          => 'chat_id opened_chat_id',
    openGiftAuction                              => 'gift_id',
    openMessageContent                           => 'chat_id message_id',
    openSponsoredChat                            => 'sponsored_chat_unique_id',
    openStory                                    => 'story_poster_chat_id story_id',
    openWebApp                                   => 'chat_id bot_user_id url topic_id reply_to parameters',
    optimizeStorage                              => 'size ttl count immunity_delay file_types chat_ids exclude_chat_ids return_deleted_file_statistics chat_limit',
    parseMarkdown                                => 'text',
    parseTextEntities                            => 'text parse_mode',
    pinChatMessage                               => 'chat_id message_id disable_notification only_for_self',
    pingProxy                                    => 'proxy',
    placeGiftAuctionBid                          => 'gift_id star_count user_id text is_private',
    postStory                                    => 'chat_id content areas caption privacy_settings album_ids active_period from_story_full_id is_posted_to_chat_page protect_content',
    preliminaryUploadFile                        => 'file file_type priority',
    processChatFolderNewChats                    => 'chat_folder_id added_chat_ids',
    processChatHasProtectedContentDisableRequest => 'chat_id request_message_id approve',
    processChatJoinRequest                       => 'chat_id user_id approve',
    processChatJoinRequests                      => 'chat_id invite_link approve',
    processGiftPurchaseOffer                     => 'message_id accept',
    processPushNotification                      => 'payload',
    rateSpeechRecognition                        => 'chat_id message_id is_good',
    readAllChatMentions                          => 'chat_id',
    readAllChatPollVotes                         => 'chat_id',
    readAllChatReactions                         => 'chat_id',
    readAllDirectMessagesChatTopicReactions      => 'chat_id topic_id',
    readAllForumTopicMentions                    => 'chat_id forum_topic_id',
    readAllForumTopicPollVotes                   => 'chat_id forum_topic_id',
    readAllForumTopicReactions                   => 'chat_id forum_topic_id',
    readBusinessMessage                          => 'business_connection_id chat_id message_id',
    readChatList                                 => 'chat_list',
    readFilePart                                 => 'file_id offset count',
    readdQuickReplyShortcutMessages              => 'shortcut_name message_ids',
    recognizeSpeech                              => 'chat_id message_id',
    recoverAuthenticationPassword                => 'recovery_code new_password new_hint',
    recoverPassword                              => 'recovery_code new_password new_hint',
    refundStarPayment                            => 'user_id telegram_payment_charge_id',
    registerDevice                               => 'device_token other_user_ids',
    registerUser                                 => 'first_name last_name disable_notification',
    removeAllFilesFromDownloads                  => 'only_active only_completed delete_from_cache',
    removeAllWebBrowserSettingsExceptions        => '',
    removeBusinessConnectedBotFromChat           => 'chat_id',
    removeChatActionBar                          => 'chat_id',
    removeContacts                               => 'user_ids',
    removeFavoriteSticker                        => 'sticker',
    removeFileFromDownloads                      => 'file_id delete_from_cache',
    removeGiftCollectionGifts                    => 'owner_id collection_id received_gift_ids',
    removeInstalledBackground                    => 'background_id',
    removeLoginPasskey                           => 'passkey_id',
    removeMessageReaction                        => 'chat_id message_id reaction_type',
    removeMessageSenderBotVerification           => 'bot_user_id verified_id',
    removeNotification                           => 'notification_group_id notification_id',
    removeNotificationGroup                      => 'notification_group_id max_notification_id',
    removePendingLiveStoryReactions              => 'group_call_id',
    removePendingPaidMessageReactions            => 'chat_id message_id',
    removeProfileAudio                           => 'file_id',
    removeProxy                                  => 'proxy_id',
    removeRecentHashtag                          => 'hashtag',
    removeRecentSticker                          => 'is_attached sticker',
    removeRecentlyFoundChat                      => 'chat_id',
    removeSavedAnimation                         => 'animation',
    removeSavedNotificationSound                 => 'notification_sound_id',
    removeSearchedForTag                         => 'tag',
    removeStickerFromSet                         => 'sticker',
    removeStoryAlbumStories                      => 'chat_id story_album_id story_ids',
    removeTextCompositionStyle                   => 'name',
    removeTopChat                                => 'category chat_id',
    removeWebBrowserSettingsException            => 'url',
    reorderActiveUsernames                       => 'usernames',
    reorderBotActiveUsernames                    => 'bot_user_id usernames',
    reorderBotMediaPreviews                      => 'bot_user_id language_code file_ids',
    reorderChatFolders                           => 'chat_folder_ids main_chat_list_position',
    reorderGiftCollectionGifts                   => 'owner_id collection_id received_gift_ids',
    reorderGiftCollections                       => 'owner_id collection_ids',
    reorderInstalledStickerSets                  => 'sticker_type sticker_set_ids',
    reorderQuickReplyShortcuts                   => 'shortcut_ids',
    reorderStoryAlbumStories                     => 'chat_id story_album_id story_ids',
    reorderStoryAlbums                           => 'chat_id story_album_ids',
    reorderSupergroupActiveUsernames             => 'supergroup_id usernames',
    replaceLiveStoryRtmpUrl                      => 'chat_id',
    replacePrimaryChatInviteLink                 => 'chat_id',
    replaceStickerInSet                          => 'user_id name old_sticker new_sticker',
    replaceVideoChatRtmpUrl                      => 'chat_id',
    reportAuthenticationCodeMissing              => 'mobile_network_code',
    reportChat                                   => 'chat_id option_id message_ids text',
    reportChatPhoto                              => 'chat_id file_id reason text',
    reportChatSponsoredMessage                   => 'chat_id message_id option_id',
    reportMessageReactions                       => 'chat_id message_id sender_id',
    reportPhoneNumberCodeMissing                 => 'mobile_network_code',
    reportSponsoredChat                          => 'sponsored_chat_unique_id option_id',
    reportStory                                  => 'story_poster_chat_id story_id option_id text',
    reportSupergroupAntiSpamFalsePositive        => 'supergroup_id message_id',
    reportSupergroupSpam                         => 'supergroup_id message_ids',
    reportVideoMessageAdvertisement              => 'advertisement_unique_id option_id',
    requestAuthenticationPasswordRecovery        => '',
    requestPasswordRecovery                      => '',
    requestQrCodeAuthentication                  => 'other_user_ids',
    resendAuthenticationCode                     => 'reason',
    resendEmailAddressVerificationCode           => '',
    resendLoginEmailAddressCode                  => '',
    resendMessages                               => 'chat_id message_ids quote paid_message_star_count',
    resendPhoneNumberCode                        => 'reason',
    resendRecoveryEmailAddressCode               => '',
    resetAllNotificationSettings                 => '',
    resetAuthenticationEmailAddress              => '',
    resetInstalledBackgrounds                    => '',
    resetNetworkStatistics                       => '',
    resetPassword                                => '',
    reuseStarSubscription                        => 'subscription_id',
    revokeChatInviteLink                         => 'chat_id invite_link',
    revokeGroupCallInviteLink                    => 'group_call_id',
    saveApplicationLogEvent                      => 'type chat_id data',
    savePreparedInlineMessage                    => 'user_id result chat_types',
    savePreparedKeyboardButton                   => 'user_id button',
    searchAffiliatePrograms                      => 'affiliate sort_order offset limit',
    searchBackground                             => 'name',
    searchCallMessages                           => 'offset limit only_missed',
    searchChatAffiliateProgram                   => 'username referrer',
    searchChatMembers                            => 'chat_id query limit filter',
    searchChatMessages                           => 'chat_id topic_id query sender_id from_message_id offset limit filter',
    searchChatRecentLocationMessages             => 'chat_id limit',
    searchChats                                  => 'query type_filter limit',
    searchChatsOnServer                          => 'query type_filter limit',
    searchContacts                               => 'query limit',
    searchEmojis                                 => 'text input_language_codes',
    searchFileDownloads                          => 'query only_active only_completed offset limit',
    searchGiftsForResale                         => 'gift_id order for_crafting for_stars attributes offset limit',
    searchHashtags                               => 'prefix limit',
    searchInstalledStickerSets                   => 'sticker_type query limit',
    searchMessages                               => 'chat_list query offset limit filter chat_type_filter min_date max_date',
    searchOutgoingDocumentMessages               => 'query limit',
    searchPublicChat                             => 'username',
    searchPublicChats                            => 'query type_filter',
    searchPublicMessagesByTag                    => 'tag offset limit',
    searchPublicPosts                            => 'query offset limit star_count',
    searchPublicStoriesByLocation                => 'address offset limit',
    searchPublicStoriesByTag                     => 'story_poster_chat_id tag offset limit',
    searchPublicStoriesByVenue                   => 'venue_provider venue_id offset limit',
    searchQuote                                  => 'text quote quote_position',
    searchRecentlyFoundChats                     => 'query type_filter limit',
    searchSavedMessages                          => 'saved_messages_topic_id tag query from_message_id offset limit',
    searchSecretMessages                         => 'chat_id query offset limit filter',
    searchStickerSet                             => 'name ignore_cache',
    searchStickerSets                            => 'sticker_type query',
    searchStickers                               => 'sticker_type emojis query input_language_codes offset limit',
    searchStringsByPrefix                        => 'strings query limit return_none_for_empty_query',
    searchTextCompositionStyle                   => 'name',
    searchUserByPhoneNumber                      => 'phone_number only_local',
    searchUserByToken                            => 'token',
    searchWebApp                                 => 'bot_user_id web_app_short_name',
    sellGift                                     => 'business_connection_id received_gift_id',
    sendAuthenticationFirebaseSms                => 'token',
    sendBotStartMessage                          => 'bot_user_id chat_id parameter',
    sendBusinessMessage                          => 'business_connection_id chat_id reply_to disable_notification protect_content effect_id reply_markup input_message_content',
    sendBusinessMessageAlbum                     => 'business_connection_id chat_id reply_to disable_notification protect_content effect_id input_message_contents',
    sendCallDebugInformation                     => 'call_id debug_information',
    sendCallLog                                  => 'call_id log_file',
    sendCallRating                               => 'call_id rating comment problems',
    sendCallSignalingData                        => 'call_id data',
    sendChatAction                               => 'chat_id topic_id business_connection_id action',
    sendCustomRequest                            => 'method parameters',
    sendEmailAddressVerificationCode             => 'email_address',
    sendEphemeralMessage                         => 'chat_id topic_id receiver_user_id callback_query_id reply_to sending_id only_preview reply_markup input_message_content',
    sendGift                                     => 'gift_id owner_id text is_private pay_for_upgrade',
    sendGiftPurchaseOffer                        => 'owner_id gift_name price duration paid_message_star_count',
    sendGroupCallMessage                         => 'group_call_id text paid_message_star_count',
    sendInlineQueryResultMessage                 => 'chat_id topic_id reply_to options query_id result_id hide_via_bot',
    sendMessage                                  => 'chat_id topic_id reply_to options reply_markup input_message_content',
    sendMessageAlbum                             => 'chat_id topic_id reply_to options input_message_contents',
    sendMessageViewMetrics                       => 'chat_id message_id time_in_view_ms active_time_in_view_ms height_to_viewport_ratio_per_mille seen_range_ratio_per_mille',
    sendPassportAuthorizationForm                => 'authorization_form_id types',
    sendPaymentForm                              => 'input_invoice payment_form_id order_info_id shipping_option_id credentials tip_amount',
    sendPhoneNumberCode                          => 'phone_number settings type',
    sendPhoneNumberFirebaseSms                   => 'token',
    sendQuickReplyShortcutMessages               => 'chat_id shortcut_id sending_id',
    sendResoldGift                               => 'gift_name owner_id price',
    sendRichMessageDraft                         => 'chat_id forum_topic_id draft_id message',
    sendTextMessageDraft                         => 'chat_id forum_topic_id draft_id text',
    sendWebAppCustomRequest                      => 'bot_user_id method parameters',
    sendWebAppData                               => 'bot_user_id button_text data',
    setAccentColor                               => 'accent_color_id background_custom_emoji_id',
    setAccountTtl                                => 'ttl',
    setAlarm                                     => 'seconds',
    setApplicationVerificationToken              => 'verification_id token',
    setArchiveChatListSettings                   => 'settings',
    setAuthenticationEmailAddress                => 'email_address',
    setAuthenticationPhoneNumber                 => 'phone_number settings',
    setAuthenticationPremiumPurchaseTransaction  => 'transaction is_restore premium_day_count currency amount',
    setAutoDownloadSettings                      => 'settings type',
    setAutosaveSettings                          => 'scope settings',
    setBio                                       => 'bio',
    setBirthdate                                 => 'birthdate',
    setBotInfoDescription                        => 'bot_user_id language_code description',
    setBotInfoShortDescription                   => 'bot_user_id language_code short_description',
    setBotName                                   => 'bot_user_id language_code name',
    setBotProfilePhoto                           => 'bot_user_id photo',
    setBotUpdatesStatus                          => 'pending_update_count error_message',
    setBusinessAccountBio                        => 'business_connection_id bio',
    setBusinessAccountGiftSettings               => 'business_connection_id settings',
    setBusinessAccountName                       => 'business_connection_id first_name last_name',
    setBusinessAccountProfilePhoto               => 'business_connection_id photo is_public',
    setBusinessAccountUsername                   => 'business_connection_id username',
    setBusinessAwayMessageSettings               => 'away_message_settings',
    setBusinessConnectedBot                      => 'bot',
    setBusinessGreetingMessageSettings           => 'greeting_message_settings',
    setBusinessLocation                          => 'location',
    setBusinessMessageIsPinned                   => 'business_connection_id chat_id message_id is_pinned',
    setBusinessOpeningHours                      => 'opening_hours',
    setBusinessStartPage                         => 'start_page',
    setChatAccentColor                           => 'chat_id accent_color_id background_custom_emoji_id',
    setChatActiveStoriesList                     => 'chat_id story_list',
    setChatAffiliateProgram                      => 'chat_id parameters',
    setChatAvailableReactions                    => 'chat_id available_reactions',
    setChatBackground                            => 'chat_id background type dark_theme_dimming only_for_self',
    setChatClientData                            => 'chat_id client_data',
    setChatDescription                           => 'chat_id description',
    setChatDirectMessagesGroup                   => 'chat_id is_enabled paid_message_star_count',
    setChatDiscussionGroup                       => 'chat_id discussion_chat_id',
    setChatDraftMessage                          => 'chat_id topic_id draft_message',
    setChatEmojiStatus                           => 'chat_id emoji_status',
    setChatLocation                              => 'chat_id location',
    setChatMemberStatus                          => 'chat_id member_id status',
    setChatMemberTag                             => 'chat_id user_id tag',
    setChatMessageAutoDeleteTime                 => 'chat_id message_auto_delete_time',
    setChatMessageSender                         => 'chat_id message_sender_id',
    setChatNotificationSettings                  => 'chat_id notification_settings',
    setChatPaidMessageStarCount                  => 'chat_id paid_message_star_count',
    setChatPermissions                           => 'chat_id permissions',
    setChatPhoto                                 => 'chat_id photo',
    setChatPinnedStories                         => 'chat_id story_ids',
    setChatProfileAccentColor                    => 'chat_id profile_accent_color_id profile_background_custom_emoji_id',
    setChatSlowModeDelay                         => 'chat_id slow_mode_delay',
    setChatTheme                                 => 'chat_id theme',
    setChatTitle                                 => 'chat_id title',
    setCloseFriends                              => 'user_ids',
    setCommands                                  => 'scope language_code commands',
    setCustomEmojiStickerSetThumbnail            => 'name custom_emoji_id',
    setCustomLanguagePack                        => 'info strings',
    setCustomLanguagePackString                  => 'language_pack_id new_string',
    setDatabaseEncryptionKey                     => 'new_encryption_key',
    setDefaultBackground                         => 'background type for_dark_theme',
    setDefaultChannelAdministratorRights         => 'default_channel_administrator_rights',
    setDefaultGroupAdministratorRights           => 'default_group_administrator_rights',
    setDefaultMessageAutoDeleteTime              => 'message_auto_delete_time',
    setDefaultReactionType                       => 'reaction_type',
    setDirectMessagesChatTopicIsMarkedAsUnread   => 'chat_id topic_id is_marked_as_unread',
    setEmojiStatus                               => 'emoji_status',
    setFileGenerationProgress                    => 'generation_id expected_size local_prefix_size',
    setForumTopicNotificationSettings            => 'chat_id forum_topic_id notification_settings',
    setGameScore                                 => 'chat_id message_id edit_message user_id score force',
    setGiftCollectionName                        => 'owner_id collection_id name',
    setGiftResalePrice                           => 'received_gift_id price',
    setGiftSettings                              => 'settings',
    setGroupCallPaidMessageStarCount             => 'group_call_id paid_message_star_count',
    setGroupCallParticipantIsSpeaking            => 'group_call_id audio_source is_speaking',
    setGroupCallParticipantVolumeLevel           => 'group_call_id participant_id volume_level',
    setInactiveSessionTtl                        => 'inactive_session_ttl_days',
    setInlineGameScore                           => 'inline_message_id edit_message user_id score force',
    setLiveStoryMessageSender                    => 'group_call_id message_sender_id',
    setLogStream                                 => 'log_stream',
    setLogTagVerbosityLevel                      => 'tag new_verbosity_level',
    setLogVerbosityLevel                         => 'new_verbosity_level',
    setLoginEmailAddress                         => 'new_login_email_address',
    setMainProfileTab                            => 'main_profile_tab',
    setManagedBotAccessSettings                  => 'bot_user_id settings',
    setMenuButton                                => 'user_id menu_button',
    setMessageFactCheck                          => 'chat_id message_id text',
    setMessageReactions                          => 'chat_id message_id reaction_types is_big',
    setMessageSenderBlockList                    => 'sender_id block_list',
    setMessageSenderBotVerification              => 'bot_user_id verified_id custom_description',
    setName                                      => 'first_name last_name',
    setNetworkType                               => 'type',
    setNewChatPrivacySettings                    => 'settings',
    setOption                                    => 'name value',
    setPaidMessageReactionType                   => 'chat_id message_id type',
    setPassportElement                           => 'element password',
    setPassportElementErrors                     => 'user_id errors',
    setPassword                                  => 'old_password new_password new_hint set_recovery_email_address new_recovery_email_address',
    setPersonalChat                              => 'chat_id',
    setPinnedChats                               => 'chat_list chat_ids',
    setPinnedForumTopics                         => 'chat_id forum_topic_ids',
    setPinnedGifts                               => 'owner_id received_gift_ids',
    setPinnedSavedMessagesTopics                 => 'saved_messages_topic_ids',
    setPollAnswer                                => 'chat_id message_id option_ids',
    setProfileAccentColor                        => 'profile_accent_color_id profile_background_custom_emoji_id',
    setProfileAudioPosition                      => 'file_id after_file_id',
    setProfilePhoto                              => 'photo is_public',
    setQuickReplyShortcutName                    => 'shortcut_id name',
    setReactionNotificationSettings              => 'notification_settings',
    setReadDatePrivacySettings                   => 'settings',
    setRecoveryEmailAddress                      => 'password new_recovery_email_address',
    setSavedMessagesTagLabel                     => 'tag label',
    setScopeNotificationSettings                 => 'scope notification_settings',
    setStickerEmojis                             => 'sticker emojis',
    setStickerKeywords                           => 'sticker keywords',
    setStickerMaskPosition                       => 'sticker mask_position',
    setStickerPositionInSet                      => 'sticker position',
    setStickerSetThumbnail                       => 'user_id name thumbnail format',
    setStickerSetTitle                           => 'name title',
    setStoryAlbumName                            => 'chat_id story_album_id name',
    setStoryPrivacySettings                      => 'story_id privacy_settings',
    setStoryReaction                             => 'story_poster_chat_id story_id reaction_type update_recent_reactions',
    setSupergroupCustomEmojiStickerSet           => 'supergroup_id custom_emoji_sticker_set_id',
    setSupergroupMainProfileTab                  => 'supergroup_id main_profile_tab',
    setSupergroupStickerSet                      => 'supergroup_id sticker_set_id',
    setSupergroupUnrestrictBoostCount            => 'supergroup_id unrestrict_boost_count',
    setSupergroupUsername                        => 'supergroup_id username',
    setTdlibParameters                           => 'use_test_dc database_directory files_directory database_encryption_key use_file_database use_chat_info_database use_message_database use_secret_chats api_id api_hash system_language_code device_model system_version application_version',
    setUpgradedGiftColors                        => 'upgraded_gift_colors_id',
    setUserEmojiStatus                           => 'user_id emoji_status',
    setUserNote                                  => 'user_id note',
    setUserPersonalProfilePhoto                  => 'user_id photo',
    setUserPrivacySettingRules                   => 'setting rules',
    setUserSupportInfo                           => 'user_id message',
    setUsername                                  => 'username',
    setVideoChatDefaultParticipant               => 'chat_id default_participant_id',
    setVideoChatTitle                            => 'group_call_id title',
    shareChatWithBot                             => 'source button_id shared_chat_id only_check',
    sharePhoneNumber                             => 'user_id',
    shareUsersWithBot                            => 'source button_id shared_user_ids only_check',
    startGroupCallRecording                      => 'group_call_id title record_video use_portrait_orientation',
    startGroupCallScreenSharing                  => 'group_call_id audio_source_id payload',
    startLiveStory                               => 'chat_id privacy_settings protect_content is_rtmp_stream enable_messages paid_message_star_count',
    startScheduledVideoChat                      => 'group_call_id',
    stopBusinessPoll                             => 'business_connection_id chat_id message_id reply_markup',
    stopPoll                                     => 'chat_id message_id reply_markup',
    suggestUserBirthdate                         => 'user_id birthdate',
    suggestUserProfilePhoto                      => 'user_id photo',
    summarizeMessage                             => 'chat_id message_id translate_to_language_code tone',
    synchronizeLanguagePack                      => 'language_pack_id',
    terminateAllOtherSessions                    => '',
    terminateSession                             => 'session_id',
    testCallBytes                                => 'x',
    testCallEmpty                                => '',
    testCallString                               => 'x',
    testCallVectorInt                            => 'x',
    testCallVectorIntObject                      => 'x',
    testCallVectorString                         => 'x',
    testCallVectorStringObject                   => 'x',
    testGetDifference                            => '',
    testNetwork                                  => '',
    testProxy                                    => 'proxy dc_id timeout',
    testReturnError                              => 'error',
    testSquareInt                                => 'x',
    testUseUpdate                                => '',
    toggleAllDownloadsArePaused                  => 'are_paused',
    toggleBotCanManageEmojiStatus                => 'bot_user_id can_manage_emoji_status',
    toggleBotIsAddedToAttachmentMenu             => 'bot_user_id is_added allow_write_access',
    toggleBotUsernameIsActive                    => 'bot_user_id username is_active',
    toggleBusinessConnectedBotChatIsPaused       => 'chat_id is_paused',
    toggleChatDefaultDisableNotification         => 'chat_id default_disable_notification',
    toggleChatFolderTags                         => 'are_tags_enabled',
    toggleChatGiftNotifications                  => 'chat_id are_enabled',
    toggleChatHasProtectedContent                => 'chat_id has_protected_content',
    toggleChatIsMarkedAsUnread                   => 'chat_id is_marked_as_unread',
    toggleChatIsPinned                           => 'chat_list chat_id is_pinned',
    toggleChatIsTranslatable                     => 'chat_id is_translatable',
    toggleChatViewAsTopics                       => 'chat_id view_as_topics',
    toggleDirectMessagesChatTopicCanSendUnpaidMessages => 'chat_id topic_id can_send_unpaid_messages refund_payments',
    toggleDownloadIsPaused                       => 'file_id is_paused',
    toggleForumTopicIsClosed                     => 'chat_id forum_topic_id is_closed',
    toggleForumTopicIsPinned                     => 'chat_id forum_topic_id is_pinned',
    toggleGeneralForumTopicIsHidden              => 'chat_id is_hidden',
    toggleGiftIsSaved                            => 'received_gift_id is_saved',
    toggleGroupCallAreMessagesAllowed            => 'group_call_id are_messages_allowed',
    toggleGroupCallIsMyVideoEnabled              => 'group_call_id is_my_video_enabled',
    toggleGroupCallIsMyVideoPaused               => 'group_call_id is_my_video_paused',
    toggleGroupCallParticipantIsHandRaised       => 'group_call_id participant_id is_hand_raised',
    toggleGroupCallParticipantIsMuted            => 'group_call_id participant_id is_muted',
    toggleGroupCallScreenSharingIsPaused         => 'group_call_id is_paused',
    toggleHasSponsoredMessagesEnabled            => 'has_sponsored_messages_enabled',
    toggleSavedMessagesTopicIsPinned             => 'saved_messages_topic_id is_pinned',
    toggleSessionCanAcceptCalls                  => 'session_id can_accept_calls',
    toggleSessionCanAcceptSecretChats            => 'session_id can_accept_secret_chats',
    toggleStoryIsPostedToChatPage                => 'story_poster_chat_id story_id is_posted_to_chat_page',
    toggleSupergroupCanHaveSponsoredMessages     => 'supergroup_id can_have_sponsored_messages',
    toggleSupergroupHasAggressiveAntiSpamEnabled => 'supergroup_id has_aggressive_anti_spam_enabled',
    toggleSupergroupHasAutomaticTranslation      => 'supergroup_id has_automatic_translation',
    toggleSupergroupHasHiddenMembers             => 'supergroup_id has_hidden_members',
    toggleSupergroupIsAllHistoryAvailable        => 'supergroup_id is_all_history_available',
    toggleSupergroupIsBroadcastGroup             => 'supergroup_id',
    toggleSupergroupIsForum                      => 'supergroup_id is_forum has_forum_tabs',
    toggleSupergroupJoinByRequest                => 'supergroup_id join_by_request guard_bot_user_id apply_to_invite_links',
    toggleSupergroupJoinToSendMessages           => 'supergroup_id join_to_send_messages',
    toggleSupergroupSignMessages                 => 'supergroup_id sign_messages show_message_sender',
    toggleSupergroupUsernameIsActive             => 'supergroup_id username is_active',
    toggleUsernameIsActive                       => 'username is_active',
    toggleVideoChatEnabledStartNotification      => 'group_call_id enabled_start_notification',
    toggleVideoChatMuteNewParticipants           => 'group_call_id mute_new_participants',
    transferBusinessAccountStars                 => 'business_connection_id star_count',
    transferChatOwnership                        => 'chat_id user_id password',
    transferGift                                 => 'business_connection_id received_gift_id new_owner_id star_count',
    translateMessageRichMessage                  => 'chat_id message_id to_language_code tone',
    translateMessageText                         => 'chat_id message_id to_language_code tone',
    translateRichMessage                         => 'message to_language_code tone',
    translateText                                => 'text to_language_code tone',
    unpinAllChatMessages                         => 'chat_id',
    unpinAllDirectMessagesChatTopicMessages      => 'chat_id topic_id',
    unpinAllForumTopicMessages                   => 'chat_id forum_topic_id',
    unpinChatMessage                             => 'chat_id message_id',
    upgradeBasicGroupChatToSupergroupChat        => 'chat_id',
    upgradeGift                                  => 'business_connection_id received_gift_id keep_original_details star_count',
    uploadStickerFile                            => 'user_id sticker_format sticker',
    validateOrderInfo                            => 'input_invoice order_info allow_save',
    viewMessages                                 => 'chat_id message_ids source force_read',
    viewPremiumFeature                           => 'feature',
    viewSponsoredChat                            => 'sponsored_chat_unique_id',
    viewTrendingStickerSets                      => 'sticker_set_ids',
    viewVideoMessageAdvertisement                => 'advertisement_unique_id',
    writeGeneratedFilePart                       => 'generation_id offset data',
);

our %CLASS_BASE = (
    accentColor                                    => 'Object',
    acceptCall                                     => 'Function',
    acceptOauthRequest                             => 'Function',
    acceptTermsOfService                           => 'Function',
    acceptedGiftTypes                              => 'Object',
    accountInfo                                    => 'Object',
    accountTtl                                     => 'Object',
    activateStoryStealthMode                       => 'Function',
    activeStoryStateLive                           => 'ActiveStoryState',
    activeStoryStateRead                           => 'ActiveStoryState',
    activeStoryStateUnread                         => 'ActiveStoryState',
    addBotMediaPreview                             => 'Function',
    addChatFolderByInviteLink                      => 'Function',
    addChatMember                                  => 'Function',
    addChatMembers                                 => 'Function',
    addChatToList                                  => 'Function',
    addChecklistTasks                              => 'Function',
    addContact                                     => 'Function',
    addCustomServerLanguagePack                    => 'Function',
    addFavoriteSticker                             => 'Function',
    addFileToDownloads                             => 'Function',
    addGiftCollectionGifts                         => 'Function',
    addLocalMessage                                => 'Function',
    addLogMessage                                  => 'Function',
    addLoginPasskey                                => 'Function',
    addMessageReaction                             => 'Function',
    addNetworkStatistics                           => 'Function',
    addOffer                                       => 'Function',
    addPendingLiveStoryReaction                    => 'Function',
    addPendingPaidMessageReaction                  => 'Function',
    addPollOption                                  => 'Function',
    addProfileAudio                                => 'Function',
    addProxy                                       => 'Function',
    addQuickReplyShortcutInlineQueryResultMessage  => 'Function',
    addQuickReplyShortcutMessage                   => 'Function',
    addQuickReplyShortcutMessageAlbum              => 'Function',
    addRecentSticker                               => 'Function',
    addRecentlyFoundChat                           => 'Function',
    addSavedAnimation                              => 'Function',
    addSavedNotificationSound                      => 'Function',
    addStickerToSet                                => 'Function',
    addStoryAlbumStories                           => 'Function',
    addTextCompositionStyle                        => 'Function',
    addWebBrowserSettingsException                 => 'Function',
    addedProxies                                   => 'Object',
    addedProxy                                     => 'Object',
    addedReaction                                  => 'Object',
    addedReactions                                 => 'Object',
    address                                        => 'Object',
    advertisementSponsor                           => 'Object',
    affiliateInfo                                  => 'Object',
    affiliateProgramInfo                           => 'Object',
    affiliateProgramParameters                     => 'Object',
    affiliateProgramSortOrderCreationDate          => 'AffiliateProgramSortOrder',
    affiliateProgramSortOrderProfitability         => 'AffiliateProgramSortOrder',
    affiliateProgramSortOrderRevenue               => 'AffiliateProgramSortOrder',
    affiliateTypeBot                               => 'AffiliateType',
    affiliateTypeChannel                           => 'AffiliateType',
    affiliateTypeCurrentUser                       => 'AffiliateType',
    ageVerificationParameters                      => 'Object',
    allowBotToSendMessages                         => 'Function',
    allowUnpaidMessagesFromUser                    => 'Function',
    alternativeVideo                               => 'Object',
    animatedChatPhoto                              => 'Object',
    animatedEmoji                                  => 'Object',
    animation                                      => 'Object',
    animations                                     => 'Object',
    answerCallbackQuery                            => 'Function',
    answerChatJoinRequestQuery                     => 'Function',
    answerCustomQuery                              => 'Function',
    answerGuestQuery                               => 'Function',
    answerInlineQuery                              => 'Function',
    answerPreCheckoutQuery                         => 'Function',
    answerShippingQuery                            => 'Function',
    answerWebAppQuery                              => 'Function',
    applyPremiumGiftCode                           => 'Function',
    approveSuggestedPost                           => 'Function',
    archiveChatListSettings                        => 'Object',
    assignStoreTransaction                         => 'Function',
    attachmentMenuBot                              => 'Object',
    attachmentMenuBotColor                         => 'Object',
    attributeCraftPersistenceProbability           => 'Object',
    auctionBid                                     => 'Object',
    auctionRound                                   => 'Object',
    auctionStateActive                             => 'AuctionState',
    auctionStateFinished                           => 'AuctionState',
    audio                                          => 'Object',
    audios                                         => 'Object',
    authenticationCodeInfo                         => 'Object',
    authenticationCodeTypeCall                     => 'AuthenticationCodeType',
    authenticationCodeTypeFirebaseAndroid          => 'AuthenticationCodeType',
    authenticationCodeTypeFirebaseIos              => 'AuthenticationCodeType',
    authenticationCodeTypeFlashCall                => 'AuthenticationCodeType',
    authenticationCodeTypeFragment                 => 'AuthenticationCodeType',
    authenticationCodeTypeMissedCall               => 'AuthenticationCodeType',
    authenticationCodeTypeSms                      => 'AuthenticationCodeType',
    authenticationCodeTypeSmsPhrase                => 'AuthenticationCodeType',
    authenticationCodeTypeSmsWord                  => 'AuthenticationCodeType',
    authenticationCodeTypeTelegramMessage          => 'AuthenticationCodeType',
    authorizationStateClosed                       => 'AuthorizationState',
    authorizationStateClosing                      => 'AuthorizationState',
    authorizationStateLoggingOut                   => 'AuthorizationState',
    authorizationStateReady                        => 'AuthorizationState',
    authorizationStateWaitCode                     => 'AuthorizationState',
    authorizationStateWaitEmailAddress             => 'AuthorizationState',
    authorizationStateWaitEmailCode                => 'AuthorizationState',
    authorizationStateWaitOtherDeviceConfirmation  => 'AuthorizationState',
    authorizationStateWaitPassword                 => 'AuthorizationState',
    authorizationStateWaitPhoneNumber              => 'AuthorizationState',
    authorizationStateWaitPremiumPurchase          => 'AuthorizationState',
    authorizationStateWaitRegistration             => 'AuthorizationState',
    authorizationStateWaitTdlibParameters          => 'AuthorizationState',
    autoDownloadSettings                           => 'Object',
    autoDownloadSettingsPresets                    => 'Object',
    autosaveSettings                               => 'Object',
    autosaveSettingsException                      => 'Object',
    autosaveSettingsScopeChannelChats              => 'AutosaveSettingsScope',
    autosaveSettingsScopeChat                      => 'AutosaveSettingsScope',
    autosaveSettingsScopeGroupChats                => 'AutosaveSettingsScope',
    autosaveSettingsScopePrivateChats              => 'AutosaveSettingsScope',
    availableGift                                  => 'Object',
    availableGifts                                 => 'Object',
    availableReaction                              => 'Object',
    availableReactions                             => 'Object',
    background                                     => 'Object',
    backgroundFillFreeformGradient                 => 'BackgroundFill',
    backgroundFillGradient                         => 'BackgroundFill',
    backgroundFillSolid                            => 'BackgroundFill',
    backgroundTypeChatTheme                        => 'BackgroundType',
    backgroundTypeFill                             => 'BackgroundType',
    backgroundTypePattern                          => 'BackgroundType',
    backgroundTypeWallpaper                        => 'BackgroundType',
    backgrounds                                    => 'Object',
    banChatMember                                  => 'Function',
    banGroupCallParticipants                       => 'Function',
    bankCardActionOpenUrl                          => 'Object',
    bankCardInfo                                   => 'Object',
    basicGroup                                     => 'Object',
    basicGroupFullInfo                             => 'Object',
    birthdate                                      => 'Object',
    blockListMain                                  => 'BlockList',
    blockListStories                               => 'BlockList',
    blockMessageSenderFromReplies                  => 'Function',
    boostChat                                      => 'Function',
    botAccessSettings                              => 'Object',
    botCommand                                     => 'Object',
    botCommandScopeAllChatAdministrators           => 'BotCommandScope',
    botCommandScopeAllGroupChats                   => 'BotCommandScope',
    botCommandScopeAllPrivateChats                 => 'BotCommandScope',
    botCommandScopeChat                            => 'BotCommandScope',
    botCommandScopeChatAdministrators              => 'BotCommandScope',
    botCommandScopeChatMember                      => 'BotCommandScope',
    botCommandScopeDefault                         => 'BotCommandScope',
    botCommands                                    => 'Object',
    botInfo                                        => 'Object',
    botMediaPreview                                => 'Object',
    botMediaPreviewInfo                            => 'Object',
    botMediaPreviews                               => 'Object',
    botMenuButton                                  => 'Object',
    botVerification                                => 'Object',
    botVerificationParameters                      => 'Object',
    botWriteAccessAllowReasonAcceptedRequest       => 'BotWriteAccessAllowReason',
    botWriteAccessAllowReasonAddedToAttachmentMenu => 'BotWriteAccessAllowReason',
    botWriteAccessAllowReasonConnectedWebsite      => 'BotWriteAccessAllowReason',
    botWriteAccessAllowReasonLaunchedWebApp        => 'BotWriteAccessAllowReason',
    builtInThemeArctic                             => 'BuiltInTheme',
    builtInThemeClassic                            => 'BuiltInTheme',
    builtInThemeDay                                => 'BuiltInTheme',
    builtInThemeNight                              => 'BuiltInTheme',
    builtInThemeTinted                             => 'BuiltInTheme',
    businessAwayMessageScheduleAlways              => 'BusinessAwayMessageSchedule',
    businessAwayMessageScheduleCustom              => 'BusinessAwayMessageSchedule',
    businessAwayMessageScheduleOutsideOfOpeningHours => 'BusinessAwayMessageSchedule',
    businessAwayMessageSettings                    => 'Object',
    businessBotManageBar                           => 'Object',
    businessBotRights                              => 'Object',
    businessChatLink                               => 'Object',
    businessChatLinkInfo                           => 'Object',
    businessChatLinks                              => 'Object',
    businessConnectedBot                           => 'Object',
    businessConnectedBotInfo                       => 'Object',
    businessConnection                             => 'Object',
    businessFeatureAccountLinks                    => 'BusinessFeature',
    businessFeatureAwayMessage                     => 'BusinessFeature',
    businessFeatureBots                            => 'BusinessFeature',
    businessFeatureChatFolderTags                  => 'BusinessFeature',
    businessFeatureEmojiStatus                     => 'BusinessFeature',
    businessFeatureGreetingMessage                 => 'BusinessFeature',
    businessFeatureLocation                        => 'BusinessFeature',
    businessFeatureOpeningHours                    => 'BusinessFeature',
    businessFeaturePromotionAnimation              => 'Object',
    businessFeatureQuickReplies                    => 'BusinessFeature',
    businessFeatureStartPage                       => 'BusinessFeature',
    businessFeatureUpgradedStories                 => 'BusinessFeature',
    businessFeatures                               => 'Object',
    businessGreetingMessageSettings                => 'Object',
    businessInfo                                   => 'Object',
    businessLocation                               => 'Object',
    businessMessage                                => 'Object',
    businessMessages                               => 'Object',
    businessOpeningHours                           => 'Object',
    businessOpeningHoursInterval                   => 'Object',
    businessRecipients                             => 'Object',
    businessStartPage                              => 'Object',
    buttonStyleDanger                              => 'ButtonStyle',
    buttonStyleDefault                             => 'ButtonStyle',
    buttonStylePrimary                             => 'ButtonStyle',
    buttonStyleSuccess                             => 'ButtonStyle',
    buyGiftUpgrade                                 => 'Function',
    call                                           => 'Object',
    callDiscardReasonDeclined                      => 'CallDiscardReason',
    callDiscardReasonDisconnected                  => 'CallDiscardReason',
    callDiscardReasonEmpty                         => 'CallDiscardReason',
    callDiscardReasonHungUp                        => 'CallDiscardReason',
    callDiscardReasonMissed                        => 'CallDiscardReason',
    callDiscardReasonUpgradeToGroupCall            => 'CallDiscardReason',
    callId                                         => 'Object',
    callProblemDistortedSpeech                     => 'CallProblem',
    callProblemDistortedVideo                      => 'CallProblem',
    callProblemDropped                             => 'CallProblem',
    callProblemEcho                                => 'CallProblem',
    callProblemInterruptions                       => 'CallProblem',
    callProblemNoise                               => 'CallProblem',
    callProblemPixelatedVideo                      => 'CallProblem',
    callProblemSilentLocal                         => 'CallProblem',
    callProblemSilentRemote                        => 'CallProblem',
    callProtocol                                   => 'Object',
    callServer                                     => 'Object',
    callServerTypeTelegramReflector                => 'CallServerType',
    callServerTypeWebrtc                           => 'CallServerType',
    callStateDiscarded                             => 'CallState',
    callStateError                                 => 'CallState',
    callStateExchangingKeys                        => 'CallState',
    callStateHangingUp                             => 'CallState',
    callStatePending                               => 'CallState',
    callStateReady                                 => 'CallState',
    callbackQueryAnswer                            => 'Object',
    callbackQueryPayloadData                       => 'CallbackQueryPayload',
    callbackQueryPayloadDataWithPassword           => 'CallbackQueryPayload',
    callbackQueryPayloadGame                       => 'CallbackQueryPayload',
    canBotSendMessages                             => 'Function',
    canPostStory                                   => 'Function',
    canPostStoryResultActiveStoryLimitExceeded     => 'CanPostStoryResult',
    canPostStoryResultBoostNeeded                  => 'CanPostStoryResult',
    canPostStoryResultLiveStoryIsActive            => 'CanPostStoryResult',
    canPostStoryResultMonthlyLimitExceeded         => 'CanPostStoryResult',
    canPostStoryResultOk                           => 'CanPostStoryResult',
    canPostStoryResultPremiumNeeded                => 'CanPostStoryResult',
    canPostStoryResultWeeklyLimitExceeded          => 'CanPostStoryResult',
    canPurchaseFromStore                           => 'Function',
    canSendGift                                    => 'Function',
    canSendGiftResultFail                          => 'CanSendGiftResult',
    canSendGiftResultOk                            => 'CanSendGiftResult',
    canSendMessageToUser                           => 'Function',
    canSendMessageToUserResultOk                   => 'CanSendMessageToUserResult',
    canSendMessageToUserResultUserHasPaidMessages  => 'CanSendMessageToUserResult',
    canSendMessageToUserResultUserIsDeleted        => 'CanSendMessageToUserResult',
    canSendMessageToUserResultUserRestrictsNewChats => 'CanSendMessageToUserResult',
    canTransferOwnership                           => 'Function',
    canTransferOwnershipResultOk                   => 'CanTransferOwnershipResult',
    canTransferOwnershipResultPasswordNeeded       => 'CanTransferOwnershipResult',
    canTransferOwnershipResultPasswordTooFresh     => 'CanTransferOwnershipResult',
    canTransferOwnershipResultSessionTooFresh      => 'CanTransferOwnershipResult',
    cancelDownloadFile                             => 'Function',
    cancelPasswordReset                            => 'Function',
    cancelPreliminaryUploadFile                    => 'Function',
    cancelRecoveryEmailAddressVerification         => 'Function',
    changeImportedContacts                         => 'Function',
    changeStickerSet                               => 'Function',
    changeWebBrowserSettings                       => 'Function',
    chat                                           => 'Object',
    chatActionBarAddContact                        => 'ChatActionBar',
    chatActionBarInviteMembers                     => 'ChatActionBar',
    chatActionBarJoinRequest                       => 'ChatActionBar',
    chatActionBarReportAddBlock                    => 'ChatActionBar',
    chatActionBarReportSpam                        => 'ChatActionBar',
    chatActionBarSharePhoneNumber                  => 'ChatActionBar',
    chatActionCancel                               => 'ChatAction',
    chatActionChoosingContact                      => 'ChatAction',
    chatActionChoosingLocation                     => 'ChatAction',
    chatActionChoosingSticker                      => 'ChatAction',
    chatActionRecordingVideo                       => 'ChatAction',
    chatActionRecordingVideoNote                   => 'ChatAction',
    chatActionRecordingVoiceNote                   => 'ChatAction',
    chatActionStartPlayingGame                     => 'ChatAction',
    chatActionTyping                               => 'ChatAction',
    chatActionUploadingDocument                    => 'ChatAction',
    chatActionUploadingPhoto                       => 'ChatAction',
    chatActionUploadingVideo                       => 'ChatAction',
    chatActionUploadingVideoNote                   => 'ChatAction',
    chatActionUploadingVoiceNote                   => 'ChatAction',
    chatActionWatchingAnimations                   => 'ChatAction',
    chatActiveStories                              => 'Object',
    chatAdministrator                              => 'Object',
    chatAdministratorRights                        => 'Object',
    chatAdministrators                             => 'Object',
    chatAvailableReactionsAll                      => 'ChatAvailableReactions',
    chatAvailableReactionsSome                     => 'ChatAvailableReactions',
    chatBackground                                 => 'Object',
    chatBoost                                      => 'Object',
    chatBoostFeatures                              => 'Object',
    chatBoostLevelFeatures                         => 'Object',
    chatBoostLink                                  => 'Object',
    chatBoostLinkInfo                              => 'Object',
    chatBoostSlot                                  => 'Object',
    chatBoostSlots                                 => 'Object',
    chatBoostSourceGiftCode                        => 'ChatBoostSource',
    chatBoostSourceGiveaway                        => 'ChatBoostSource',
    chatBoostSourcePremium                         => 'ChatBoostSource',
    chatBoostStatus                                => 'Object',
    chatEvent                                      => 'Object',
    chatEventAccentColorChanged                    => 'ChatEventAction',
    chatEventActiveUsernamesChanged                => 'ChatEventAction',
    chatEventAutomaticTranslationToggled           => 'ChatEventAction',
    chatEventAvailableReactionsChanged             => 'ChatEventAction',
    chatEventBackgroundChanged                     => 'ChatEventAction',
    chatEventCustomEmojiStickerSetChanged          => 'ChatEventAction',
    chatEventDescriptionChanged                    => 'ChatEventAction',
    chatEventEmojiStatusChanged                    => 'ChatEventAction',
    chatEventForumTopicCreated                     => 'ChatEventAction',
    chatEventForumTopicDeleted                     => 'ChatEventAction',
    chatEventForumTopicEdited                      => 'ChatEventAction',
    chatEventForumTopicPinned                      => 'ChatEventAction',
    chatEventForumTopicToggleIsClosed              => 'ChatEventAction',
    chatEventForumTopicToggleIsHidden              => 'ChatEventAction',
    chatEventHasAggressiveAntiSpamEnabledToggled   => 'ChatEventAction',
    chatEventHasProtectedContentToggled            => 'ChatEventAction',
    chatEventInviteLinkDeleted                     => 'ChatEventAction',
    chatEventInviteLinkEdited                      => 'ChatEventAction',
    chatEventInviteLinkRevoked                     => 'ChatEventAction',
    chatEventInvitesToggled                        => 'ChatEventAction',
    chatEventIsAllHistoryAvailableToggled          => 'ChatEventAction',
    chatEventIsForumToggled                        => 'ChatEventAction',
    chatEventLinkedChatChanged                     => 'ChatEventAction',
    chatEventLocationChanged                       => 'ChatEventAction',
    chatEventLogFilters                            => 'Object',
    chatEventMemberInvited                         => 'ChatEventAction',
    chatEventMemberJoined                          => 'ChatEventAction',
    chatEventMemberJoinedByInviteLink              => 'ChatEventAction',
    chatEventMemberJoinedByRequest                 => 'ChatEventAction',
    chatEventMemberLeft                            => 'ChatEventAction',
    chatEventMemberPromoted                        => 'ChatEventAction',
    chatEventMemberRestricted                      => 'ChatEventAction',
    chatEventMemberSubscriptionExtended            => 'ChatEventAction',
    chatEventMemberTagChanged                      => 'ChatEventAction',
    chatEventMessageAutoDeleteTimeChanged          => 'ChatEventAction',
    chatEventMessageDeleted                        => 'ChatEventAction',
    chatEventMessageEdited                         => 'ChatEventAction',
    chatEventMessagePinned                         => 'ChatEventAction',
    chatEventMessageUnpinned                       => 'ChatEventAction',
    chatEventPermissionsChanged                    => 'ChatEventAction',
    chatEventPhotoChanged                          => 'ChatEventAction',
    chatEventPollStopped                           => 'ChatEventAction',
    chatEventProfileAccentColorChanged             => 'ChatEventAction',
    chatEventShowMessageSenderToggled              => 'ChatEventAction',
    chatEventSignMessagesToggled                   => 'ChatEventAction',
    chatEventSlowModeDelayChanged                  => 'ChatEventAction',
    chatEventStickerSetChanged                     => 'ChatEventAction',
    chatEventTitleChanged                          => 'ChatEventAction',
    chatEventUsernameChanged                       => 'ChatEventAction',
    chatEventVideoChatCreated                      => 'ChatEventAction',
    chatEventVideoChatEnded                        => 'ChatEventAction',
    chatEventVideoChatMuteNewParticipantsToggled   => 'ChatEventAction',
    chatEventVideoChatParticipantIsMutedToggled    => 'ChatEventAction',
    chatEventVideoChatParticipantVolumeLevelChanged => 'ChatEventAction',
    chatEvents                                     => 'Object',
    chatFolder                                     => 'Object',
    chatFolderIcon                                 => 'Object',
    chatFolderInfo                                 => 'Object',
    chatFolderInviteLink                           => 'Object',
    chatFolderInviteLinkInfo                       => 'Object',
    chatFolderInviteLinks                          => 'Object',
    chatFolderName                                 => 'Object',
    chatInviteLink                                 => 'Object',
    chatInviteLinkCount                            => 'Object',
    chatInviteLinkCounts                           => 'Object',
    chatInviteLinkInfo                             => 'Object',
    chatInviteLinkMember                           => 'Object',
    chatInviteLinkMembers                          => 'Object',
    chatInviteLinkSubscriptionInfo                 => 'Object',
    chatInviteLinks                                => 'Object',
    chatJoinRequest                                => 'Object',
    chatJoinRequestResultApproved                  => 'ChatJoinRequestResult',
    chatJoinRequestResultDeclined                  => 'ChatJoinRequestResult',
    chatJoinRequestResultQueued                    => 'ChatJoinRequestResult',
    chatJoinRequests                               => 'Object',
    chatJoinRequestsInfo                           => 'Object',
    chatJoinResultDeclined                         => 'ChatJoinResult',
    chatJoinResultGuardBotApprovalRequired         => 'ChatJoinResult',
    chatJoinResultRequestSent                      => 'ChatJoinResult',
    chatJoinResultSuccess                          => 'ChatJoinResult',
    chatListArchive                                => 'ChatList',
    chatListFolder                                 => 'ChatList',
    chatListMain                                   => 'ChatList',
    chatLists                                      => 'Object',
    chatLocation                                   => 'Object',
    chatMember                                     => 'Object',
    chatMemberStatusAdministrator                  => 'ChatMemberStatus',
    chatMemberStatusBanned                         => 'ChatMemberStatus',
    chatMemberStatusCreator                        => 'ChatMemberStatus',
    chatMemberStatusLeft                           => 'ChatMemberStatus',
    chatMemberStatusMember                         => 'ChatMemberStatus',
    chatMemberStatusRestricted                     => 'ChatMemberStatus',
    chatMembers                                    => 'Object',
    chatMembersFilterAdministrators                => 'ChatMembersFilter',
    chatMembersFilterBanned                        => 'ChatMembersFilter',
    chatMembersFilterBots                          => 'ChatMembersFilter',
    chatMembersFilterContacts                      => 'ChatMembersFilter',
    chatMembersFilterMembers                       => 'ChatMembersFilter',
    chatMembersFilterMention                       => 'ChatMembersFilter',
    chatMembersFilterRestricted                    => 'ChatMembersFilter',
    chatMessageSender                              => 'Object',
    chatMessageSenders                             => 'Object',
    chatNotificationSettings                       => 'Object',
    chatPermissions                                => 'Object',
    chatPhoto                                      => 'Object',
    chatPhotoInfo                                  => 'Object',
    chatPhotoSticker                               => 'Object',
    chatPhotoStickerTypeCustomEmoji                => 'ChatPhotoStickerType',
    chatPhotoStickerTypeRegularOrMask              => 'ChatPhotoStickerType',
    chatPhotos                                     => 'Object',
    chatPosition                                   => 'Object',
    chatRevenueAmount                              => 'Object',
    chatRevenueStatistics                          => 'Object',
    chatRevenueTransaction                         => 'Object',
    chatRevenueTransactionTypeFragmentRefund       => 'ChatRevenueTransactionType',
    chatRevenueTransactionTypeFragmentWithdrawal   => 'ChatRevenueTransactionType',
    chatRevenueTransactionTypeSponsoredMessageEarnings => 'ChatRevenueTransactionType',
    chatRevenueTransactionTypeSuggestedPostEarnings => 'ChatRevenueTransactionType',
    chatRevenueTransactionTypeUnsupported          => 'ChatRevenueTransactionType',
    chatRevenueTransactions                        => 'Object',
    chatSourceMtprotoProxy                         => 'ChatSource',
    chatSourcePublicServiceAnnouncement            => 'ChatSource',
    chatStatisticsAdministratorActionsInfo         => 'Object',
    chatStatisticsChannel                          => 'ChatStatistics',
    chatStatisticsInteractionInfo                  => 'Object',
    chatStatisticsInviterInfo                      => 'Object',
    chatStatisticsMessageSenderInfo                => 'Object',
    chatStatisticsObjectTypeMessage                => 'ChatStatisticsObjectType',
    chatStatisticsObjectTypeStory                  => 'ChatStatisticsObjectType',
    chatStatisticsSupergroup                       => 'ChatStatistics',
    chatThemeEmoji                                 => 'ChatTheme',
    chatThemeGift                                  => 'ChatTheme',
    chatTypeBasicGroup                             => 'ChatType',
    chatTypePrivate                                => 'ChatType',
    chatTypeSecret                                 => 'ChatType',
    chatTypeSupergroup                             => 'ChatType',
    chats                                          => 'Object',
    checkAuthenticationBotToken                    => 'Function',
    checkAuthenticationCode                        => 'Function',
    checkAuthenticationEmailCode                   => 'Function',
    checkAuthenticationPasskey                     => 'Function',
    checkAuthenticationPassword                    => 'Function',
    checkAuthenticationPasswordRecoveryCode        => 'Function',
    checkAuthenticationPremiumPurchase             => 'Function',
    checkAuthenticationWebToken                    => 'Function',
    checkBotUsername                               => 'Function',
    checkChatFolderInviteLink                      => 'Function',
    checkChatInviteLink                            => 'Function',
    checkChatUsername                              => 'Function',
    checkChatUsernameResultOk                      => 'CheckChatUsernameResult',
    checkChatUsernameResultPublicChatsTooMany      => 'CheckChatUsernameResult',
    checkChatUsernameResultPublicGroupsUnavailable => 'CheckChatUsernameResult',
    checkChatUsernameResultUsernameInvalid         => 'CheckChatUsernameResult',
    checkChatUsernameResultUsernameOccupied        => 'CheckChatUsernameResult',
    checkChatUsernameResultUsernamePurchasable     => 'CheckChatUsernameResult',
    checkCreatedPublicChatsLimit                   => 'Function',
    checkEmailAddressVerificationCode              => 'Function',
    checkLoginEmailAddressCode                     => 'Function',
    checkOauthRequestMatchCode                     => 'Function',
    checkPasswordRecoveryCode                      => 'Function',
    checkPhoneNumberCode                           => 'Function',
    checkPremiumGiftCode                           => 'Function',
    checkQuickReplyShortcutName                    => 'Function',
    checkRecoveryEmailAddressCode                  => 'Function',
    checkStickerSetName                            => 'Function',
    checkStickerSetNameResultNameInvalid           => 'CheckStickerSetNameResult',
    checkStickerSetNameResultNameOccupied          => 'CheckStickerSetNameResult',
    checkStickerSetNameResultOk                    => 'CheckStickerSetNameResult',
    checkWebAppFileDownload                        => 'Function',
    checklist                                      => 'Object',
    checklistTask                                  => 'Object',
    cleanFileName                                  => 'Function',
    clearAllDraftMessages                          => 'Function',
    clearAutosaveSettingsExceptions                => 'Function',
    clearImportedContacts                          => 'Function',
    clearRecentEmojiStatuses                       => 'Function',
    clearRecentReactions                           => 'Function',
    clearRecentStickers                            => 'Function',
    clearRecentlyFoundChats                        => 'Function',
    clearSearchedForTags                           => 'Function',
    clickAnimatedEmojiMessage                      => 'Function',
    clickChatSponsoredMessage                      => 'Function',
    clickPremiumSubscriptionButton                 => 'Function',
    clickVideoMessageAdvertisement                 => 'Function',
    close                                          => 'Function',
    closeBirthdayUser                              => 'Object',
    closeChat                                      => 'Function',
    closeGiftAuction                               => 'Function',
    closeSecretChat                                => 'Function',
    closeStory                                     => 'Function',
    closeWebApp                                    => 'Function',
    closedVectorPath                               => 'Object',
    collectibleItemInfo                            => 'Object',
    collectibleItemTypePhoneNumber                 => 'CollectibleItemType',
    collectibleItemTypeUsername                    => 'CollectibleItemType',
    commitPendingLiveStoryReactions                => 'Function',
    commitPendingPaidMessageReactions              => 'Function',
    community                                      => 'Object',
    communityAdministratorRights                   => 'Object',
    communityMemberStatusAdministrator             => 'CommunityMemberStatus',
    communityMemberStatusBanned                    => 'CommunityMemberStatus',
    communityMemberStatusCreator                   => 'CommunityMemberStatus',
    communityMemberStatusLeft                      => 'CommunityMemberStatus',
    communityMemberStatusMember                    => 'CommunityMemberStatus',
    communityPermissions                           => 'Object',
    composeRichMessageWithAi                       => 'Function',
    composeTextWithAi                              => 'Function',
    confirmBusinessConnectedBot                    => 'Function',
    confirmQrCodeAuthentication                    => 'Function',
    confirmSession                                 => 'Function',
    connectAffiliateProgram                        => 'Function',
    connectedAffiliateProgram                      => 'Object',
    connectedAffiliatePrograms                     => 'Object',
    connectedWebsite                               => 'Object',
    connectedWebsites                              => 'Object',
    connectionStateConnecting                      => 'ConnectionState',
    connectionStateConnectingToProxy               => 'ConnectionState',
    connectionStateReady                           => 'ConnectionState',
    connectionStateUpdating                        => 'ConnectionState',
    connectionStateWaitingForNetwork               => 'ConnectionState',
    contact                                        => 'Object',
    count                                          => 'Object',
    countries                                      => 'Object',
    countryInfo                                    => 'Object',
    craftGift                                      => 'Function',
    craftGiftResultFail                            => 'CraftGiftResult',
    craftGiftResultInvalidGift                     => 'CraftGiftResult',
    craftGiftResultSuccess                         => 'CraftGiftResult',
    craftGiftResultTooEarly                        => 'CraftGiftResult',
    createBasicGroupChat                           => 'Function',
    createBot                                      => 'Function',
    createBusinessChatLink                         => 'Function',
    createCall                                     => 'Function',
    createChatFolder                               => 'Function',
    createChatFolderInviteLink                     => 'Function',
    createChatInviteLink                           => 'Function',
    createChatSubscriptionInviteLink               => 'Function',
    createForumTopic                               => 'Function',
    createGiftCollection                           => 'Function',
    createGroupCall                                => 'Function',
    createInvoiceLink                              => 'Function',
    createNewBasicGroupChat                        => 'Function',
    createNewSecretChat                            => 'Function',
    createNewStickerSet                            => 'Function',
    createNewSupergroupChat                        => 'Function',
    createPrivateChat                              => 'Function',
    createRichMessageWithAi                        => 'Function',
    createSecretChat                               => 'Function',
    createStoryAlbum                               => 'Function',
    createSupergroupChat                           => 'Function',
    createTemporaryPassword                        => 'Function',
    createTextCompositionStyle                     => 'Function',
    createVideoChat                                => 'Function',
    createdBasicGroupChat                          => 'Object',
    currentWeather                                 => 'Object',
    customRequestResult                            => 'Object',
    data                                           => 'Object',
    databaseStatistics                             => 'Object',
    date                                           => 'Object',
    dateRange                                      => 'Object',
    dateTimeFormattingTypeAbsolute                 => 'DateTimeFormattingType',
    dateTimeFormattingTypeRelative                 => 'DateTimeFormattingType',
    dateTimePartPrecisionLong                      => 'DateTimePartPrecision',
    dateTimePartPrecisionNone                      => 'DateTimePartPrecision',
    dateTimePartPrecisionShort                     => 'DateTimePartPrecision',
    datedFile                                      => 'Object',
    declineGroupCallInvitation                     => 'Function',
    declineOauthRequest                            => 'Function',
    declineSuggestedPost                           => 'Function',
    decryptGroupCallData                           => 'Function',
    deepLinkInfo                                   => 'Object',
    deleteAccount                                  => 'Function',
    deleteAllCallMessages                          => 'Function',
    deleteAllRecentMessageReactionsFromSender      => 'Function',
    deleteAllRevokedChatInviteLinks                => 'Function',
    deleteBotMediaPreviews                         => 'Function',
    deleteBusinessChatLink                         => 'Function',
    deleteBusinessConnectedBot                     => 'Function',
    deleteBusinessMessages                         => 'Function',
    deleteBusinessStory                            => 'Function',
    deleteChat                                     => 'Function',
    deleteChatBackground                           => 'Function',
    deleteChatFolder                               => 'Function',
    deleteChatFolderInviteLink                     => 'Function',
    deleteChatHistory                              => 'Function',
    deleteChatMessagesByDate                       => 'Function',
    deleteChatMessagesBySender                     => 'Function',
    deleteChatReplyMarkup                          => 'Function',
    deleteCommands                                 => 'Function',
    deleteDefaultBackground                        => 'Function',
    deleteDirectMessagesChatTopicHistory           => 'Function',
    deleteDirectMessagesChatTopicMessagesByDate    => 'Function',
    deleteEphemeralMessage                         => 'Function',
    deleteFile                                     => 'Function',
    deleteForumTopic                               => 'Function',
    deleteGiftCollection                           => 'Function',
    deleteGroupCallMessages                        => 'Function',
    deleteGroupCallMessagesBySender                => 'Function',
    deleteLanguagePack                             => 'Function',
    deleteMessageReactionsFromSender               => 'Function',
    deleteMessages                                 => 'Function',
    deletePassportElement                          => 'Function',
    deletePollOption                               => 'Function',
    deleteProfilePhoto                             => 'Function',
    deleteQuickReplyShortcut                       => 'Function',
    deleteQuickReplyShortcutMessages               => 'Function',
    deleteRevokedChatInviteLink                    => 'Function',
    deleteSavedCredentials                         => 'Function',
    deleteSavedMessagesTopicHistory                => 'Function',
    deleteSavedMessagesTopicMessagesByDate         => 'Function',
    deleteSavedOrderInfo                           => 'Function',
    deleteStickerSet                               => 'Function',
    deleteStory                                    => 'Function',
    deleteStoryAlbum                               => 'Function',
    deleteTextCompositionStyle                     => 'Function',
    destroy                                        => 'Function',
    deviceTokenApplePush                           => 'DeviceToken',
    deviceTokenApplePushVoIP                       => 'DeviceToken',
    deviceTokenBlackBerryPush                      => 'DeviceToken',
    deviceTokenFirebaseCloudMessaging              => 'DeviceToken',
    deviceTokenHuaweiPush                          => 'DeviceToken',
    deviceTokenMicrosoftPush                       => 'DeviceToken',
    deviceTokenMicrosoftPushVoIP                   => 'DeviceToken',
    deviceTokenSimplePush                          => 'DeviceToken',
    deviceTokenTizenPush                           => 'DeviceToken',
    deviceTokenUbuntuPush                          => 'DeviceToken',
    deviceTokenWebPush                             => 'DeviceToken',
    deviceTokenWindowsPush                         => 'DeviceToken',
    diceStickersRegular                            => 'DiceStickers',
    diceStickersSlotMachine                        => 'DiceStickers',
    diffEntity                                     => 'Object',
    diffEntityTypeDelete                           => 'DiffEntityType',
    diffEntityTypeInsert                           => 'DiffEntityType',
    diffEntityTypeReplace                          => 'DiffEntityType',
    diffText                                       => 'Object',
    directMessagesChatTopic                        => 'Object',
    disableAllSupergroupUsernames                  => 'Function',
    disableProxy                                   => 'Function',
    discardCall                                    => 'Function',
    disconnectAffiliateProgram                     => 'Function',
    disconnectAllWebsites                          => 'Function',
    disconnectWebsite                              => 'Function',
    document                                       => 'Object',
    downloadFile                                   => 'Function',
    downloadedFileCounts                           => 'Object',
    draftMessage                                   => 'Object',
    draftMessageContentRichMessage                 => 'DraftMessageContent',
    draftMessageContentText                        => 'DraftMessageContent',
    draftMessageContentVideoNote                   => 'DraftMessageContent',
    draftMessageContentVoiceNote                   => 'DraftMessageContent',
    dropGiftOriginalDetails                        => 'Function',
    editBotMediaPreview                            => 'Function',
    editBusinessChatLink                           => 'Function',
    editBusinessMessageCaption                     => 'Function',
    editBusinessMessageChecklist                   => 'Function',
    editBusinessMessageLiveLocation                => 'Function',
    editBusinessMessageMedia                       => 'Function',
    editBusinessMessageReplyMarkup                 => 'Function',
    editBusinessMessageText                        => 'Function',
    editBusinessStory                              => 'Function',
    editChatFolder                                 => 'Function',
    editChatFolderInviteLink                       => 'Function',
    editChatInviteLink                             => 'Function',
    editChatSubscriptionInviteLink                 => 'Function',
    editCustomLanguagePackInfo                     => 'Function',
    editEphemeralMessage                           => 'Function',
    editForumTopic                                 => 'Function',
    editInlineMessageCaption                       => 'Function',
    editInlineMessageLiveLocation                  => 'Function',
    editInlineMessageMedia                         => 'Function',
    editInlineMessageReplyMarkup                   => 'Function',
    editInlineMessageText                          => 'Function',
    editMessageCaption                             => 'Function',
    editMessageChecklist                           => 'Function',
    editMessageLiveLocation                        => 'Function',
    editMessageMedia                               => 'Function',
    editMessageReplyMarkup                         => 'Function',
    editMessageSchedulingState                     => 'Function',
    editMessageText                                => 'Function',
    editProxy                                      => 'Function',
    editQuickReplyMessage                          => 'Function',
    editStarSubscription                           => 'Function',
    editStory                                      => 'Function',
    editStoryCover                                 => 'Function',
    editTextCompositionStyle                       => 'Function',
    editUserStarSubscription                       => 'Function',
    emailAddressAuthenticationAppleId              => 'EmailAddressAuthentication',
    emailAddressAuthenticationCode                 => 'EmailAddressAuthentication',
    emailAddressAuthenticationCodeInfo             => 'Object',
    emailAddressAuthenticationGoogleId             => 'EmailAddressAuthentication',
    emailAddressResetStateAvailable                => 'EmailAddressResetState',
    emailAddressResetStatePending                  => 'EmailAddressResetState',
    emojiCategories                                => 'Object',
    emojiCategory                                  => 'Object',
    emojiCategorySourcePremium                     => 'EmojiCategorySource',
    emojiCategorySourceSearch                      => 'EmojiCategorySource',
    emojiCategoryTypeChatPhoto                     => 'EmojiCategoryType',
    emojiCategoryTypeDefault                       => 'EmojiCategoryType',
    emojiCategoryTypeEmojiStatus                   => 'EmojiCategoryType',
    emojiCategoryTypeRegularStickers               => 'EmojiCategoryType',
    emojiChatTheme                                 => 'Object',
    emojiKeyword                                   => 'Object',
    emojiKeywords                                  => 'Object',
    emojiReaction                                  => 'Object',
    emojiStatus                                    => 'Object',
    emojiStatusCustomEmojis                        => 'Object',
    emojiStatusTypeCustomEmoji                     => 'EmojiStatusType',
    emojiStatusTypeUpgradedGift                    => 'EmojiStatusType',
    emojiStatuses                                  => 'Object',
    emojis                                         => 'Object',
    enableProxy                                    => 'Function',
    encryptGroupCallData                           => 'Function',
    encryptedCredentials                           => 'Object',
    encryptedPassportElement                       => 'Object',
    endGroupCall                                   => 'Function',
    endGroupCallRecording                          => 'Function',
    endGroupCallScreenSharing                      => 'Function',
    error                                          => 'Object',
    factCheck                                      => 'Object',
    failedToAddMember                              => 'Object',
    failedToAddMembers                             => 'Object',
    file                                           => 'Object',
    fileDownload                                   => 'Object',
    fileDownloadedPrefixSize                       => 'Object',
    fileTypeAnimation                              => 'FileType',
    fileTypeAudio                                  => 'FileType',
    fileTypeDocument                               => 'FileType',
    fileTypeLivePhotoVideo                         => 'FileType',
    fileTypeNone                                   => 'FileType',
    fileTypeNotificationSound                      => 'FileType',
    fileTypePhoto                                  => 'FileType',
    fileTypePhotoStory                             => 'FileType',
    fileTypeProfilePhoto                           => 'FileType',
    fileTypeSecret                                 => 'FileType',
    fileTypeSecretThumbnail                        => 'FileType',
    fileTypeSecure                                 => 'FileType',
    fileTypeSelfDestructingLivePhotoVideo          => 'FileType',
    fileTypeSelfDestructingPhoto                   => 'FileType',
    fileTypeSelfDestructingVideo                   => 'FileType',
    fileTypeSelfDestructingVideoNote               => 'FileType',
    fileTypeSelfDestructingVoiceNote               => 'FileType',
    fileTypeSticker                                => 'FileType',
    fileTypeThumbnail                              => 'FileType',
    fileTypeUnknown                                => 'FileType',
    fileTypeVideo                                  => 'FileType',
    fileTypeVideoNote                              => 'FileType',
    fileTypeVideoStory                             => 'FileType',
    fileTypeVoiceNote                              => 'FileType',
    fileTypeWallpaper                              => 'FileType',
    finishFileGeneration                           => 'Function',
    firebaseAuthenticationSettingsAndroid          => 'FirebaseAuthenticationSettings',
    firebaseAuthenticationSettingsIos              => 'FirebaseAuthenticationSettings',
    firebaseDeviceVerificationParametersPlayIntegrity => 'FirebaseDeviceVerificationParameters',
    firebaseDeviceVerificationParametersSafetyNet  => 'FirebaseDeviceVerificationParameters',
    fixRichMessageWithAi                           => 'Function',
    fixTextWithAi                                  => 'Function',
    fixedText                                      => 'Object',
    formattedText                                  => 'Object',
    forumTopic                                     => 'Object',
    forumTopicIcon                                 => 'Object',
    forumTopicInfo                                 => 'Object',
    forumTopics                                    => 'Object',
    forwardMessages                                => 'Function',
    forwardSource                                  => 'Object',
    foundAffiliateProgram                          => 'Object',
    foundAffiliatePrograms                         => 'Object',
    foundChatBoosts                                => 'Object',
    foundChatMessages                              => 'Object',
    foundFileDownloads                             => 'Object',
    foundMessages                                  => 'Object',
    foundPosition                                  => 'Object',
    foundPositions                                 => 'Object',
    foundPublicPosts                               => 'Object',
    foundStories                                   => 'Object',
    foundUsers                                     => 'Object',
    foundWebApp                                    => 'Object',
    game                                           => 'Object',
    gameHighScore                                  => 'Object',
    gameHighScores                                 => 'Object',
    getAccountTtl                                  => 'Function',
    getActiveSessions                              => 'Function',
    getAllPassportElements                         => 'Function',
    getAllStickerEmojis                            => 'Function',
    getAnimatedEmoji                               => 'Function',
    getApplicationConfig                           => 'Function',
    getApplicationDownloadLink                     => 'Function',
    getArchiveChatListSettings                     => 'Function',
    getArchivedStickerSets                         => 'Function',
    getAttachedStickerSets                         => 'Function',
    getAttachmentMenuBot                           => 'Function',
    getAuthenticationPasskeyParameters             => 'Function',
    getAuthorizationState                          => 'Function',
    getAutoDownloadSettingsPresets                 => 'Function',
    getAutosaveSettings                            => 'Function',
    getAvailableChatBoostSlots                     => 'Function',
    getAvailableGifts                              => 'Function',
    getBackgroundUrl                               => 'Function',
    getBankCardInfo                                => 'Function',
    getBasicGroup                                  => 'Function',
    getBasicGroupFullInfo                          => 'Function',
    getBlockedMessageSenders                       => 'Function',
    getBotInfoDescription                          => 'Function',
    getBotInfoShortDescription                     => 'Function',
    getBotMediaPreviewInfo                         => 'Function',
    getBotMediaPreviews                            => 'Function',
    getBotName                                     => 'Function',
    getBotSimilarBotCount                          => 'Function',
    getBotSimilarBots                              => 'Function',
    getBusinessAccountStarAmount                   => 'Function',
    getBusinessChatLinkInfo                        => 'Function',
    getBusinessChatLinks                           => 'Function',
    getBusinessConnectedBot                        => 'Function',
    getBusinessConnection                          => 'Function',
    getBusinessFeatures                            => 'Function',
    getCallbackQueryAnswer                         => 'Function',
    getCallbackQueryMessage                        => 'Function',
    getChat                                        => 'Function',
    getChatActiveStories                           => 'Function',
    getChatAdministrators                          => 'Function',
    getChatArchivedStories                         => 'Function',
    getChatAvailableMessageSenders                 => 'Function',
    getChatAvailablePaidMessageReactionSenders     => 'Function',
    getChatBoostFeatures                           => 'Function',
    getChatBoostLevelFeatures                      => 'Function',
    getChatBoostLink                               => 'Function',
    getChatBoostLinkInfo                           => 'Function',
    getChatBoostStatus                             => 'Function',
    getChatBoosts                                  => 'Function',
    getChatEventLog                                => 'Function',
    getChatFolder                                  => 'Function',
    getChatFolderChatCount                         => 'Function',
    getChatFolderChatsToLeave                      => 'Function',
    getChatFolderDefaultIconName                   => 'Function',
    getChatFolderInviteLinks                       => 'Function',
    getChatFolderNewChats                          => 'Function',
    getChatHistory                                 => 'Function',
    getChatInviteLink                              => 'Function',
    getChatInviteLinkCounts                        => 'Function',
    getChatInviteLinkMembers                       => 'Function',
    getChatInviteLinks                             => 'Function',
    getChatJoinRequests                            => 'Function',
    getChatListsToAddChat                          => 'Function',
    getChatMember                                  => 'Function',
    getChatMessageByDate                           => 'Function',
    getChatMessageCalendar                         => 'Function',
    getChatMessageCount                            => 'Function',
    getChatMessagePosition                         => 'Function',
    getChatNotificationSettingsExceptions          => 'Function',
    getChatOwnerAfterLeaving                       => 'Function',
    getChatPinnedMessage                           => 'Function',
    getChatPostedToChatPageStories                 => 'Function',
    getChatRevenueStatistics                       => 'Function',
    getChatRevenueTransactions                     => 'Function',
    getChatRevenueWithdrawalUrl                    => 'Function',
    getChatScheduledMessages                       => 'Function',
    getChatSimilarChatCount                        => 'Function',
    getChatSimilarChats                            => 'Function',
    getChatSparseMessagePositions                  => 'Function',
    getChatSponsoredMessages                       => 'Function',
    getChatStatistics                              => 'Function',
    getChatStoryAlbums                             => 'Function',
    getChatStoryInteractions                       => 'Function',
    getChats                                       => 'Function',
    getChatsForChatFolderInviteLink                => 'Function',
    getChatsToPostStories                          => 'Function',
    getCloseFriends                                => 'Function',
    getCollectibleItemInfo                         => 'Function',
    getCommands                                    => 'Function',
    getConnectedAffiliateProgram                   => 'Function',
    getConnectedAffiliatePrograms                  => 'Function',
    getConnectedWebsites                           => 'Function',
    getContacts                                    => 'Function',
    getCountries                                   => 'Function',
    getCountry                                     => 'Function',
    getCountryCode                                 => 'Function',
    getCountryFlagEmoji                            => 'Function',
    getCreatedPublicChats                          => 'Function',
    getCurrentState                                => 'Function',
    getCurrentWeather                              => 'Function',
    getCustomEmojiReactionAnimations               => 'Function',
    getCustomEmojiStickers                         => 'Function',
    getDatabaseStatistics                          => 'Function',
    getDeepLinkInfo                                => 'Function',
    getDefaultBackgroundCustomEmojiStickers        => 'Function',
    getDefaultChatEmojiStatuses                    => 'Function',
    getDefaultChatPhotoCustomEmojiStickers         => 'Function',
    getDefaultEmojiStatuses                        => 'Function',
    getDefaultMessageAutoDeleteTime                => 'Function',
    getDefaultProfilePhotoCustomEmojiStickers      => 'Function',
    getDirectMessagesChatTopic                     => 'Function',
    getDirectMessagesChatTopicHistory              => 'Function',
    getDirectMessagesChatTopicMessageByDate        => 'Function',
    getDirectMessagesChatTopicRevenue              => 'Function',
    getDisallowedChatEmojiStatuses                 => 'Function',
    getEmojiCategories                             => 'Function',
    getEmojiReaction                               => 'Function',
    getEmojiSuggestionsUrl                         => 'Function',
    getExternalLink                                => 'Function',
    getExternalLinkInfo                            => 'Function',
    getFavoriteStickers                            => 'Function',
    getFile                                        => 'Function',
    getFileDownloadedPrefixSize                    => 'Function',
    getFileExtension                               => 'Function',
    getFileMimeType                                => 'Function',
    getForumTopic                                  => 'Function',
    getForumTopicDefaultIcons                      => 'Function',
    getForumTopicHistory                           => 'Function',
    getForumTopicLink                              => 'Function',
    getForumTopics                                 => 'Function',
    getFullRichMessage                             => 'Function',
    getGameHighScores                              => 'Function',
    getGiftAuctionAcquiredGifts                    => 'Function',
    getGiftAuctionState                            => 'Function',
    getGiftChatThemes                              => 'Function',
    getGiftCollections                             => 'Function',
    getGiftUpgradePreview                          => 'Function',
    getGiftsForCrafting                            => 'Function',
    getGiveawayInfo                                => 'Function',
    getGramRevenueStatistics                       => 'Function',
    getGramWithdrawalUrl                           => 'Function',
    getGreetingStickers                            => 'Function',
    getGrossingWebAppBots                          => 'Function',
    getGroupCall                                   => 'Function',
    getGroupCallParticipants                       => 'Function',
    getGroupCallStreamSegment                      => 'Function',
    getGroupCallStreams                            => 'Function',
    getGroupsInCommon                              => 'Function',
    getGuardBotWebAppUrl                           => 'Function',
    getImportedContactCount                        => 'Function',
    getInactiveSupergroupChats                     => 'Function',
    getInlineGameHighScores                        => 'Function',
    getInlineQueryResults                          => 'Function',
    getInstalledBackgrounds                        => 'Function',
    getInstalledStickerSets                        => 'Function',
    getInternalLink                                => 'Function',
    getInternalLinkType                            => 'Function',
    getJsonString                                  => 'Function',
    getJsonValue                                   => 'Function',
    getKeywordEmojis                               => 'Function',
    getLanguagePackInfo                            => 'Function',
    getLanguagePackString                          => 'Function',
    getLanguagePackStrings                         => 'Function',
    getLinkPreview                                 => 'Function',
    getLinkWebBrowserType                          => 'Function',
    getLiveStoryAvailableMessageSenders            => 'Function',
    getLiveStoryRtmpUrl                            => 'Function',
    getLiveStoryStreamer                           => 'Function',
    getLiveStoryTopDonors                          => 'Function',
    getLocalizationTargetInfo                      => 'Function',
    getLogStream                                   => 'Function',
    getLogTagVerbosityLevel                        => 'Function',
    getLogTags                                     => 'Function',
    getLogVerbosityLevel                           => 'Function',
    getLoginPasskeys                               => 'Function',
    getLoginUrl                                    => 'Function',
    getLoginUrlInfo                                => 'Function',
    getMainWebApp                                  => 'Function',
    getManagedBotAccessSettings                    => 'Function',
    getManagedBotToken                             => 'Function',
    getMapThumbnailFile                            => 'Function',
    getMarkdownText                                => 'Function',
    getMe                                          => 'Function',
    getMenuButton                                  => 'Function',
    getMessage                                     => 'Function',
    getMessageAddedReactions                       => 'Function',
    getMessageAuthor                               => 'Function',
    getMessageAvailableReactions                   => 'Function',
    getMessageEffect                               => 'Function',
    getMessageEmbeddingCode                        => 'Function',
    getMessageFileType                             => 'Function',
    getMessageImportConfirmationText               => 'Function',
    getMessageLink                                 => 'Function',
    getMessageLinkInfo                             => 'Function',
    getMessageLocally                              => 'Function',
    getMessageProperties                           => 'Function',
    getMessagePublicForwards                       => 'Function',
    getMessageReadDate                             => 'Function',
    getMessageStatistics                           => 'Function',
    getMessageThread                               => 'Function',
    getMessageThreadHistory                        => 'Function',
    getMessageViewers                              => 'Function',
    getMessages                                    => 'Function',
    getNetworkStatistics                           => 'Function',
    getNewChatPrivacySettings                      => 'Function',
    getOauthLinkInfo                               => 'Function',
    getOption                                      => 'Function',
    getOwnedBots                                   => 'Function',
    getOwnedStickerSets                            => 'Function',
    getPaidMessageRevenue                          => 'Function',
    getPasskeyParameters                           => 'Function',
    getPassportAuthorizationForm                   => 'Function',
    getPassportAuthorizationFormAvailableElements  => 'Function',
    getPassportElement                             => 'Function',
    getPasswordState                               => 'Function',
    getPaymentForm                                 => 'Function',
    getPaymentReceipt                              => 'Function',
    getPersonalChatHistory                         => 'Function',
    getPhoneNumberInfo                             => 'Function',
    getPhoneNumberInfoSync                         => 'Function',
    getPollOptionProperties                        => 'Function',
    getPollVoteStatistics                          => 'Function',
    getPollVoters                                  => 'Function',
    getPreferredCountryLanguage                    => 'Function',
    getPremiumFeatures                             => 'Function',
    getPremiumGiftPaymentOptions                   => 'Function',
    getPremiumGiveawayPaymentOptions               => 'Function',
    getPremiumInfoSticker                          => 'Function',
    getPremiumLimit                                => 'Function',
    getPremiumState                                => 'Function',
    getPremiumStickerExamples                      => 'Function',
    getPremiumStickers                             => 'Function',
    getPreparedInlineMessage                       => 'Function',
    getPreparedKeyboardButton                      => 'Function',
    getProxies                                     => 'Function',
    getPublicPostSearchLimits                      => 'Function',
    getPushReceiverId                              => 'Function',
    getReadDatePrivacySettings                     => 'Function',
    getReceivedGift                                => 'Function',
    getReceivedGifts                               => 'Function',
    getRecentEmojiStatuses                         => 'Function',
    getRecentInlineBots                            => 'Function',
    getRecentStickers                              => 'Function',
    getRecentlyOpenedChats                         => 'Function',
    getRecentlyVisitedTMeUrls                      => 'Function',
    getRecommendedChatFolders                      => 'Function',
    getRecommendedChats                            => 'Function',
    getRecoveryEmailAddress                        => 'Function',
    getRemoteFile                                  => 'Function',
    getRepliedMessage                              => 'Function',
    getSavedAnimations                             => 'Function',
    getSavedMessagesTags                           => 'Function',
    getSavedMessagesTopicHistory                   => 'Function',
    getSavedMessagesTopicMessageByDate             => 'Function',
    getSavedNotificationSound                      => 'Function',
    getSavedNotificationSounds                     => 'Function',
    getSavedOrderInfo                              => 'Function',
    getScopeNotificationSettings                   => 'Function',
    getSearchSponsoredChats                        => 'Function',
    getSearchedForTags                             => 'Function',
    getSecretChat                                  => 'Function',
    getStakeDiceState                              => 'Function',
    getStarAdAccountUrl                            => 'Function',
    getStarGiftPaymentOptions                      => 'Function',
    getStarGiveawayPaymentOptions                  => 'Function',
    getStarPaymentOptions                          => 'Function',
    getStarRevenueStatistics                       => 'Function',
    getStarSubscriptions                           => 'Function',
    getStarTransactions                            => 'Function',
    getStarWithdrawalUrl                           => 'Function',
    getStatisticalGraph                            => 'Function',
    getStickerEmojis                               => 'Function',
    getStickerOutline                              => 'Function',
    getStickerOutlineSvgPath                       => 'Function',
    getStickerSet                                  => 'Function',
    getStickerSetName                              => 'Function',
    getStickers                                    => 'Function',
    getStorageStatistics                           => 'Function',
    getStorageStatisticsFast                       => 'Function',
    getStory                                       => 'Function',
    getStoryAlbumStories                           => 'Function',
    getStoryAvailableReactions                     => 'Function',
    getStoryInteractions                           => 'Function',
    getStoryNotificationSettingsExceptions         => 'Function',
    getStoryPublicForwards                         => 'Function',
    getStoryStatistics                             => 'Function',
    getSuggestedFileName                           => 'Function',
    getSuggestedStickerSetName                     => 'Function',
    getSuitableDiscussionChats                     => 'Function',
    getSuitablePersonalChats                       => 'Function',
    getSupergroup                                  => 'Function',
    getSupergroupFullInfo                          => 'Function',
    getSupergroupMembers                           => 'Function',
    getSupportName                                 => 'Function',
    getSupportUser                                 => 'Function',
    getTemporaryPasswordState                      => 'Function',
    getTextCompositionStyleExample                 => 'Function',
    getTextEntities                                => 'Function',
    getThemeParametersJsonString                   => 'Function',
    getThemedChatEmojiStatuses                     => 'Function',
    getThemedEmojiStatuses                         => 'Function',
    getTimeZones                                   => 'Function',
    getTonTransactions                             => 'Function',
    getTopChats                                    => 'Function',
    getTrendingStickerSets                         => 'Function',
    getUpgradedGift                                => 'Function',
    getUpgradedGiftEmojiStatuses                   => 'Function',
    getUpgradedGiftValueInfo                       => 'Function',
    getUpgradedGiftVariants                        => 'Function',
    getUpgradedGiftWithdrawalUrl                   => 'Function',
    getUpgradedGiftsPromotionalAnimation           => 'Function',
    getUser                                        => 'Function',
    getUserChatBoosts                              => 'Function',
    getUserFullInfo                                => 'Function',
    getUserLink                                    => 'Function',
    getUserPrivacySettingRules                     => 'Function',
    getUserProfileAudios                           => 'Function',
    getUserProfilePhotos                           => 'Function',
    getUserSupportInfo                             => 'Function',
    getVideoChatAvailableParticipants              => 'Function',
    getVideoChatInviteLink                         => 'Function',
    getVideoChatRtmpUrl                            => 'Function',
    getVideoMessageAdvertisements                  => 'Function',
    getWebAppLinkUrl                               => 'Function',
    getWebAppPlaceholder                           => 'Function',
    getWebAppUrl                                   => 'Function',
    getWebPageInstantView                          => 'Function',
    gift                                           => 'Object',
    giftAuction                                    => 'Object',
    giftAuctionAcquiredGift                        => 'Object',
    giftAuctionAcquiredGifts                       => 'Object',
    giftAuctionState                               => 'Object',
    giftBackground                                 => 'Object',
    giftChatTheme                                  => 'Object',
    giftChatThemes                                 => 'Object',
    giftCollection                                 => 'Object',
    giftCollections                                => 'Object',
    giftForResale                                  => 'Object',
    giftForResaleOrderNumber                       => 'GiftForResaleOrder',
    giftForResaleOrderPrice                        => 'GiftForResaleOrder',
    giftForResaleOrderPriceChangeDate              => 'GiftForResaleOrder',
    giftPremiumWithStars                           => 'Function',
    giftPurchaseLimits                             => 'Object',
    giftPurchaseOfferStateAccepted                 => 'GiftPurchaseOfferState',
    giftPurchaseOfferStatePending                  => 'GiftPurchaseOfferState',
    giftPurchaseOfferStateRejected                 => 'GiftPurchaseOfferState',
    giftResaleParameters                           => 'Object',
    giftResalePriceGram                            => 'GiftResalePrice',
    giftResalePriceStar                            => 'GiftResalePrice',
    giftResaleResultOk                             => 'GiftResaleResult',
    giftResaleResultPriceIncreased                 => 'GiftResaleResult',
    giftSettings                                   => 'Object',
    giftUpgradePreview                             => 'Object',
    giftUpgradePrice                               => 'Object',
    giftUpgradeVariants                            => 'Object',
    giftsForCrafting                               => 'Object',
    giftsForResale                                 => 'Object',
    giveawayInfoCompleted                          => 'GiveawayInfo',
    giveawayInfoOngoing                            => 'GiveawayInfo',
    giveawayParameters                             => 'Object',
    giveawayParticipantStatusAdministrator         => 'GiveawayParticipantStatus',
    giveawayParticipantStatusAlreadyWasMember      => 'GiveawayParticipantStatus',
    giveawayParticipantStatusDisallowedCountry     => 'GiveawayParticipantStatus',
    giveawayParticipantStatusEligible              => 'GiveawayParticipantStatus',
    giveawayParticipantStatusParticipating         => 'GiveawayParticipantStatus',
    giveawayPrizePremium                           => 'GiveawayPrize',
    giveawayPrizeStars                             => 'GiveawayPrize',
    gramRevenueStatistics                          => 'Object',
    gramRevenueStatus                              => 'Object',
    groupCall                                      => 'Object',
    groupCallDataChannelMain                       => 'GroupCallDataChannel',
    groupCallDataChannelScreenSharing              => 'GroupCallDataChannel',
    groupCallId                                    => 'Object',
    groupCallInfo                                  => 'Object',
    groupCallJoinParameters                        => 'Object',
    groupCallMessage                               => 'Object',
    groupCallMessageLevel                          => 'Object',
    groupCallParticipant                           => 'Object',
    groupCallParticipantVideoInfo                  => 'Object',
    groupCallParticipants                          => 'Object',
    groupCallRecentSpeaker                         => 'Object',
    groupCallStream                                => 'Object',
    groupCallStreams                               => 'Object',
    groupCallVideoQualityFull                      => 'GroupCallVideoQuality',
    groupCallVideoQualityMedium                    => 'GroupCallVideoQuality',
    groupCallVideoQualityThumbnail                 => 'GroupCallVideoQuality',
    groupCallVideoSourceGroup                      => 'Object',
    hashtags                                       => 'Object',
    hideContactCloseBirthdays                      => 'Function',
    hideSuggestedAction                            => 'Function',
    httpUrl                                        => 'Object',
    identityDocument                               => 'Object',
    importContacts                                 => 'Function',
    importMessages                                 => 'Function',
    importedContact                                => 'Object',
    importedContacts                               => 'Object',
    increaseGiftAuctionBid                         => 'Function',
    inlineKeyboardButton                           => 'Object',
    inlineKeyboardButtonTypeBuy                    => 'InlineKeyboardButtonType',
    inlineKeyboardButtonTypeCallback               => 'InlineKeyboardButtonType',
    inlineKeyboardButtonTypeCallbackGame           => 'InlineKeyboardButtonType',
    inlineKeyboardButtonTypeCallbackWithPassword   => 'InlineKeyboardButtonType',
    inlineKeyboardButtonTypeCopyText               => 'InlineKeyboardButtonType',
    inlineKeyboardButtonTypeLoginUrl               => 'InlineKeyboardButtonType',
    inlineKeyboardButtonTypeSwitchInline           => 'InlineKeyboardButtonType',
    inlineKeyboardButtonTypeUrl                    => 'InlineKeyboardButtonType',
    inlineKeyboardButtonTypeUser                   => 'InlineKeyboardButtonType',
    inlineKeyboardButtonTypeWebApp                 => 'InlineKeyboardButtonType',
    inlineMessageId                                => 'Object',
    inlineQueryResultAnimation                     => 'InlineQueryResult',
    inlineQueryResultArticle                       => 'InlineQueryResult',
    inlineQueryResultAudio                         => 'InlineQueryResult',
    inlineQueryResultContact                       => 'InlineQueryResult',
    inlineQueryResultDocument                      => 'InlineQueryResult',
    inlineQueryResultGame                          => 'InlineQueryResult',
    inlineQueryResultLocation                      => 'InlineQueryResult',
    inlineQueryResultPhoto                         => 'InlineQueryResult',
    inlineQueryResultSticker                       => 'InlineQueryResult',
    inlineQueryResultVenue                         => 'InlineQueryResult',
    inlineQueryResultVideo                         => 'InlineQueryResult',
    inlineQueryResultVoiceNote                     => 'InlineQueryResult',
    inlineQueryResults                             => 'Object',
    inlineQueryResultsButton                       => 'Object',
    inlineQueryResultsButtonTypeStartBot           => 'InlineQueryResultsButtonType',
    inlineQueryResultsButtonTypeWebApp             => 'InlineQueryResultsButtonType',
    inputAnimation                                 => 'Object',
    inputAudio                                     => 'Object',
    inputBackgroundLocal                           => 'InputBackground',
    inputBackgroundPrevious                        => 'InputBackground',
    inputBackgroundRemote                          => 'InputBackground',
    inputBusinessChatLink                          => 'Object',
    inputBusinessStartPage                         => 'Object',
    inputCallDiscarded                             => 'InputCall',
    inputCallFromMessage                           => 'InputCall',
    inputChatPhotoAnimation                        => 'InputChatPhoto',
    inputChatPhotoPrevious                         => 'InputChatPhoto',
    inputChatPhotoStatic                           => 'InputChatPhoto',
    inputChatPhotoSticker                          => 'InputChatPhoto',
    inputChatThemeEmoji                            => 'InputChatTheme',
    inputChatThemeGift                             => 'InputChatTheme',
    inputChecklist                                 => 'Object',
    inputChecklistTask                             => 'Object',
    inputCredentialsApplePay                       => 'InputCredentials',
    inputCredentialsGooglePay                      => 'InputCredentials',
    inputCredentialsNew                            => 'InputCredentials',
    inputCredentialsSaved                          => 'InputCredentials',
    inputDocument                                  => 'Object',
    inputFileGenerated                             => 'InputFile',
    inputFileId                                    => 'InputFile',
    inputFileLocal                                 => 'InputFile',
    inputFileRemote                                => 'InputFile',
    inputGroupCallLink                             => 'InputGroupCall',
    inputGroupCallMessage                          => 'InputGroupCall',
    inputIdentityDocument                          => 'Object',
    inputInlineQueryResultAnimation                => 'InputInlineQueryResult',
    inputInlineQueryResultArticle                  => 'InputInlineQueryResult',
    inputInlineQueryResultAudio                    => 'InputInlineQueryResult',
    inputInlineQueryResultContact                  => 'InputInlineQueryResult',
    inputInlineQueryResultDocument                 => 'InputInlineQueryResult',
    inputInlineQueryResultGame                     => 'InputInlineQueryResult',
    inputInlineQueryResultLocation                 => 'InputInlineQueryResult',
    inputInlineQueryResultPhoto                    => 'InputInlineQueryResult',
    inputInlineQueryResultSticker                  => 'InputInlineQueryResult',
    inputInlineQueryResultVenue                    => 'InputInlineQueryResult',
    inputInlineQueryResultVideo                    => 'InputInlineQueryResult',
    inputInlineQueryResultVoiceNote                => 'InputInlineQueryResult',
    inputInvoiceMessage                            => 'InputInvoice',
    inputInvoiceName                               => 'InputInvoice',
    inputInvoiceTelegram                           => 'InputInvoice',
    inputMessageAnimation                          => 'InputMessageContent',
    inputMessageAudio                              => 'InputMessageContent',
    inputMessageChecklist                          => 'InputMessageContent',
    inputMessageContact                            => 'InputMessageContent',
    inputMessageDice                               => 'InputMessageContent',
    inputMessageDocument                           => 'InputMessageContent',
    inputMessageForwarded                          => 'InputMessageContent',
    inputMessageGame                               => 'InputMessageContent',
    inputMessageInvoice                            => 'InputMessageContent',
    inputMessageLiveLocation                       => 'InputMessageContent',
    inputMessageLocation                           => 'InputMessageContent',
    inputMessagePaidMedia                          => 'InputMessageContent',
    inputMessagePhoto                              => 'InputMessageContent',
    inputMessagePoll                               => 'InputMessageContent',
    inputMessageReplyToEphemeralMessage            => 'InputMessageReplyTo',
    inputMessageReplyToExternalMessage             => 'InputMessageReplyTo',
    inputMessageReplyToMessage                     => 'InputMessageReplyTo',
    inputMessageReplyToStory                       => 'InputMessageReplyTo',
    inputMessageRichMessage                        => 'InputMessageContent',
    inputMessageStakeDice                          => 'InputMessageContent',
    inputMessageSticker                            => 'InputMessageContent',
    inputMessageStory                              => 'InputMessageContent',
    inputMessageText                               => 'InputMessageContent',
    inputMessageVenue                              => 'InputMessageContent',
    inputMessageVideo                              => 'InputMessageContent',
    inputMessageVideoNote                          => 'InputMessageContent',
    inputMessageVoiceNote                          => 'InputMessageContent',
    inputPageBlockAnchor                           => 'InputPageBlock',
    inputPageBlockAnimation                        => 'InputPageBlock',
    inputPageBlockAudio                            => 'InputPageBlock',
    inputPageBlockBlockQuote                       => 'InputPageBlock',
    inputPageBlockCollage                          => 'InputPageBlock',
    inputPageBlockDetails                          => 'InputPageBlock',
    inputPageBlockDivider                          => 'InputPageBlock',
    inputPageBlockFooter                           => 'InputPageBlock',
    inputPageBlockList                             => 'InputPageBlock',
    inputPageBlockListItem                         => 'Object',
    inputPageBlockMap                              => 'InputPageBlock',
    inputPageBlockMathematicalExpression           => 'InputPageBlock',
    inputPageBlockParagraph                        => 'InputPageBlock',
    inputPageBlockPhoto                            => 'InputPageBlock',
    inputPageBlockPreformatted                     => 'InputPageBlock',
    inputPageBlockPullQuote                        => 'InputPageBlock',
    inputPageBlockSectionHeading                   => 'InputPageBlock',
    inputPageBlockSlideshow                        => 'InputPageBlock',
    inputPageBlockTable                            => 'InputPageBlock',
    inputPageBlockThinking                         => 'InputPageBlock',
    inputPageBlockVideo                            => 'InputPageBlock',
    inputPageBlockVoiceNote                        => 'InputPageBlock',
    inputPaidMedia                                 => 'Object',
    inputPaidMediaTypePhoto                        => 'InputPaidMediaType',
    inputPaidMediaTypeVideo                        => 'InputPaidMediaType',
    inputPassportElementAddress                    => 'InputPassportElement',
    inputPassportElementBankStatement              => 'InputPassportElement',
    inputPassportElementDriverLicense              => 'InputPassportElement',
    inputPassportElementEmailAddress               => 'InputPassportElement',
    inputPassportElementError                      => 'Object',
    inputPassportElementErrorSourceDataField       => 'InputPassportElementErrorSource',
    inputPassportElementErrorSourceFile            => 'InputPassportElementErrorSource',
    inputPassportElementErrorSourceFiles           => 'InputPassportElementErrorSource',
    inputPassportElementErrorSourceFrontSide       => 'InputPassportElementErrorSource',
    inputPassportElementErrorSourceReverseSide     => 'InputPassportElementErrorSource',
    inputPassportElementErrorSourceSelfie          => 'InputPassportElementErrorSource',
    inputPassportElementErrorSourceTranslationFile => 'InputPassportElementErrorSource',
    inputPassportElementErrorSourceTranslationFiles => 'InputPassportElementErrorSource',
    inputPassportElementErrorSourceUnspecified     => 'InputPassportElementErrorSource',
    inputPassportElementIdentityCard               => 'InputPassportElement',
    inputPassportElementInternalPassport           => 'InputPassportElement',
    inputPassportElementPassport                   => 'InputPassportElement',
    inputPassportElementPassportRegistration       => 'InputPassportElement',
    inputPassportElementPersonalDetails            => 'InputPassportElement',
    inputPassportElementPhoneNumber                => 'InputPassportElement',
    inputPassportElementRentalAgreement            => 'InputPassportElement',
    inputPassportElementTemporaryRegistration      => 'InputPassportElement',
    inputPassportElementUtilityBill                => 'InputPassportElement',
    inputPersonalDocument                          => 'Object',
    inputPhoto                                     => 'Object',
    inputPollMediaAnimation                        => 'InputPollMedia',
    inputPollMediaAudio                            => 'InputPollMedia',
    inputPollMediaDocument                         => 'InputPollMedia',
    inputPollMediaLink                             => 'InputPollMedia',
    inputPollMediaLocation                         => 'InputPollMedia',
    inputPollMediaPhoto                            => 'InputPollMedia',
    inputPollMediaSticker                          => 'InputPollMedia',
    inputPollMediaVenue                            => 'InputPollMedia',
    inputPollMediaVideo                            => 'InputPollMedia',
    inputPollOption                                => 'Object',
    inputPollTypeQuiz                              => 'InputPollType',
    inputPollTypeRegular                           => 'InputPollType',
    inputRichMessage                               => 'Object',
    inputRichMessageMedia                          => 'Object',
    inputSticker                                   => 'Object',
    inputStoryArea                                 => 'Object',
    inputStoryAreaTypeFoundVenue                   => 'InputStoryAreaType',
    inputStoryAreaTypeLink                         => 'InputStoryAreaType',
    inputStoryAreaTypeLocation                     => 'InputStoryAreaType',
    inputStoryAreaTypeMessage                      => 'InputStoryAreaType',
    inputStoryAreaTypePreviousVenue                => 'InputStoryAreaType',
    inputStoryAreaTypeSuggestedReaction            => 'InputStoryAreaType',
    inputStoryAreaTypeUpgradedGift                 => 'InputStoryAreaType',
    inputStoryAreaTypeWeather                      => 'InputStoryAreaType',
    inputStoryAreas                                => 'Object',
    inputStoryContentPhoto                         => 'InputStoryContent',
    inputStoryContentVideo                         => 'InputStoryContent',
    inputSuggestedPostInfo                         => 'Object',
    inputTextQuote                                 => 'Object',
    inputThumbnail                                 => 'Object',
    inputVideo                                     => 'Object',
    inputVideoNote                                 => 'Object',
    inputVoiceNote                                 => 'Object',
    internalLinkTypeAttachmentMenuBot              => 'InternalLinkType',
    internalLinkTypeAuthenticationCode             => 'InternalLinkType',
    internalLinkTypeBackground                     => 'InternalLinkType',
    internalLinkTypeBotAddToChannel                => 'InternalLinkType',
    internalLinkTypeBotStart                       => 'InternalLinkType',
    internalLinkTypeBotStartInGroup                => 'InternalLinkType',
    internalLinkTypeBusinessChat                   => 'InternalLinkType',
    internalLinkTypeCallsPage                      => 'InternalLinkType',
    internalLinkTypeChatAffiliateProgram           => 'InternalLinkType',
    internalLinkTypeChatBoost                      => 'InternalLinkType',
    internalLinkTypeChatFolderInvite               => 'InternalLinkType',
    internalLinkTypeChatInvite                     => 'InternalLinkType',
    internalLinkTypeChatSelection                  => 'InternalLinkType',
    internalLinkTypeContactsPage                   => 'InternalLinkType',
    internalLinkTypeDirectMessagesChat             => 'InternalLinkType',
    internalLinkTypeGame                           => 'InternalLinkType',
    internalLinkTypeGiftAuction                    => 'InternalLinkType',
    internalLinkTypeGiftCollection                 => 'InternalLinkType',
    internalLinkTypeGroupCall                      => 'InternalLinkType',
    internalLinkTypeInstantView                    => 'InternalLinkType',
    internalLinkTypeInvoice                        => 'InternalLinkType',
    internalLinkTypeLanguagePack                   => 'InternalLinkType',
    internalLinkTypeLiveStory                      => 'InternalLinkType',
    internalLinkTypeMainWebApp                     => 'InternalLinkType',
    internalLinkTypeMessage                        => 'InternalLinkType',
    internalLinkTypeMessageDraft                   => 'InternalLinkType',
    internalLinkTypeMyProfilePage                  => 'InternalLinkType',
    internalLinkTypeNewChannelChat                 => 'InternalLinkType',
    internalLinkTypeNewGroupChat                   => 'InternalLinkType',
    internalLinkTypeNewPrivateChat                 => 'InternalLinkType',
    internalLinkTypeNewStory                       => 'InternalLinkType',
    internalLinkTypeOauth                          => 'InternalLinkType',
    internalLinkTypePassportDataRequest            => 'InternalLinkType',
    internalLinkTypePhoneNumberConfirmation        => 'InternalLinkType',
    internalLinkTypePremiumFeaturesPage            => 'InternalLinkType',
    internalLinkTypePremiumGiftCode                => 'InternalLinkType',
    internalLinkTypePremiumGiftPurchase            => 'InternalLinkType',
    internalLinkTypeProxy                          => 'InternalLinkType',
    internalLinkTypePublicChat                     => 'InternalLinkType',
    internalLinkTypeQrCodeAuthentication           => 'InternalLinkType',
    internalLinkTypeRequestManagedBot              => 'InternalLinkType',
    internalLinkTypeRestorePurchases               => 'InternalLinkType',
    internalLinkTypeSavedMessages                  => 'InternalLinkType',
    internalLinkTypeSearch                         => 'InternalLinkType',
    internalLinkTypeSettings                       => 'InternalLinkType',
    internalLinkTypeStarPurchase                   => 'InternalLinkType',
    internalLinkTypeStickerSet                     => 'InternalLinkType',
    internalLinkTypeStory                          => 'InternalLinkType',
    internalLinkTypeStoryAlbum                     => 'InternalLinkType',
    internalLinkTypeTextCompositionStyle           => 'InternalLinkType',
    internalLinkTypeTheme                          => 'InternalLinkType',
    internalLinkTypeUnknownDeepLink                => 'InternalLinkType',
    internalLinkTypeUpgradedGift                   => 'InternalLinkType',
    internalLinkTypeUserPhoneNumber                => 'InternalLinkType',
    internalLinkTypeUserToken                      => 'InternalLinkType',
    internalLinkTypeVideoChat                      => 'InternalLinkType',
    internalLinkTypeWebApp                         => 'InternalLinkType',
    inviteGroupCallParticipant                     => 'Function',
    inviteGroupCallParticipantResultSuccess        => 'InviteGroupCallParticipantResult',
    inviteGroupCallParticipantResultUserAlreadyParticipant => 'InviteGroupCallParticipantResult',
    inviteGroupCallParticipantResultUserPrivacyRestricted => 'InviteGroupCallParticipantResult',
    inviteGroupCallParticipantResultUserWasBanned  => 'InviteGroupCallParticipantResult',
    inviteLinkChatTypeBasicGroup                   => 'InviteLinkChatType',
    inviteLinkChatTypeChannel                      => 'InviteLinkChatType',
    inviteLinkChatTypeSupergroup                   => 'InviteLinkChatType',
    inviteVideoChatParticipants                    => 'Function',
    invoice                                        => 'Object',
    isLoginEmailAddressRequired                    => 'Function',
    isProfileAudio                                 => 'Function',
    joinChat                                       => 'Function',
    joinChatByInviteLink                           => 'Function',
    joinGroupCall                                  => 'Function',
    joinLiveStory                                  => 'Function',
    joinVideoChat                                  => 'Function',
    jsonObjectMember                               => 'Object',
    jsonValueArray                                 => 'JsonValue',
    jsonValueBoolean                               => 'JsonValue',
    jsonValueNull                                  => 'JsonValue',
    jsonValueNumber                                => 'JsonValue',
    jsonValueObject                                => 'JsonValue',
    jsonValueString                                => 'JsonValue',
    keyboardButton                                 => 'Object',
    keyboardButtonSourceMessage                    => 'KeyboardButtonSource',
    keyboardButtonSourceWebApp                     => 'KeyboardButtonSource',
    keyboardButtonTypeRequestChat                  => 'KeyboardButtonType',
    keyboardButtonTypeRequestLocation              => 'KeyboardButtonType',
    keyboardButtonTypeRequestManagedBot            => 'KeyboardButtonType',
    keyboardButtonTypeRequestPhoneNumber           => 'KeyboardButtonType',
    keyboardButtonTypeRequestPoll                  => 'KeyboardButtonType',
    keyboardButtonTypeRequestUsers                 => 'KeyboardButtonType',
    keyboardButtonTypeText                         => 'KeyboardButtonType',
    keyboardButtonTypeWebApp                       => 'KeyboardButtonType',
    labeledPricePart                               => 'Object',
    languagePackInfo                               => 'Object',
    languagePackString                             => 'Object',
    languagePackStringValueDeleted                 => 'LanguagePackStringValue',
    languagePackStringValueOrdinary                => 'LanguagePackStringValue',
    languagePackStringValuePluralized              => 'LanguagePackStringValue',
    languagePackStrings                            => 'Object',
    launchPrepaidGiveaway                          => 'Function',
    leaveChat                                      => 'Function',
    leaveGroupCall                                 => 'Function',
    linkPreview                                    => 'Object',
    linkPreviewAlbumMediaPhoto                     => 'LinkPreviewAlbumMedia',
    linkPreviewAlbumMediaVideo                     => 'LinkPreviewAlbumMedia',
    linkPreviewOptions                             => 'Object',
    linkPreviewTypeAlbum                           => 'LinkPreviewType',
    linkPreviewTypeAnimation                       => 'LinkPreviewType',
    linkPreviewTypeApp                             => 'LinkPreviewType',
    linkPreviewTypeArticle                         => 'LinkPreviewType',
    linkPreviewTypeAudio                           => 'LinkPreviewType',
    linkPreviewTypeBackground                      => 'LinkPreviewType',
    linkPreviewTypeChannelBoost                    => 'LinkPreviewType',
    linkPreviewTypeChat                            => 'LinkPreviewType',
    linkPreviewTypeDirectMessagesChat              => 'LinkPreviewType',
    linkPreviewTypeDocument                        => 'LinkPreviewType',
    linkPreviewTypeEmbeddedAnimationPlayer         => 'LinkPreviewType',
    linkPreviewTypeEmbeddedAudioPlayer             => 'LinkPreviewType',
    linkPreviewTypeEmbeddedVideoPlayer             => 'LinkPreviewType',
    linkPreviewTypeExternalAudio                   => 'LinkPreviewType',
    linkPreviewTypeExternalVideo                   => 'LinkPreviewType',
    linkPreviewTypeGiftAuction                     => 'LinkPreviewType',
    linkPreviewTypeGiftCollection                  => 'LinkPreviewType',
    linkPreviewTypeGroupCall                       => 'LinkPreviewType',
    linkPreviewTypeInvoice                         => 'LinkPreviewType',
    linkPreviewTypeLiveStory                       => 'LinkPreviewType',
    linkPreviewTypeMessage                         => 'LinkPreviewType',
    linkPreviewTypePhoto                           => 'LinkPreviewType',
    linkPreviewTypePremiumGiftCode                 => 'LinkPreviewType',
    linkPreviewTypeRequestManagedBot               => 'LinkPreviewType',
    linkPreviewTypeShareableChatFolder             => 'LinkPreviewType',
    linkPreviewTypeSticker                         => 'LinkPreviewType',
    linkPreviewTypeStickerSet                      => 'LinkPreviewType',
    linkPreviewTypeStory                           => 'LinkPreviewType',
    linkPreviewTypeStoryAlbum                      => 'LinkPreviewType',
    linkPreviewTypeSupergroupBoost                 => 'LinkPreviewType',
    linkPreviewTypeTextCompositionStyle            => 'LinkPreviewType',
    linkPreviewTypeTheme                           => 'LinkPreviewType',
    linkPreviewTypeUnsupported                     => 'LinkPreviewType',
    linkPreviewTypeUpgradedGift                    => 'LinkPreviewType',
    linkPreviewTypeUser                            => 'LinkPreviewType',
    linkPreviewTypeVideo                           => 'LinkPreviewType',
    linkPreviewTypeVideoChat                       => 'LinkPreviewType',
    linkPreviewTypeVideoNote                       => 'LinkPreviewType',
    linkPreviewTypeVoiceNote                       => 'LinkPreviewType',
    linkPreviewTypeWebApp                          => 'LinkPreviewType',
    listenToAudio                                  => 'Function',
    liveLocation                                   => 'Object',
    liveStoryDonors                                => 'Object',
    loadActiveStories                              => 'Function',
    loadChats                                      => 'Function',
    loadDirectMessagesChatTopics                   => 'Function',
    loadGroupCallParticipants                      => 'Function',
    loadQuickReplyShortcutMessages                 => 'Function',
    loadQuickReplyShortcuts                        => 'Function',
    loadSavedMessagesTopics                        => 'Function',
    localFile                                      => 'Object',
    localizationTargetInfo                         => 'Object',
    location                                       => 'Object',
    locationAddress                                => 'Object',
    logOut                                         => 'Function',
    logStreamDefault                               => 'LogStream',
    logStreamEmpty                                 => 'LogStream',
    logStreamFile                                  => 'LogStream',
    logTags                                        => 'Object',
    logVerbosityLevel                              => 'Object',
    loginUrlInfoOpen                               => 'LoginUrlInfo',
    loginUrlInfoRequestConfirmation                => 'LoginUrlInfo',
    mainWebApp                                     => 'Object',
    markChecklistTasksAsDone                       => 'Function',
    maskPointChin                                  => 'MaskPoint',
    maskPointEyes                                  => 'MaskPoint',
    maskPointForehead                              => 'MaskPoint',
    maskPointMouth                                 => 'MaskPoint',
    maskPosition                                   => 'Object',
    message                                        => 'Object',
    messageAnimatedEmoji                           => 'MessageContent',
    messageAnimation                               => 'MessageContent',
    messageAudio                                   => 'MessageContent',
    messageAutoDeleteTime                          => 'Object',
    messageBasicGroupChatCreate                    => 'MessageContent',
    messageBotWriteAccessAllowed                   => 'MessageContent',
    messageCalendar                                => 'Object',
    messageCalendarDay                             => 'Object',
    messageCall                                    => 'MessageContent',
    messageChatAddMembers                          => 'MessageContent',
    messageChatAddedToCommunity                    => 'MessageContent',
    messageChatBoost                               => 'MessageContent',
    messageChatChangePhoto                         => 'MessageContent',
    messageChatChangeTitle                         => 'MessageContent',
    messageChatDeleteMember                        => 'MessageContent',
    messageChatDeletePhoto                         => 'MessageContent',
    messageChatHasProtectedContentDisableRequested => 'MessageContent',
    messageChatHasProtectedContentToggled          => 'MessageContent',
    messageChatJoinByLink                          => 'MessageContent',
    messageChatJoinByRequest                       => 'MessageContent',
    messageChatOwnerChanged                        => 'MessageContent',
    messageChatOwnerLeft                           => 'MessageContent',
    messageChatRemovedFromCommunity                => 'MessageContent',
    messageChatSetBackground                       => 'MessageContent',
    messageChatSetMessageAutoDeleteTime            => 'MessageContent',
    messageChatSetTheme                            => 'MessageContent',
    messageChatShared                              => 'MessageContent',
    messageChatUpgradeFrom                         => 'MessageContent',
    messageChatUpgradeTo                           => 'MessageContent',
    messageChecklist                               => 'MessageContent',
    messageChecklistTasksAdded                     => 'MessageContent',
    messageChecklistTasksDone                      => 'MessageContent',
    messageContact                                 => 'MessageContent',
    messageContactRegistered                       => 'MessageContent',
    messageCopyOptions                             => 'Object',
    messageCustomServiceAction                     => 'MessageContent',
    messageDice                                    => 'MessageContent',
    messageDirectMessagePriceChanged               => 'MessageContent',
    messageDocument                                => 'MessageContent',
    messageEffect                                  => 'Object',
    messageEffectTypeEmojiReaction                 => 'MessageEffectType',
    messageEffectTypePremiumSticker                => 'MessageEffectType',
    messageExpiredPhoto                            => 'MessageContent',
    messageExpiredVideo                            => 'MessageContent',
    messageExpiredVideoNote                        => 'MessageContent',
    messageExpiredVoiceNote                        => 'MessageContent',
    messageFileTypeGroup                           => 'MessageFileType',
    messageFileTypePrivate                         => 'MessageFileType',
    messageFileTypeUnknown                         => 'MessageFileType',
    messageForumTopicCreated                       => 'MessageContent',
    messageForumTopicEdited                        => 'MessageContent',
    messageForumTopicIsClosedToggled               => 'MessageContent',
    messageForumTopicIsHiddenToggled               => 'MessageContent',
    messageForwardInfo                             => 'Object',
    messageGame                                    => 'MessageContent',
    messageGameScore                               => 'MessageContent',
    messageGift                                    => 'MessageContent',
    messageGiftedPremium                           => 'MessageContent',
    messageGiftedStars                             => 'MessageContent',
    messageGiftedTon                               => 'MessageContent',
    messageGiveaway                                => 'MessageContent',
    messageGiveawayCompleted                       => 'MessageContent',
    messageGiveawayCreated                         => 'MessageContent',
    messageGiveawayPrizeStars                      => 'MessageContent',
    messageGiveawayWinners                         => 'MessageContent',
    messageGroupCall                               => 'MessageContent',
    messageImportInfo                              => 'Object',
    messageInteractionInfo                         => 'Object',
    messageInviteVideoChatParticipants             => 'MessageContent',
    messageInvoice                                 => 'MessageContent',
    messageLink                                    => 'Object',
    messageLinkInfo                                => 'Object',
    messageLiveLocation                            => 'MessageContent',
    messageLocation                                => 'MessageContent',
    messageManagedBotCreated                       => 'MessageContent',
    messageOriginChannel                           => 'MessageOrigin',
    messageOriginChat                              => 'MessageOrigin',
    messageOriginHiddenUser                        => 'MessageOrigin',
    messageOriginUser                              => 'MessageOrigin',
    messagePaidMedia                               => 'MessageContent',
    messagePaidMessagePriceChanged                 => 'MessageContent',
    messagePaidMessagesRefunded                    => 'MessageContent',
    messagePassportDataReceived                    => 'MessageContent',
    messagePassportDataSent                        => 'MessageContent',
    messagePaymentRefunded                         => 'MessageContent',
    messagePaymentSuccessful                       => 'MessageContent',
    messagePaymentSuccessfulBot                    => 'MessageContent',
    messagePhoto                                   => 'MessageContent',
    messagePinMessage                              => 'MessageContent',
    messagePoll                                    => 'MessageContent',
    messagePollOptionAdded                         => 'MessageContent',
    messagePollOptionDeleted                       => 'MessageContent',
    messagePosition                                => 'Object',
    messagePositions                               => 'Object',
    messagePremiumGiftCode                         => 'MessageContent',
    messageProperties                              => 'Object',
    messageProximityAlertTriggered                 => 'MessageContent',
    messageReaction                                => 'Object',
    messageReactions                               => 'Object',
    messageReadDateMyPrivacyRestricted             => 'MessageReadDate',
    messageReadDateRead                            => 'MessageReadDate',
    messageReadDateTooOld                          => 'MessageReadDate',
    messageReadDateUnread                          => 'MessageReadDate',
    messageReadDateUserPrivacyRestricted           => 'MessageReadDate',
    messageRefundedUpgradedGift                    => 'MessageContent',
    messageReplyInfo                               => 'Object',
    messageReplyToMessage                          => 'MessageReplyTo',
    messageReplyToStory                            => 'MessageReplyTo',
    messageRichMessage                             => 'MessageContent',
    messageSchedulingStateSendAtDate               => 'MessageSchedulingState',
    messageSchedulingStateSendWhenOnline           => 'MessageSchedulingState',
    messageSchedulingStateSendWhenVideoProcessed   => 'MessageSchedulingState',
    messageScreenshotTaken                         => 'MessageContent',
    messageSelfDestructTypeImmediately             => 'MessageSelfDestructType',
    messageSelfDestructTypeTimer                   => 'MessageSelfDestructType',
    messageSendOptions                             => 'Object',
    messageSenderChat                              => 'MessageSender',
    messageSenderUser                              => 'MessageSender',
    messageSenders                                 => 'Object',
    messageSendingStateFailed                      => 'MessageSendingState',
    messageSendingStatePending                     => 'MessageSendingState',
    messageSourceChatEventLog                      => 'MessageSource',
    messageSourceChatHistory                       => 'MessageSource',
    messageSourceChatList                          => 'MessageSource',
    messageSourceDirectMessagesChatTopicHistory    => 'MessageSource',
    messageSourceForumTopicHistory                 => 'MessageSource',
    messageSourceHistoryPreview                    => 'MessageSource',
    messageSourceMessageThreadHistory              => 'MessageSource',
    messageSourceNotification                      => 'MessageSource',
    messageSourceOther                             => 'MessageSource',
    messageSourceScreenshot                        => 'MessageSource',
    messageSourceSearch                            => 'MessageSource',
    messageStakeDice                               => 'MessageContent',
    messageStatistics                              => 'Object',
    messageSticker                                 => 'MessageContent',
    messageStory                                   => 'MessageContent',
    messageSuggestBirthdate                        => 'MessageContent',
    messageSuggestProfilePhoto                     => 'MessageContent',
    messageSuggestedPostApprovalFailed             => 'MessageContent',
    messageSuggestedPostApproved                   => 'MessageContent',
    messageSuggestedPostDeclined                   => 'MessageContent',
    messageSuggestedPostPaid                       => 'MessageContent',
    messageSuggestedPostRefunded                   => 'MessageContent',
    messageSupergroupChatCreate                    => 'MessageContent',
    messageText                                    => 'MessageContent',
    messageThreadInfo                              => 'Object',
    messageTopicDirectMessages                     => 'MessageTopic',
    messageTopicForum                              => 'MessageTopic',
    messageTopicSavedMessages                      => 'MessageTopic',
    messageTopicThread                             => 'MessageTopic',
    messageUnsupported                             => 'MessageContent',
    messageUpgradedGift                            => 'MessageContent',
    messageUpgradedGiftPurchaseOffer               => 'MessageContent',
    messageUpgradedGiftPurchaseOfferRejected       => 'MessageContent',
    messageUsersShared                             => 'MessageContent',
    messageVenue                                   => 'MessageContent',
    messageVideo                                   => 'MessageContent',
    messageVideoChatEnded                          => 'MessageContent',
    messageVideoChatScheduled                      => 'MessageContent',
    messageVideoChatStarted                        => 'MessageContent',
    messageVideoNote                               => 'MessageContent',
    messageViewer                                  => 'Object',
    messageViewers                                 => 'Object',
    messageVoiceNote                               => 'MessageContent',
    messageWebAppDataReceived                      => 'MessageContent',
    messageWebAppDataSent                          => 'MessageContent',
    messages                                       => 'Object',
    minithumbnail                                  => 'Object',
    networkStatistics                              => 'Object',
    networkStatisticsEntryCall                     => 'NetworkStatisticsEntry',
    networkStatisticsEntryFile                     => 'NetworkStatisticsEntry',
    networkTypeMobile                              => 'NetworkType',
    networkTypeMobileRoaming                       => 'NetworkType',
    networkTypeNone                                => 'NetworkType',
    networkTypeOther                               => 'NetworkType',
    networkTypeWiFi                                => 'NetworkType',
    newChatPrivacySettings                         => 'Object',
    newSticker                                     => 'Object',
    notification                                   => 'Object',
    notificationGroup                              => 'Object',
    notificationGroupTypeCalls                     => 'NotificationGroupType',
    notificationGroupTypeMentions                  => 'NotificationGroupType',
    notificationGroupTypeMessages                  => 'NotificationGroupType',
    notificationGroupTypeSecretChat                => 'NotificationGroupType',
    notificationSettingsScopeChannelChats          => 'NotificationSettingsScope',
    notificationSettingsScopeGroupChats            => 'NotificationSettingsScope',
    notificationSettingsScopePrivateChats          => 'NotificationSettingsScope',
    notificationSound                              => 'Object',
    notificationSounds                             => 'Object',
    notificationTypeNewCall                        => 'NotificationType',
    notificationTypeNewMessage                     => 'NotificationType',
    notificationTypeNewPushMessage                 => 'NotificationType',
    notificationTypeNewSecretChat                  => 'NotificationType',
    oauthLinkInfo                                  => 'Object',
    ok                                             => 'Object',
    openBotSimilarBot                              => 'Function',
    openChat                                       => 'Function',
    openChatSimilarChat                            => 'Function',
    openGiftAuction                                => 'Function',
    openMessageContent                             => 'Function',
    openSponsoredChat                              => 'Function',
    openStory                                      => 'Function',
    openWebApp                                     => 'Function',
    optimizeStorage                                => 'Function',
    optionValueBoolean                             => 'OptionValue',
    optionValueEmpty                               => 'OptionValue',
    optionValueInteger                             => 'OptionValue',
    optionValueString                              => 'OptionValue',
    orderInfo                                      => 'Object',
    outline                                        => 'Object',
    pageBlockAnchor                                => 'PageBlock',
    pageBlockAnimation                             => 'PageBlock',
    pageBlockAudio                                 => 'PageBlock',
    pageBlockAuthorDate                            => 'PageBlock',
    pageBlockBlockQuote                            => 'PageBlock',
    pageBlockCaption                               => 'Object',
    pageBlockChatLink                              => 'PageBlock',
    pageBlockCollage                               => 'PageBlock',
    pageBlockCover                                 => 'PageBlock',
    pageBlockDetails                               => 'PageBlock',
    pageBlockDivider                               => 'PageBlock',
    pageBlockEmbedded                              => 'PageBlock',
    pageBlockEmbeddedPost                          => 'PageBlock',
    pageBlockFooter                                => 'PageBlock',
    pageBlockHeader                                => 'PageBlock',
    pageBlockHorizontalAlignmentCenter             => 'PageBlockHorizontalAlignment',
    pageBlockHorizontalAlignmentLeft               => 'PageBlockHorizontalAlignment',
    pageBlockHorizontalAlignmentRight              => 'PageBlockHorizontalAlignment',
    pageBlockKicker                                => 'PageBlock',
    pageBlockList                                  => 'PageBlock',
    pageBlockListItem                              => 'Object',
    pageBlockMap                                   => 'PageBlock',
    pageBlockMathematicalExpression                => 'PageBlock',
    pageBlockParagraph                             => 'PageBlock',
    pageBlockPhoto                                 => 'PageBlock',
    pageBlockPreformatted                          => 'PageBlock',
    pageBlockPullQuote                             => 'PageBlock',
    pageBlockRelatedArticle                        => 'Object',
    pageBlockRelatedArticles                       => 'PageBlock',
    pageBlockSectionHeading                        => 'PageBlock',
    pageBlockSlideshow                             => 'PageBlock',
    pageBlockSubheader                             => 'PageBlock',
    pageBlockSubtitle                              => 'PageBlock',
    pageBlockTable                                 => 'PageBlock',
    pageBlockTableCell                             => 'Object',
    pageBlockThinking                              => 'PageBlock',
    pageBlockTitle                                 => 'PageBlock',
    pageBlockVerticalAlignmentBottom               => 'PageBlockVerticalAlignment',
    pageBlockVerticalAlignmentMiddle               => 'PageBlockVerticalAlignment',
    pageBlockVerticalAlignmentTop                  => 'PageBlockVerticalAlignment',
    pageBlockVideo                                 => 'PageBlock',
    pageBlockVoiceNote                             => 'PageBlock',
    paidMediaPhoto                                 => 'PaidMedia',
    paidMediaPreview                               => 'PaidMedia',
    paidMediaUnsupported                           => 'PaidMedia',
    paidMediaVideo                                 => 'PaidMedia',
    paidReactionTypeAnonymous                      => 'PaidReactionType',
    paidReactionTypeChat                           => 'PaidReactionType',
    paidReactionTypeRegular                        => 'PaidReactionType',
    paidReactor                                    => 'Object',
    parseMarkdown                                  => 'Function',
    parseTextEntities                              => 'Function',
    passkey                                        => 'Object',
    passkeys                                       => 'Object',
    passportAuthorizationForm                      => 'Object',
    passportElementAddress                         => 'PassportElement',
    passportElementBankStatement                   => 'PassportElement',
    passportElementDriverLicense                   => 'PassportElement',
    passportElementEmailAddress                    => 'PassportElement',
    passportElementError                           => 'Object',
    passportElementErrorSourceDataField            => 'PassportElementErrorSource',
    passportElementErrorSourceFile                 => 'PassportElementErrorSource',
    passportElementErrorSourceFiles                => 'PassportElementErrorSource',
    passportElementErrorSourceFrontSide            => 'PassportElementErrorSource',
    passportElementErrorSourceReverseSide          => 'PassportElementErrorSource',
    passportElementErrorSourceSelfie               => 'PassportElementErrorSource',
    passportElementErrorSourceTranslationFile      => 'PassportElementErrorSource',
    passportElementErrorSourceTranslationFiles     => 'PassportElementErrorSource',
    passportElementErrorSourceUnspecified          => 'PassportElementErrorSource',
    passportElementIdentityCard                    => 'PassportElement',
    passportElementInternalPassport                => 'PassportElement',
    passportElementPassport                        => 'PassportElement',
    passportElementPassportRegistration            => 'PassportElement',
    passportElementPersonalDetails                 => 'PassportElement',
    passportElementPhoneNumber                     => 'PassportElement',
    passportElementRentalAgreement                 => 'PassportElement',
    passportElementTemporaryRegistration           => 'PassportElement',
    passportElementTypeAddress                     => 'PassportElementType',
    passportElementTypeBankStatement               => 'PassportElementType',
    passportElementTypeDriverLicense               => 'PassportElementType',
    passportElementTypeEmailAddress                => 'PassportElementType',
    passportElementTypeIdentityCard                => 'PassportElementType',
    passportElementTypeInternalPassport            => 'PassportElementType',
    passportElementTypePassport                    => 'PassportElementType',
    passportElementTypePassportRegistration        => 'PassportElementType',
    passportElementTypePersonalDetails             => 'PassportElementType',
    passportElementTypePhoneNumber                 => 'PassportElementType',
    passportElementTypeRentalAgreement             => 'PassportElementType',
    passportElementTypeTemporaryRegistration       => 'PassportElementType',
    passportElementTypeUtilityBill                 => 'PassportElementType',
    passportElementUtilityBill                     => 'PassportElement',
    passportElements                               => 'Object',
    passportElementsWithErrors                     => 'Object',
    passportRequiredElement                        => 'Object',
    passportSuitableElement                        => 'Object',
    passwordState                                  => 'Object',
    paymentForm                                    => 'Object',
    paymentFormTypeRegular                         => 'PaymentFormType',
    paymentFormTypeStarSubscription                => 'PaymentFormType',
    paymentFormTypeStars                           => 'PaymentFormType',
    paymentOption                                  => 'Object',
    paymentProviderOther                           => 'PaymentProvider',
    paymentProviderSmartGlocal                     => 'PaymentProvider',
    paymentProviderStripe                          => 'PaymentProvider',
    paymentReceipt                                 => 'Object',
    paymentReceiptTypeRegular                      => 'PaymentReceiptType',
    paymentReceiptTypeStars                        => 'PaymentReceiptType',
    paymentResult                                  => 'Object',
    personalDetails                                => 'Object',
    personalDocument                               => 'Object',
    phoneNumberAuthenticationSettings              => 'Object',
    phoneNumberCodeTypeChange                      => 'PhoneNumberCodeType',
    phoneNumberCodeTypeConfirmOwnership            => 'PhoneNumberCodeType',
    phoneNumberCodeTypeVerify                      => 'PhoneNumberCodeType',
    phoneNumberInfo                                => 'Object',
    photo                                          => 'Object',
    photoSize                                      => 'Object',
    pinChatMessage                                 => 'Function',
    pingProxy                                      => 'Function',
    placeGiftAuctionBid                            => 'Function',
    point                                          => 'Object',
    poll                                           => 'Object',
    pollMediaAnimation                             => 'PollMedia',
    pollMediaAudio                                 => 'PollMedia',
    pollMediaDocument                              => 'PollMedia',
    pollMediaLink                                  => 'PollMedia',
    pollMediaLocation                              => 'PollMedia',
    pollMediaPhoto                                 => 'PollMedia',
    pollMediaSticker                               => 'PollMedia',
    pollMediaVenue                                 => 'PollMedia',
    pollMediaVideo                                 => 'PollMedia',
    pollOption                                     => 'Object',
    pollOptionProperties                           => 'Object',
    pollTypeQuiz                                   => 'PollType',
    pollTypeRegular                                => 'PollType',
    pollVoteRestrictionReasonClosed                => 'PollVoteRestrictionReason',
    pollVoteRestrictionReasonCountryRestricted     => 'PollVoteRestrictionReason',
    pollVoteRestrictionReasonMembershipRequired    => 'PollVoteRestrictionReason',
    pollVoteRestrictionReasonOther                 => 'PollVoteRestrictionReason',
    pollVoteRestrictionReasonScheduled             => 'PollVoteRestrictionReason',
    pollVoteRestrictionReasonYetUnsent             => 'PollVoteRestrictionReason',
    pollVoteStatistics                             => 'Object',
    pollVoter                                      => 'Object',
    pollVoters                                     => 'Object',
    postStory                                      => 'Function',
    preliminaryUploadFile                          => 'Function',
    premiumFeatureAccentColor                      => 'PremiumFeature',
    premiumFeatureAdvancedChatManagement           => 'PremiumFeature',
    premiumFeatureAnimatedProfilePhoto             => 'PremiumFeature',
    premiumFeatureAppIcons                         => 'PremiumFeature',
    premiumFeatureBackgroundForBoth                => 'PremiumFeature',
    premiumFeatureBusiness                         => 'PremiumFeature',
    premiumFeatureChatBoost                        => 'PremiumFeature',
    premiumFeatureChecklists                       => 'PremiumFeature',
    premiumFeatureCustomEmoji                      => 'PremiumFeature',
    premiumFeatureDisabledAds                      => 'PremiumFeature',
    premiumFeatureEmojiStatus                      => 'PremiumFeature',
    premiumFeatureForumTopicIcon                   => 'PremiumFeature',
    premiumFeatureImprovedDownloadSpeed            => 'PremiumFeature',
    premiumFeatureIncreasedLimits                  => 'PremiumFeature',
    premiumFeatureIncreasedUploadFileSize          => 'PremiumFeature',
    premiumFeatureLastSeenTimes                    => 'PremiumFeature',
    premiumFeatureMessageEffects                   => 'PremiumFeature',
    premiumFeatureMessagePrivacy                   => 'PremiumFeature',
    premiumFeaturePaidMessages                     => 'PremiumFeature',
    premiumFeatureProfileBadge                     => 'PremiumFeature',
    premiumFeaturePromotionAnimation               => 'Object',
    premiumFeatureProtectPrivateChatContent        => 'PremiumFeature',
    premiumFeatureRealTimeChatTranslation          => 'PremiumFeature',
    premiumFeatureRichMessages                     => 'PremiumFeature',
    premiumFeatureSavedMessagesTags                => 'PremiumFeature',
    premiumFeatureTextComposition                  => 'PremiumFeature',
    premiumFeatureUniqueReactions                  => 'PremiumFeature',
    premiumFeatureUniqueStickers                   => 'PremiumFeature',
    premiumFeatureUpgradedStories                  => 'PremiumFeature',
    premiumFeatureVoiceRecognition                 => 'PremiumFeature',
    premiumFeatures                                => 'Object',
    premiumGiftCodeInfo                            => 'Object',
    premiumGiftPaymentOption                       => 'Object',
    premiumGiftPaymentOptions                      => 'Object',
    premiumGiveawayPaymentOption                   => 'Object',
    premiumGiveawayPaymentOptions                  => 'Object',
    premiumLimit                                   => 'Object',
    premiumLimitTypeActiveStoryCount               => 'PremiumLimitType',
    premiumLimitTypeBioLength                      => 'PremiumLimitType',
    premiumLimitTypeCaptionLength                  => 'PremiumLimitType',
    premiumLimitTypeChatFolderChosenChatCount      => 'PremiumLimitType',
    premiumLimitTypeChatFolderCount                => 'PremiumLimitType',
    premiumLimitTypeChatFolderInviteLinkCount      => 'PremiumLimitType',
    premiumLimitTypeCreatedPublicChatCount         => 'PremiumLimitType',
    premiumLimitTypeCustomTextCompositionStyleCount => 'PremiumLimitType',
    premiumLimitTypeFavoriteStickerCount           => 'PremiumLimitType',
    premiumLimitTypeMessageTextLength              => 'PremiumLimitType',
    premiumLimitTypeMonthlyPostedStoryCount        => 'PremiumLimitType',
    premiumLimitTypeOwnedBotCount                  => 'PremiumLimitType',
    premiumLimitTypePinnedArchivedChatCount        => 'PremiumLimitType',
    premiumLimitTypePinnedChatCount                => 'PremiumLimitType',
    premiumLimitTypePinnedSavedMessagesTopicCount  => 'PremiumLimitType',
    premiumLimitTypeSavedAnimationCount            => 'PremiumLimitType',
    premiumLimitTypeShareableChatFolderCount       => 'PremiumLimitType',
    premiumLimitTypeSimilarChatCount               => 'PremiumLimitType',
    premiumLimitTypeStoryCaptionLength             => 'PremiumLimitType',
    premiumLimitTypeStorySuggestedReactionAreaCount => 'PremiumLimitType',
    premiumLimitTypeSupergroupCount                => 'PremiumLimitType',
    premiumLimitTypeWeeklyPostedStoryCount         => 'PremiumLimitType',
    premiumPaymentOption                           => 'Object',
    premiumSourceBusinessFeature                   => 'PremiumSource',
    premiumSourceFeature                           => 'PremiumSource',
    premiumSourceLimitExceeded                     => 'PremiumSource',
    premiumSourceLink                              => 'PremiumSource',
    premiumSourceSettings                          => 'PremiumSource',
    premiumSourceStoryFeature                      => 'PremiumSource',
    premiumState                                   => 'Object',
    premiumStatePaymentOption                      => 'Object',
    premiumStoryFeatureCustomExpirationDuration    => 'PremiumStoryFeature',
    premiumStoryFeatureLinksAndFormatting          => 'PremiumStoryFeature',
    premiumStoryFeaturePermanentViewsHistory       => 'PremiumStoryFeature',
    premiumStoryFeaturePriorityOrder               => 'PremiumStoryFeature',
    premiumStoryFeatureSaveStories                 => 'PremiumStoryFeature',
    premiumStoryFeatureStealthMode                 => 'PremiumStoryFeature',
    premiumStoryFeatureVideoQuality                => 'PremiumStoryFeature',
    prepaidGiveaway                                => 'Object',
    preparedInlineMessage                          => 'Object',
    preparedInlineMessageId                        => 'Object',
    processChatFolderNewChats                      => 'Function',
    processChatHasProtectedContentDisableRequest   => 'Function',
    processChatJoinRequest                         => 'Function',
    processChatJoinRequests                        => 'Function',
    processGiftPurchaseOffer                       => 'Function',
    processPushNotification                        => 'Function',
    productInfo                                    => 'Object',
    profileAccentColor                             => 'Object',
    profileAccentColors                            => 'Object',
    profilePhoto                                   => 'Object',
    profileTabFiles                                => 'ProfileTab',
    profileTabGifs                                 => 'ProfileTab',
    profileTabGifts                                => 'ProfileTab',
    profileTabLinks                                => 'ProfileTab',
    profileTabMedia                                => 'ProfileTab',
    profileTabMusic                                => 'ProfileTab',
    profileTabPosts                                => 'ProfileTab',
    profileTabVoice                                => 'ProfileTab',
    proxy                                          => 'Object',
    proxyTypeHttp                                  => 'ProxyType',
    proxyTypeMtproto                               => 'ProxyType',
    proxyTypeSocks5                                => 'ProxyType',
    publicChatTypeHasUsername                      => 'PublicChatType',
    publicChatTypeIsLocationBased                  => 'PublicChatType',
    publicForwardMessage                           => 'PublicForward',
    publicForwardStory                             => 'PublicForward',
    publicForwards                                 => 'Object',
    publicPostSearchLimits                         => 'Object',
    pushMessageContentAnimation                    => 'PushMessageContent',
    pushMessageContentAudio                        => 'PushMessageContent',
    pushMessageContentBasicGroupChatCreate         => 'PushMessageContent',
    pushMessageContentChatAddMembers               => 'PushMessageContent',
    pushMessageContentChatChangePhoto              => 'PushMessageContent',
    pushMessageContentChatChangeTitle              => 'PushMessageContent',
    pushMessageContentChatDeleteMember             => 'PushMessageContent',
    pushMessageContentChatJoinByLink               => 'PushMessageContent',
    pushMessageContentChatJoinByRequest            => 'PushMessageContent',
    pushMessageContentChatSetBackground            => 'PushMessageContent',
    pushMessageContentChatSetTheme                 => 'PushMessageContent',
    pushMessageContentChecklist                    => 'PushMessageContent',
    pushMessageContentChecklistTasksAdded          => 'PushMessageContent',
    pushMessageContentChecklistTasksDone           => 'PushMessageContent',
    pushMessageContentContact                      => 'PushMessageContent',
    pushMessageContentContactRegistered            => 'PushMessageContent',
    pushMessageContentDocument                     => 'PushMessageContent',
    pushMessageContentGame                         => 'PushMessageContent',
    pushMessageContentGameScore                    => 'PushMessageContent',
    pushMessageContentGift                         => 'PushMessageContent',
    pushMessageContentGiveaway                     => 'PushMessageContent',
    pushMessageContentHidden                       => 'PushMessageContent',
    pushMessageContentInviteVideoChatParticipants  => 'PushMessageContent',
    pushMessageContentInvoice                      => 'PushMessageContent',
    pushMessageContentLocation                     => 'PushMessageContent',
    pushMessageContentMediaAlbum                   => 'PushMessageContent',
    pushMessageContentMessageForwards              => 'PushMessageContent',
    pushMessageContentPaidMedia                    => 'PushMessageContent',
    pushMessageContentPhoto                        => 'PushMessageContent',
    pushMessageContentPoll                         => 'PushMessageContent',
    pushMessageContentPollOptionAdded              => 'PushMessageContent',
    pushMessageContentPremiumGiftCode              => 'PushMessageContent',
    pushMessageContentProximityAlertTriggered      => 'PushMessageContent',
    pushMessageContentRecurringPayment             => 'PushMessageContent',
    pushMessageContentScreenshotTaken              => 'PushMessageContent',
    pushMessageContentSticker                      => 'PushMessageContent',
    pushMessageContentStory                        => 'PushMessageContent',
    pushMessageContentSuggestBirthdate             => 'PushMessageContent',
    pushMessageContentSuggestProfilePhoto          => 'PushMessageContent',
    pushMessageContentText                         => 'PushMessageContent',
    pushMessageContentUpgradedGift                 => 'PushMessageContent',
    pushMessageContentVideo                        => 'PushMessageContent',
    pushMessageContentVideoChatEnded               => 'PushMessageContent',
    pushMessageContentVideoChatStarted             => 'PushMessageContent',
    pushMessageContentVideoNote                    => 'PushMessageContent',
    pushMessageContentVoiceNote                    => 'PushMessageContent',
    pushReceiverId                                 => 'Object',
    quickReplyMessage                              => 'Object',
    quickReplyMessages                             => 'Object',
    quickReplyShortcut                             => 'Object',
    rateSpeechRecognition                          => 'Function',
    reactionNotificationSettings                   => 'Object',
    reactionNotificationSourceAll                  => 'ReactionNotificationSource',
    reactionNotificationSourceContacts             => 'ReactionNotificationSource',
    reactionNotificationSourceNone                 => 'ReactionNotificationSource',
    reactionTypeCustomEmoji                        => 'ReactionType',
    reactionTypeEmoji                              => 'ReactionType',
    reactionTypePaid                               => 'ReactionType',
    reactionUnavailabilityReasonAnonymousAdministrator => 'ReactionUnavailabilityReason',
    reactionUnavailabilityReasonGuest              => 'ReactionUnavailabilityReason',
    reactionUnavailabilityReasonRestricted         => 'ReactionUnavailabilityReason',
    readAllChatMentions                            => 'Function',
    readAllChatPollVotes                           => 'Function',
    readAllChatReactions                           => 'Function',
    readAllDirectMessagesChatTopicReactions        => 'Function',
    readAllForumTopicMentions                      => 'Function',
    readAllForumTopicPollVotes                     => 'Function',
    readAllForumTopicReactions                     => 'Function',
    readBusinessMessage                            => 'Function',
    readChatList                                   => 'Function',
    readDatePrivacySettings                        => 'Object',
    readFilePart                                   => 'Function',
    readdQuickReplyShortcutMessages                => 'Function',
    receivedGift                                   => 'Object',
    receivedGifts                                  => 'Object',
    recognizeSpeech                                => 'Function',
    recommendedChatFolder                          => 'Object',
    recommendedChatFolders                         => 'Object',
    recoverAuthenticationPassword                  => 'Function',
    recoverPassword                                => 'Function',
    recoveryEmailAddress                           => 'Object',
    refundStarPayment                              => 'Function',
    registerDevice                                 => 'Function',
    registerUser                                   => 'Function',
    remoteFile                                     => 'Object',
    removeAllFilesFromDownloads                    => 'Function',
    removeAllWebBrowserSettingsExceptions          => 'Function',
    removeBusinessConnectedBotFromChat             => 'Function',
    removeChatActionBar                            => 'Function',
    removeContacts                                 => 'Function',
    removeFavoriteSticker                          => 'Function',
    removeFileFromDownloads                        => 'Function',
    removeGiftCollectionGifts                      => 'Function',
    removeInstalledBackground                      => 'Function',
    removeLoginPasskey                             => 'Function',
    removeMessageReaction                          => 'Function',
    removeMessageSenderBotVerification             => 'Function',
    removeNotification                             => 'Function',
    removeNotificationGroup                        => 'Function',
    removePendingLiveStoryReactions                => 'Function',
    removePendingPaidMessageReactions              => 'Function',
    removeProfileAudio                             => 'Function',
    removeProxy                                    => 'Function',
    removeRecentHashtag                            => 'Function',
    removeRecentSticker                            => 'Function',
    removeRecentlyFoundChat                        => 'Function',
    removeSavedAnimation                           => 'Function',
    removeSavedNotificationSound                   => 'Function',
    removeSearchedForTag                           => 'Function',
    removeStickerFromSet                           => 'Function',
    removeStoryAlbumStories                        => 'Function',
    removeTextCompositionStyle                     => 'Function',
    removeTopChat                                  => 'Function',
    removeWebBrowserSettingsException              => 'Function',
    reorderActiveUsernames                         => 'Function',
    reorderBotActiveUsernames                      => 'Function',
    reorderBotMediaPreviews                        => 'Function',
    reorderChatFolders                             => 'Function',
    reorderGiftCollectionGifts                     => 'Function',
    reorderGiftCollections                         => 'Function',
    reorderInstalledStickerSets                    => 'Function',
    reorderQuickReplyShortcuts                     => 'Function',
    reorderStoryAlbumStories                       => 'Function',
    reorderStoryAlbums                             => 'Function',
    reorderSupergroupActiveUsernames               => 'Function',
    replaceLiveStoryRtmpUrl                        => 'Function',
    replacePrimaryChatInviteLink                   => 'Function',
    replaceStickerInSet                            => 'Function',
    replaceVideoChatRtmpUrl                        => 'Function',
    replyMarkupForceReply                          => 'ReplyMarkup',
    replyMarkupInlineKeyboard                      => 'ReplyMarkup',
    replyMarkupRemoveKeyboard                      => 'ReplyMarkup',
    replyMarkupShowKeyboard                        => 'ReplyMarkup',
    reportAuthenticationCodeMissing                => 'Function',
    reportChat                                     => 'Function',
    reportChatPhoto                                => 'Function',
    reportChatResultMessagesRequired               => 'ReportChatResult',
    reportChatResultOk                             => 'ReportChatResult',
    reportChatResultOptionRequired                 => 'ReportChatResult',
    reportChatResultTextRequired                   => 'ReportChatResult',
    reportChatSponsoredMessage                     => 'Function',
    reportMessageReactions                         => 'Function',
    reportOption                                   => 'Object',
    reportPhoneNumberCodeMissing                   => 'Function',
    reportReasonChildAbuse                         => 'ReportReason',
    reportReasonCopyright                          => 'ReportReason',
    reportReasonCustom                             => 'ReportReason',
    reportReasonFake                               => 'ReportReason',
    reportReasonIllegalDrugs                       => 'ReportReason',
    reportReasonPersonalDetails                    => 'ReportReason',
    reportReasonPornography                        => 'ReportReason',
    reportReasonSpam                               => 'ReportReason',
    reportReasonUnrelatedLocation                  => 'ReportReason',
    reportReasonViolence                           => 'ReportReason',
    reportSponsoredChat                            => 'Function',
    reportSponsoredResultAdsHidden                 => 'ReportSponsoredResult',
    reportSponsoredResultFailed                    => 'ReportSponsoredResult',
    reportSponsoredResultOk                        => 'ReportSponsoredResult',
    reportSponsoredResultOptionRequired            => 'ReportSponsoredResult',
    reportSponsoredResultPremiumRequired           => 'ReportSponsoredResult',
    reportStory                                    => 'Function',
    reportStoryResultOk                            => 'ReportStoryResult',
    reportStoryResultOptionRequired                => 'ReportStoryResult',
    reportStoryResultTextRequired                  => 'ReportStoryResult',
    reportSupergroupAntiSpamFalsePositive          => 'Function',
    reportSupergroupSpam                           => 'Function',
    reportVideoMessageAdvertisement                => 'Function',
    requestAuthenticationPasswordRecovery          => 'Function',
    requestPasswordRecovery                        => 'Function',
    requestQrCodeAuthentication                    => 'Function',
    resendAuthenticationCode                       => 'Function',
    resendCodeReasonUserRequest                    => 'ResendCodeReason',
    resendCodeReasonVerificationFailed             => 'ResendCodeReason',
    resendEmailAddressVerificationCode             => 'Function',
    resendLoginEmailAddressCode                    => 'Function',
    resendMessages                                 => 'Function',
    resendPhoneNumberCode                          => 'Function',
    resendRecoveryEmailAddressCode                 => 'Function',
    resetAllNotificationSettings                   => 'Function',
    resetAuthenticationEmailAddress                => 'Function',
    resetInstalledBackgrounds                      => 'Function',
    resetNetworkStatistics                         => 'Function',
    resetPassword                                  => 'Function',
    resetPasswordResultDeclined                    => 'ResetPasswordResult',
    resetPasswordResultOk                          => 'ResetPasswordResult',
    resetPasswordResultPending                     => 'ResetPasswordResult',
    restrictionInfo                                => 'Object',
    reuseStarSubscription                          => 'Function',
    revenueWithdrawalStateFailed                   => 'RevenueWithdrawalState',
    revenueWithdrawalStatePending                  => 'RevenueWithdrawalState',
    revenueWithdrawalStateSucceeded                => 'RevenueWithdrawalState',
    revokeChatInviteLink                           => 'Function',
    revokeGroupCallInviteLink                      => 'Function',
    richMessage                                    => 'Object',
    richMessageSourceBlocks                        => 'RichMessageSource',
    richMessageSourceHtml                          => 'RichMessageSource',
    richMessageSourceMarkdown                      => 'RichMessageSource',
    richTextAnchor                                 => 'RichText',
    richTextAnchorLink                             => 'RichText',
    richTextBankCardNumber                         => 'RichText',
    richTextBold                                   => 'RichText',
    richTextBotCommand                             => 'RichText',
    richTextCashtag                                => 'RichText',
    richTextCustomEmoji                            => 'RichText',
    richTextDateTime                               => 'RichText',
    richTextDiff                                   => 'RichText',
    richTextEmailAddress                           => 'RichText',
    richTextFixed                                  => 'RichText',
    richTextHashtag                                => 'RichText',
    richTextIcon                                   => 'RichText',
    richTextItalic                                 => 'RichText',
    richTextMarked                                 => 'RichText',
    richTextMathematicalExpression                 => 'RichText',
    richTextMention                                => 'RichText',
    richTextMentionName                            => 'RichText',
    richTextPhoneNumber                            => 'RichText',
    richTextPlain                                  => 'RichText',
    richTextReference                              => 'RichText',
    richTextReferenceLink                          => 'RichText',
    richTextSpoiler                                => 'RichText',
    richTextStrikethrough                          => 'RichText',
    richTextSubscript                              => 'RichText',
    richTextSuperscript                            => 'RichText',
    richTextUnderline                              => 'RichText',
    richTextUrl                                    => 'RichText',
    richTexts                                      => 'RichText',
    rtmpUrl                                        => 'Object',
    saveApplicationLogEvent                        => 'Function',
    savePreparedInlineMessage                      => 'Function',
    savePreparedKeyboardButton                     => 'Function',
    savedCredentials                               => 'Object',
    savedMessagesTag                               => 'Object',
    savedMessagesTags                              => 'Object',
    savedMessagesTopic                             => 'Object',
    savedMessagesTopicTypeAuthorHidden             => 'SavedMessagesTopicType',
    savedMessagesTopicTypeMyNotes                  => 'SavedMessagesTopicType',
    savedMessagesTopicTypeSavedFromChat            => 'SavedMessagesTopicType',
    scopeAutosaveSettings                          => 'Object',
    scopeNotificationSettings                      => 'Object',
    searchAffiliatePrograms                        => 'Function',
    searchBackground                               => 'Function',
    searchCallMessages                             => 'Function',
    searchChatAffiliateProgram                     => 'Function',
    searchChatMembers                              => 'Function',
    searchChatMessages                             => 'Function',
    searchChatRecentLocationMessages               => 'Function',
    searchChatTypeFilterBot                        => 'SearchChatTypeFilter',
    searchChatTypeFilterChannel                    => 'SearchChatTypeFilter',
    searchChats                                    => 'Function',
    searchChatsOnServer                            => 'Function',
    searchContacts                                 => 'Function',
    searchEmojis                                   => 'Function',
    searchFileDownloads                            => 'Function',
    searchGiftsForResale                           => 'Function',
    searchHashtags                                 => 'Function',
    searchInstalledStickerSets                     => 'Function',
    searchMessages                                 => 'Function',
    searchMessagesChatTypeFilterChannel            => 'SearchMessagesChatTypeFilter',
    searchMessagesChatTypeFilterGroup              => 'SearchMessagesChatTypeFilter',
    searchMessagesChatTypeFilterPrivate            => 'SearchMessagesChatTypeFilter',
    searchMessagesFilterAnimation                  => 'SearchMessagesFilter',
    searchMessagesFilterAudio                      => 'SearchMessagesFilter',
    searchMessagesFilterChatPhoto                  => 'SearchMessagesFilter',
    searchMessagesFilterDocument                   => 'SearchMessagesFilter',
    searchMessagesFilterEmpty                      => 'SearchMessagesFilter',
    searchMessagesFilterFailedToSend               => 'SearchMessagesFilter',
    searchMessagesFilterMention                    => 'SearchMessagesFilter',
    searchMessagesFilterPhoto                      => 'SearchMessagesFilter',
    searchMessagesFilterPhotoAndVideo              => 'SearchMessagesFilter',
    searchMessagesFilterPinned                     => 'SearchMessagesFilter',
    searchMessagesFilterPoll                       => 'SearchMessagesFilter',
    searchMessagesFilterUnreadMention              => 'SearchMessagesFilter',
    searchMessagesFilterUnreadPollVote             => 'SearchMessagesFilter',
    searchMessagesFilterUnreadReaction             => 'SearchMessagesFilter',
    searchMessagesFilterUrl                        => 'SearchMessagesFilter',
    searchMessagesFilterVideo                      => 'SearchMessagesFilter',
    searchMessagesFilterVideoNote                  => 'SearchMessagesFilter',
    searchMessagesFilterVoiceAndVideoNote          => 'SearchMessagesFilter',
    searchMessagesFilterVoiceNote                  => 'SearchMessagesFilter',
    searchOutgoingDocumentMessages                 => 'Function',
    searchPublicChat                               => 'Function',
    searchPublicChats                              => 'Function',
    searchPublicMessagesByTag                      => 'Function',
    searchPublicPosts                              => 'Function',
    searchPublicStoriesByLocation                  => 'Function',
    searchPublicStoriesByTag                       => 'Function',
    searchPublicStoriesByVenue                     => 'Function',
    searchQuote                                    => 'Function',
    searchRecentlyFoundChats                       => 'Function',
    searchSavedMessages                            => 'Function',
    searchSecretMessages                           => 'Function',
    searchStickerSet                               => 'Function',
    searchStickerSets                              => 'Function',
    searchStickers                                 => 'Function',
    searchStringsByPrefix                          => 'Function',
    searchTextCompositionStyle                     => 'Function',
    searchUserByPhoneNumber                        => 'Function',
    searchUserByToken                              => 'Function',
    searchWebApp                                   => 'Function',
    seconds                                        => 'Object',
    secretChat                                     => 'Object',
    secretChatStateClosed                          => 'SecretChatState',
    secretChatStatePending                         => 'SecretChatState',
    secretChatStateReady                           => 'SecretChatState',
    sellGift                                       => 'Function',
    sendAuthenticationFirebaseSms                  => 'Function',
    sendBotStartMessage                            => 'Function',
    sendBusinessMessage                            => 'Function',
    sendBusinessMessageAlbum                       => 'Function',
    sendCallDebugInformation                       => 'Function',
    sendCallLog                                    => 'Function',
    sendCallRating                                 => 'Function',
    sendCallSignalingData                          => 'Function',
    sendChatAction                                 => 'Function',
    sendCustomRequest                              => 'Function',
    sendEmailAddressVerificationCode               => 'Function',
    sendEphemeralMessage                           => 'Function',
    sendGift                                       => 'Function',
    sendGiftPurchaseOffer                          => 'Function',
    sendGroupCallMessage                           => 'Function',
    sendInlineQueryResultMessage                   => 'Function',
    sendMessage                                    => 'Function',
    sendMessageAlbum                               => 'Function',
    sendMessageViewMetrics                         => 'Function',
    sendPassportAuthorizationForm                  => 'Function',
    sendPaymentForm                                => 'Function',
    sendPhoneNumberCode                            => 'Function',
    sendPhoneNumberFirebaseSms                     => 'Function',
    sendQuickReplyShortcutMessages                 => 'Function',
    sendResoldGift                                 => 'Function',
    sendRichMessageDraft                           => 'Function',
    sendTextMessageDraft                           => 'Function',
    sendWebAppCustomRequest                        => 'Function',
    sendWebAppData                                 => 'Function',
    sentGiftRegular                                => 'SentGift',
    sentGiftUpgraded                               => 'SentGift',
    session                                        => 'Object',
    sessionDeviceTypeAndroid                       => 'SessionDeviceType',
    sessionDeviceTypeApple                         => 'SessionDeviceType',
    sessionDeviceTypeBrave                         => 'SessionDeviceType',
    sessionDeviceTypeChrome                        => 'SessionDeviceType',
    sessionDeviceTypeEdge                          => 'SessionDeviceType',
    sessionDeviceTypeFirefox                       => 'SessionDeviceType',
    sessionDeviceTypeIpad                          => 'SessionDeviceType',
    sessionDeviceTypeIphone                        => 'SessionDeviceType',
    sessionDeviceTypeLinux                         => 'SessionDeviceType',
    sessionDeviceTypeMac                           => 'SessionDeviceType',
    sessionDeviceTypeOpera                         => 'SessionDeviceType',
    sessionDeviceTypeSafari                        => 'SessionDeviceType',
    sessionDeviceTypeUbuntu                        => 'SessionDeviceType',
    sessionDeviceTypeUnknown                       => 'SessionDeviceType',
    sessionDeviceTypeVivaldi                       => 'SessionDeviceType',
    sessionDeviceTypeWindows                       => 'SessionDeviceType',
    sessionDeviceTypeXbox                          => 'SessionDeviceType',
    sessionTypeConnectedBot                        => 'SessionType',
    sessionTypeDevice                              => 'SessionType',
    sessions                                       => 'Object',
    setAccentColor                                 => 'Function',
    setAccountTtl                                  => 'Function',
    setAlarm                                       => 'Function',
    setApplicationVerificationToken                => 'Function',
    setArchiveChatListSettings                     => 'Function',
    setAuthenticationEmailAddress                  => 'Function',
    setAuthenticationPhoneNumber                   => 'Function',
    setAuthenticationPremiumPurchaseTransaction    => 'Function',
    setAutoDownloadSettings                        => 'Function',
    setAutosaveSettings                            => 'Function',
    setBio                                         => 'Function',
    setBirthdate                                   => 'Function',
    setBotInfoDescription                          => 'Function',
    setBotInfoShortDescription                     => 'Function',
    setBotName                                     => 'Function',
    setBotProfilePhoto                             => 'Function',
    setBotUpdatesStatus                            => 'Function',
    setBusinessAccountBio                          => 'Function',
    setBusinessAccountGiftSettings                 => 'Function',
    setBusinessAccountName                         => 'Function',
    setBusinessAccountProfilePhoto                 => 'Function',
    setBusinessAccountUsername                     => 'Function',
    setBusinessAwayMessageSettings                 => 'Function',
    setBusinessConnectedBot                        => 'Function',
    setBusinessGreetingMessageSettings             => 'Function',
    setBusinessLocation                            => 'Function',
    setBusinessMessageIsPinned                     => 'Function',
    setBusinessOpeningHours                        => 'Function',
    setBusinessStartPage                           => 'Function',
    setChatAccentColor                             => 'Function',
    setChatActiveStoriesList                       => 'Function',
    setChatAffiliateProgram                        => 'Function',
    setChatAvailableReactions                      => 'Function',
    setChatBackground                              => 'Function',
    setChatClientData                              => 'Function',
    setChatDescription                             => 'Function',
    setChatDirectMessagesGroup                     => 'Function',
    setChatDiscussionGroup                         => 'Function',
    setChatDraftMessage                            => 'Function',
    setChatEmojiStatus                             => 'Function',
    setChatLocation                                => 'Function',
    setChatMemberStatus                            => 'Function',
    setChatMemberTag                               => 'Function',
    setChatMessageAutoDeleteTime                   => 'Function',
    setChatMessageSender                           => 'Function',
    setChatNotificationSettings                    => 'Function',
    setChatPaidMessageStarCount                    => 'Function',
    setChatPermissions                             => 'Function',
    setChatPhoto                                   => 'Function',
    setChatPinnedStories                           => 'Function',
    setChatProfileAccentColor                      => 'Function',
    setChatSlowModeDelay                           => 'Function',
    setChatTheme                                   => 'Function',
    setChatTitle                                   => 'Function',
    setCloseFriends                                => 'Function',
    setCommands                                    => 'Function',
    setCustomEmojiStickerSetThumbnail              => 'Function',
    setCustomLanguagePack                          => 'Function',
    setCustomLanguagePackString                    => 'Function',
    setDatabaseEncryptionKey                       => 'Function',
    setDefaultBackground                           => 'Function',
    setDefaultChannelAdministratorRights           => 'Function',
    setDefaultGroupAdministratorRights             => 'Function',
    setDefaultMessageAutoDeleteTime                => 'Function',
    setDefaultReactionType                         => 'Function',
    setDirectMessagesChatTopicIsMarkedAsUnread     => 'Function',
    setEmojiStatus                                 => 'Function',
    setFileGenerationProgress                      => 'Function',
    setForumTopicNotificationSettings              => 'Function',
    setGameScore                                   => 'Function',
    setGiftCollectionName                          => 'Function',
    setGiftResalePrice                             => 'Function',
    setGiftSettings                                => 'Function',
    setGroupCallPaidMessageStarCount               => 'Function',
    setGroupCallParticipantIsSpeaking              => 'Function',
    setGroupCallParticipantVolumeLevel             => 'Function',
    setInactiveSessionTtl                          => 'Function',
    setInlineGameScore                             => 'Function',
    setLiveStoryMessageSender                      => 'Function',
    setLogStream                                   => 'Function',
    setLogTagVerbosityLevel                        => 'Function',
    setLogVerbosityLevel                           => 'Function',
    setLoginEmailAddress                           => 'Function',
    setMainProfileTab                              => 'Function',
    setManagedBotAccessSettings                    => 'Function',
    setMenuButton                                  => 'Function',
    setMessageFactCheck                            => 'Function',
    setMessageReactions                            => 'Function',
    setMessageSenderBlockList                      => 'Function',
    setMessageSenderBotVerification                => 'Function',
    setName                                        => 'Function',
    setNetworkType                                 => 'Function',
    setNewChatPrivacySettings                      => 'Function',
    setOption                                      => 'Function',
    setPaidMessageReactionType                     => 'Function',
    setPassportElement                             => 'Function',
    setPassportElementErrors                       => 'Function',
    setPassword                                    => 'Function',
    setPersonalChat                                => 'Function',
    setPinnedChats                                 => 'Function',
    setPinnedForumTopics                           => 'Function',
    setPinnedGifts                                 => 'Function',
    setPinnedSavedMessagesTopics                   => 'Function',
    setPollAnswer                                  => 'Function',
    setProfileAccentColor                          => 'Function',
    setProfileAudioPosition                        => 'Function',
    setProfilePhoto                                => 'Function',
    setQuickReplyShortcutName                      => 'Function',
    setReactionNotificationSettings                => 'Function',
    setReadDatePrivacySettings                     => 'Function',
    setRecoveryEmailAddress                        => 'Function',
    setSavedMessagesTagLabel                       => 'Function',
    setScopeNotificationSettings                   => 'Function',
    setStickerEmojis                               => 'Function',
    setStickerKeywords                             => 'Function',
    setStickerMaskPosition                         => 'Function',
    setStickerPositionInSet                        => 'Function',
    setStickerSetThumbnail                         => 'Function',
    setStickerSetTitle                             => 'Function',
    setStoryAlbumName                              => 'Function',
    setStoryPrivacySettings                        => 'Function',
    setStoryReaction                               => 'Function',
    setSupergroupCustomEmojiStickerSet             => 'Function',
    setSupergroupMainProfileTab                    => 'Function',
    setSupergroupStickerSet                        => 'Function',
    setSupergroupUnrestrictBoostCount              => 'Function',
    setSupergroupUsername                          => 'Function',
    setTdlibParameters                             => 'Function',
    setUpgradedGiftColors                          => 'Function',
    setUserEmojiStatus                             => 'Function',
    setUserNote                                    => 'Function',
    setUserPersonalProfilePhoto                    => 'Function',
    setUserPrivacySettingRules                     => 'Function',
    setUserSupportInfo                             => 'Function',
    setUsername                                    => 'Function',
    setVideoChatDefaultParticipant                 => 'Function',
    setVideoChatTitle                              => 'Function',
    settingsSectionAppearance                      => 'SettingsSection',
    settingsSectionAskQuestion                     => 'SettingsSection',
    settingsSectionBusiness                        => 'SettingsSection',
    settingsSectionChatFolders                     => 'SettingsSection',
    settingsSectionDataAndStorage                  => 'SettingsSection',
    settingsSectionDevices                         => 'SettingsSection',
    settingsSectionEditProfile                     => 'SettingsSection',
    settingsSectionFaq                             => 'SettingsSection',
    settingsSectionFeatures                        => 'SettingsSection',
    settingsSectionInAppBrowser                    => 'SettingsSection',
    settingsSectionLanguage                        => 'SettingsSection',
    settingsSectionMyGrams                         => 'SettingsSection',
    settingsSectionMyStars                         => 'SettingsSection',
    settingsSectionNotifications                   => 'SettingsSection',
    settingsSectionPowerSaving                     => 'SettingsSection',
    settingsSectionPremium                         => 'SettingsSection',
    settingsSectionPrivacyAndSecurity              => 'SettingsSection',
    settingsSectionPrivacyPolicy                   => 'SettingsSection',
    settingsSectionQrCode                          => 'SettingsSection',
    settingsSectionSearch                          => 'SettingsSection',
    settingsSectionSendGift                        => 'SettingsSection',
    shareChatWithBot                               => 'Function',
    sharePhoneNumber                               => 'Function',
    shareUsersWithBot                              => 'Function',
    sharedChat                                     => 'Object',
    sharedUser                                     => 'Object',
    shippingOption                                 => 'Object',
    speechRecognitionResultError                   => 'SpeechRecognitionResult',
    speechRecognitionResultPending                 => 'SpeechRecognitionResult',
    speechRecognitionResultText                    => 'SpeechRecognitionResult',
    sponsoredChat                                  => 'Object',
    sponsoredChats                                 => 'Object',
    sponsoredMessage                               => 'Object',
    sponsoredMessages                              => 'Object',
    stakeDiceState                                 => 'Object',
    starAmount                                     => 'Object',
    starCount                                      => 'Object',
    starGiveawayPaymentOption                      => 'Object',
    starGiveawayPaymentOptions                     => 'Object',
    starGiveawayWinnerOption                       => 'Object',
    starPaymentOption                              => 'Object',
    starPaymentOptions                             => 'Object',
    starRevenueStatistics                          => 'Object',
    starRevenueStatus                              => 'Object',
    starSubscription                               => 'Object',
    starSubscriptionPricing                        => 'Object',
    starSubscriptionTypeBot                        => 'StarSubscriptionType',
    starSubscriptionTypeChannel                    => 'StarSubscriptionType',
    starSubscriptions                              => 'Object',
    starTransaction                                => 'Object',
    starTransactionTypeAffiliateProgramCommission  => 'StarTransactionType',
    starTransactionTypeAppStoreDeposit             => 'StarTransactionType',
    starTransactionTypeBotInvoicePurchase          => 'StarTransactionType',
    starTransactionTypeBotInvoiceSale              => 'StarTransactionType',
    starTransactionTypeBotPaidMediaPurchase        => 'StarTransactionType',
    starTransactionTypeBotPaidMediaSale            => 'StarTransactionType',
    starTransactionTypeBotSubscriptionPurchase     => 'StarTransactionType',
    starTransactionTypeBotSubscriptionSale         => 'StarTransactionType',
    starTransactionTypeBusinessBotTransferReceive  => 'StarTransactionType',
    starTransactionTypeBusinessBotTransferSend     => 'StarTransactionType',
    starTransactionTypeChannelPaidMediaPurchase    => 'StarTransactionType',
    starTransactionTypeChannelPaidMediaSale        => 'StarTransactionType',
    starTransactionTypeChannelPaidReactionReceive  => 'StarTransactionType',
    starTransactionTypeChannelPaidReactionSend     => 'StarTransactionType',
    starTransactionTypeChannelSubscriptionPurchase => 'StarTransactionType',
    starTransactionTypeChannelSubscriptionSale     => 'StarTransactionType',
    starTransactionTypeFragmentDeposit             => 'StarTransactionType',
    starTransactionTypeFragmentWithdrawal          => 'StarTransactionType',
    starTransactionTypeGiftAuctionBid              => 'StarTransactionType',
    starTransactionTypeGiftOriginalDetailsDrop     => 'StarTransactionType',
    starTransactionTypeGiftPurchase                => 'StarTransactionType',
    starTransactionTypeGiftPurchaseOffer           => 'StarTransactionType',
    starTransactionTypeGiftSale                    => 'StarTransactionType',
    starTransactionTypeGiftTransfer                => 'StarTransactionType',
    starTransactionTypeGiftUpgrade                 => 'StarTransactionType',
    starTransactionTypeGiftUpgradePurchase         => 'StarTransactionType',
    starTransactionTypeGiveawayDeposit             => 'StarTransactionType',
    starTransactionTypeGooglePlayDeposit           => 'StarTransactionType',
    starTransactionTypePaidGroupCallMessageReceive => 'StarTransactionType',
    starTransactionTypePaidGroupCallMessageSend    => 'StarTransactionType',
    starTransactionTypePaidGroupCallReactionReceive => 'StarTransactionType',
    starTransactionTypePaidGroupCallReactionSend   => 'StarTransactionType',
    starTransactionTypePaidMessageReceive          => 'StarTransactionType',
    starTransactionTypePaidMessageSend             => 'StarTransactionType',
    starTransactionTypePremiumBotDeposit           => 'StarTransactionType',
    starTransactionTypePremiumPurchase             => 'StarTransactionType',
    starTransactionTypePublicPostSearch            => 'StarTransactionType',
    starTransactionTypeSuggestedPostPaymentReceive => 'StarTransactionType',
    starTransactionTypeSuggestedPostPaymentSend    => 'StarTransactionType',
    starTransactionTypeTelegramAdsWithdrawal       => 'StarTransactionType',
    starTransactionTypeTelegramApiUsage            => 'StarTransactionType',
    starTransactionTypeUnsupported                 => 'StarTransactionType',
    starTransactionTypeUpgradedGiftPurchase        => 'StarTransactionType',
    starTransactionTypeUpgradedGiftSale            => 'StarTransactionType',
    starTransactionTypeUserDeposit                 => 'StarTransactionType',
    starTransactions                               => 'Object',
    startGroupCallRecording                        => 'Function',
    startGroupCallScreenSharing                    => 'Function',
    startLiveStory                                 => 'Function',
    startLiveStoryResultFail                       => 'StartLiveStoryResult',
    startLiveStoryResultOk                         => 'StartLiveStoryResult',
    startScheduledVideoChat                        => 'Function',
    statisticalGraphAsync                          => 'StatisticalGraph',
    statisticalGraphData                           => 'StatisticalGraph',
    statisticalGraphError                          => 'StatisticalGraph',
    statisticalValue                               => 'Object',
    sticker                                        => 'Object',
    stickerFormatTgs                               => 'StickerFormat',
    stickerFormatWebm                              => 'StickerFormat',
    stickerFormatWebp                              => 'StickerFormat',
    stickerFullTypeCustomEmoji                     => 'StickerFullType',
    stickerFullTypeMask                            => 'StickerFullType',
    stickerFullTypeRegular                         => 'StickerFullType',
    stickerSet                                     => 'Object',
    stickerSetInfo                                 => 'Object',
    stickerSets                                    => 'Object',
    stickerTypeCustomEmoji                         => 'StickerType',
    stickerTypeMask                                => 'StickerType',
    stickerTypeRegular                             => 'StickerType',
    stickers                                       => 'Object',
    stopBusinessPoll                               => 'Function',
    stopPoll                                       => 'Function',
    storageStatistics                              => 'Object',
    storageStatisticsByChat                        => 'Object',
    storageStatisticsByFileType                    => 'Object',
    storageStatisticsFast                          => 'Object',
    storePaymentPurposeGiftedStars                 => 'StorePaymentPurpose',
    storePaymentPurposePremiumGift                 => 'StorePaymentPurpose',
    storePaymentPurposePremiumGiftCodes            => 'StorePaymentPurpose',
    storePaymentPurposePremiumGiveaway             => 'StorePaymentPurpose',
    storePaymentPurposePremiumSubscription         => 'StorePaymentPurpose',
    storePaymentPurposeStarGiveaway                => 'StorePaymentPurpose',
    storePaymentPurposeStars                       => 'StorePaymentPurpose',
    storeTransactionAppStore                       => 'StoreTransaction',
    storeTransactionGooglePlay                     => 'StoreTransaction',
    stories                                        => 'Object',
    story                                          => 'Object',
    storyAlbum                                     => 'Object',
    storyAlbums                                    => 'Object',
    storyArea                                      => 'Object',
    storyAreaPosition                              => 'Object',
    storyAreaTypeLink                              => 'StoryAreaType',
    storyAreaTypeLocation                          => 'StoryAreaType',
    storyAreaTypeMessage                           => 'StoryAreaType',
    storyAreaTypeSuggestedReaction                 => 'StoryAreaType',
    storyAreaTypeUpgradedGift                      => 'StoryAreaType',
    storyAreaTypeVenue                             => 'StoryAreaType',
    storyAreaTypeWeather                           => 'StoryAreaType',
    storyContentLive                               => 'StoryContent',
    storyContentPhoto                              => 'StoryContent',
    storyContentTypeLive                           => 'StoryContentType',
    storyContentTypePhoto                          => 'StoryContentType',
    storyContentTypeUnsupported                    => 'StoryContentType',
    storyContentTypeVideo                          => 'StoryContentType',
    storyContentUnsupported                        => 'StoryContent',
    storyContentVideo                              => 'StoryContent',
    storyFullId                                    => 'Object',
    storyInfo                                      => 'Object',
    storyInteraction                               => 'Object',
    storyInteractionInfo                           => 'Object',
    storyInteractionTypeForward                    => 'StoryInteractionType',
    storyInteractionTypeRepost                     => 'StoryInteractionType',
    storyInteractionTypeView                       => 'StoryInteractionType',
    storyInteractions                              => 'Object',
    storyListArchive                               => 'StoryList',
    storyListMain                                  => 'StoryList',
    storyOriginHiddenUser                          => 'StoryOrigin',
    storyOriginPublicStory                         => 'StoryOrigin',
    storyPrivacySettingsCloseFriends               => 'StoryPrivacySettings',
    storyPrivacySettingsContacts                   => 'StoryPrivacySettings',
    storyPrivacySettingsEveryone                   => 'StoryPrivacySettings',
    storyPrivacySettingsSelectedUsers              => 'StoryPrivacySettings',
    storyRepostInfo                                => 'Object',
    storyStatistics                                => 'Object',
    storyVideo                                     => 'Object',
    suggestUserBirthdate                           => 'Function',
    suggestUserProfilePhoto                        => 'Function',
    suggestedActionAddLoginPasskey                 => 'SuggestedAction',
    suggestedActionCheckPassword                   => 'SuggestedAction',
    suggestedActionCheckPhoneNumber                => 'SuggestedAction',
    suggestedActionConvertToBroadcastGroup         => 'SuggestedAction',
    suggestedActionCustom                          => 'SuggestedAction',
    suggestedActionEnableArchiveAndMuteNewChats    => 'SuggestedAction',
    suggestedActionExtendPremium                   => 'SuggestedAction',
    suggestedActionExtendStarSubscriptions         => 'SuggestedAction',
    suggestedActionGiftPremiumForChristmas         => 'SuggestedAction',
    suggestedActionRestorePremium                  => 'SuggestedAction',
    suggestedActionSetBirthdate                    => 'SuggestedAction',
    suggestedActionSetLoginEmailAddress            => 'SuggestedAction',
    suggestedActionSetPassword                     => 'SuggestedAction',
    suggestedActionSetProfilePhoto                 => 'SuggestedAction',
    suggestedActionSubscribeToAnnualPremium        => 'SuggestedAction',
    suggestedActionUpgradePremium                  => 'SuggestedAction',
    suggestedActionViewChecksHint                  => 'SuggestedAction',
    suggestedPostInfo                              => 'Object',
    suggestedPostPriceGram                         => 'SuggestedPostPrice',
    suggestedPostPriceStar                         => 'SuggestedPostPrice',
    suggestedPostRefundReasonPaymentRefunded       => 'SuggestedPostRefundReason',
    suggestedPostRefundReasonPostDeleted           => 'SuggestedPostRefundReason',
    suggestedPostStateApproved                     => 'SuggestedPostState',
    suggestedPostStateDeclined                     => 'SuggestedPostState',
    suggestedPostStatePending                      => 'SuggestedPostState',
    summarizeMessage                               => 'Function',
    supergroup                                     => 'Object',
    supergroupFullInfo                             => 'Object',
    supergroupMembersFilterAdministrators          => 'SupergroupMembersFilter',
    supergroupMembersFilterBanned                  => 'SupergroupMembersFilter',
    supergroupMembersFilterBots                    => 'SupergroupMembersFilter',
    supergroupMembersFilterContacts                => 'SupergroupMembersFilter',
    supergroupMembersFilterMention                 => 'SupergroupMembersFilter',
    supergroupMembersFilterRecent                  => 'SupergroupMembersFilter',
    supergroupMembersFilterRestricted              => 'SupergroupMembersFilter',
    supergroupMembersFilterSearch                  => 'SupergroupMembersFilter',
    synchronizeLanguagePack                        => 'Function',
    tMeUrl                                         => 'Object',
    tMeUrlTypeChatInvite                           => 'TMeUrlType',
    tMeUrlTypeStickerSet                           => 'TMeUrlType',
    tMeUrlTypeSupergroup                           => 'TMeUrlType',
    tMeUrlTypeUser                                 => 'TMeUrlType',
    tMeUrls                                        => 'Object',
    targetChatChosen                               => 'TargetChat',
    targetChatCurrent                              => 'TargetChat',
    targetChatInternalLink                         => 'TargetChat',
    targetChatTypes                                => 'Object',
    telegramPaymentPurposeGiftedStars              => 'TelegramPaymentPurpose',
    telegramPaymentPurposeJoinChat                 => 'TelegramPaymentPurpose',
    telegramPaymentPurposePremiumGift              => 'TelegramPaymentPurpose',
    telegramPaymentPurposePremiumGiftCodes         => 'TelegramPaymentPurpose',
    telegramPaymentPurposePremiumGiveaway          => 'TelegramPaymentPurpose',
    telegramPaymentPurposeStarGiveaway             => 'TelegramPaymentPurpose',
    telegramPaymentPurposeStars                    => 'TelegramPaymentPurpose',
    temporaryPasswordState                         => 'Object',
    terminateAllOtherSessions                      => 'Function',
    terminateSession                               => 'Function',
    termsOfService                                 => 'Object',
    testBytes                                      => 'Object',
    testCallBytes                                  => 'Function',
    testCallEmpty                                  => 'Function',
    testCallString                                 => 'Function',
    testCallVectorInt                              => 'Function',
    testCallVectorIntObject                        => 'Function',
    testCallVectorString                           => 'Function',
    testCallVectorStringObject                     => 'Function',
    testGetDifference                              => 'Function',
    testInt                                        => 'Object',
    testNetwork                                    => 'Function',
    testProxy                                      => 'Function',
    testReturnError                                => 'Function',
    testSquareInt                                  => 'Function',
    testString                                     => 'Object',
    testUseUpdate                                  => 'Function',
    testVectorInt                                  => 'Object',
    testVectorIntObject                            => 'Object',
    testVectorString                               => 'Object',
    testVectorStringObject                         => 'Object',
    text                                           => 'Object',
    textCompositionStyle                           => 'Object',
    textCompositionStyleExample                    => 'Object',
    textEntities                                   => 'Object',
    textEntity                                     => 'Object',
    textEntityTypeBankCardNumber                   => 'TextEntityType',
    textEntityTypeBlockQuote                       => 'TextEntityType',
    textEntityTypeBold                             => 'TextEntityType',
    textEntityTypeBotCommand                       => 'TextEntityType',
    textEntityTypeCashtag                          => 'TextEntityType',
    textEntityTypeCode                             => 'TextEntityType',
    textEntityTypeCustomEmoji                      => 'TextEntityType',
    textEntityTypeDateTime                         => 'TextEntityType',
    textEntityTypeEmailAddress                     => 'TextEntityType',
    textEntityTypeExpandableBlockQuote             => 'TextEntityType',
    textEntityTypeHashtag                          => 'TextEntityType',
    textEntityTypeItalic                           => 'TextEntityType',
    textEntityTypeMediaTimestamp                   => 'TextEntityType',
    textEntityTypeMention                          => 'TextEntityType',
    textEntityTypeMentionName                      => 'TextEntityType',
    textEntityTypePhoneNumber                      => 'TextEntityType',
    textEntityTypePre                              => 'TextEntityType',
    textEntityTypePreCode                          => 'TextEntityType',
    textEntityTypeSpoiler                          => 'TextEntityType',
    textEntityTypeStrikethrough                    => 'TextEntityType',
    textEntityTypeTextUrl                          => 'TextEntityType',
    textEntityTypeUnderline                        => 'TextEntityType',
    textEntityTypeUrl                              => 'TextEntityType',
    textParseModeHTML                              => 'TextParseMode',
    textParseModeMarkdown                          => 'TextParseMode',
    textQuote                                      => 'Object',
    themeParameters                                => 'Object',
    themeSettings                                  => 'Object',
    thumbnail                                      => 'Object',
    thumbnailFormatGif                             => 'ThumbnailFormat',
    thumbnailFormatJpeg                            => 'ThumbnailFormat',
    thumbnailFormatMpeg4                           => 'ThumbnailFormat',
    thumbnailFormatPng                             => 'ThumbnailFormat',
    thumbnailFormatTgs                             => 'ThumbnailFormat',
    thumbnailFormatWebm                            => 'ThumbnailFormat',
    thumbnailFormatWebp                            => 'ThumbnailFormat',
    timeZone                                       => 'Object',
    timeZones                                      => 'Object',
    toggleAllDownloadsArePaused                    => 'Function',
    toggleBotCanManageEmojiStatus                  => 'Function',
    toggleBotIsAddedToAttachmentMenu               => 'Function',
    toggleBotUsernameIsActive                      => 'Function',
    toggleBusinessConnectedBotChatIsPaused         => 'Function',
    toggleChatDefaultDisableNotification           => 'Function',
    toggleChatFolderTags                           => 'Function',
    toggleChatGiftNotifications                    => 'Function',
    toggleChatHasProtectedContent                  => 'Function',
    toggleChatIsMarkedAsUnread                     => 'Function',
    toggleChatIsPinned                             => 'Function',
    toggleChatIsTranslatable                       => 'Function',
    toggleChatViewAsTopics                         => 'Function',
    toggleDirectMessagesChatTopicCanSendUnpaidMessages => 'Function',
    toggleDownloadIsPaused                         => 'Function',
    toggleForumTopicIsClosed                       => 'Function',
    toggleForumTopicIsPinned                       => 'Function',
    toggleGeneralForumTopicIsHidden                => 'Function',
    toggleGiftIsSaved                              => 'Function',
    toggleGroupCallAreMessagesAllowed              => 'Function',
    toggleGroupCallIsMyVideoEnabled                => 'Function',
    toggleGroupCallIsMyVideoPaused                 => 'Function',
    toggleGroupCallParticipantIsHandRaised         => 'Function',
    toggleGroupCallParticipantIsMuted              => 'Function',
    toggleGroupCallScreenSharingIsPaused           => 'Function',
    toggleHasSponsoredMessagesEnabled              => 'Function',
    toggleSavedMessagesTopicIsPinned               => 'Function',
    toggleSessionCanAcceptCalls                    => 'Function',
    toggleSessionCanAcceptSecretChats              => 'Function',
    toggleStoryIsPostedToChatPage                  => 'Function',
    toggleSupergroupCanHaveSponsoredMessages       => 'Function',
    toggleSupergroupHasAggressiveAntiSpamEnabled   => 'Function',
    toggleSupergroupHasAutomaticTranslation        => 'Function',
    toggleSupergroupHasHiddenMembers               => 'Function',
    toggleSupergroupIsAllHistoryAvailable          => 'Function',
    toggleSupergroupIsBroadcastGroup               => 'Function',
    toggleSupergroupIsForum                        => 'Function',
    toggleSupergroupJoinByRequest                  => 'Function',
    toggleSupergroupJoinToSendMessages             => 'Function',
    toggleSupergroupSignMessages                   => 'Function',
    toggleSupergroupUsernameIsActive               => 'Function',
    toggleUsernameIsActive                         => 'Function',
    toggleVideoChatEnabledStartNotification        => 'Function',
    toggleVideoChatMuteNewParticipants             => 'Function',
    tonTransaction                                 => 'Object',
    tonTransactionTypeFragmentDeposit              => 'TonTransactionType',
    tonTransactionTypeFragmentWithdrawal           => 'TonTransactionType',
    tonTransactionTypeGiftPurchaseOffer            => 'TonTransactionType',
    tonTransactionTypeStakeDicePayout              => 'TonTransactionType',
    tonTransactionTypeStakeDiceStake               => 'TonTransactionType',
    tonTransactionTypeSuggestedPostPayment         => 'TonTransactionType',
    tonTransactionTypeUnsupported                  => 'TonTransactionType',
    tonTransactionTypeUpgradedGiftPurchase         => 'TonTransactionType',
    tonTransactionTypeUpgradedGiftSale             => 'TonTransactionType',
    tonTransactions                                => 'Object',
    topChatCategoryBots                            => 'TopChatCategory',
    topChatCategoryCalls                           => 'TopChatCategory',
    topChatCategoryChannels                        => 'TopChatCategory',
    topChatCategoryForwardChats                    => 'TopChatCategory',
    topChatCategoryGroups                          => 'TopChatCategory',
    topChatCategoryGuestBots                       => 'TopChatCategory',
    topChatCategoryInlineBots                      => 'TopChatCategory',
    topChatCategoryUsers                           => 'TopChatCategory',
    topChatCategoryWebAppBots                      => 'TopChatCategory',
    transactionDirectionIncoming                   => 'TransactionDirection',
    transactionDirectionOutgoing                   => 'TransactionDirection',
    transferBusinessAccountStars                   => 'Function',
    transferChatOwnership                          => 'Function',
    transferGift                                   => 'Function',
    translateMessageRichMessage                    => 'Function',
    translateMessageText                           => 'Function',
    translateRichMessage                           => 'Function',
    translateText                                  => 'Function',
    trendingStickerSets                            => 'Object',
    unconfirmedSession                             => 'Object',
    unpinAllChatMessages                           => 'Function',
    unpinAllDirectMessagesChatTopicMessages        => 'Function',
    unpinAllForumTopicMessages                     => 'Function',
    unpinChatMessage                               => 'Function',
    unreadReaction                                 => 'Object',
    updateAccentColors                             => 'Update',
    updateActiveEmojiReactions                     => 'Update',
    updateActiveGiftAuctions                       => 'Update',
    updateActiveLiveLocationMessages               => 'Update',
    updateActiveNotifications                      => 'Update',
    updateAgeVerificationParameters                => 'Update',
    updateAnimatedEmojiMessageClicked              => 'Update',
    updateAnimationSearchParameters                => 'Update',
    updateApplicationRecaptchaVerificationRequired => 'Update',
    updateApplicationVerificationRequired          => 'Update',
    updateAttachmentMenuBots                       => 'Update',
    updateAuthorizationState                       => 'Update',
    updateAutosaveSettings                         => 'Update',
    updateAvailableMessageEffects                  => 'Update',
    updateBasicGroup                               => 'Update',
    updateBasicGroupFullInfo                       => 'Update',
    updateBusinessConnection                       => 'Update',
    updateBusinessMessageEdited                    => 'Update',
    updateBusinessMessagesDeleted                  => 'Update',
    updateCall                                     => 'Update',
    updateChatAccentColors                         => 'Update',
    updateChatAction                               => 'Update',
    updateChatActionBar                            => 'Update',
    updateChatActiveStories                        => 'Update',
    updateChatAddedToList                          => 'Update',
    updateChatAvailableReactions                   => 'Update',
    updateChatBackground                           => 'Update',
    updateChatBlockList                            => 'Update',
    updateChatBoost                                => 'Update',
    updateChatBusinessBotManageBar                 => 'Update',
    updateChatDefaultDisableNotification           => 'Update',
    updateChatDraftMessage                         => 'Update',
    updateChatEmojiStatus                          => 'Update',
    updateChatFolders                              => 'Update',
    updateChatHasProtectedContent                  => 'Update',
    updateChatHasScheduledMessages                 => 'Update',
    updateChatIsMarkedAsUnread                     => 'Update',
    updateChatIsTranslatable                       => 'Update',
    updateChatJoinResult                           => 'Update',
    updateChatLastMessage                          => 'Update',
    updateChatMember                               => 'Update',
    updateChatMessageAutoDeleteTime                => 'Update',
    updateChatMessageSender                        => 'Update',
    updateChatNotificationSettings                 => 'Update',
    updateChatOnlineMemberCount                    => 'Update',
    updateChatPendingJoinRequests                  => 'Update',
    updateChatPermissions                          => 'Update',
    updateChatPhoto                                => 'Update',
    updateChatPosition                             => 'Update',
    updateChatReadInbox                            => 'Update',
    updateChatReadOutbox                           => 'Update',
    updateChatRemovedFromList                      => 'Update',
    updateChatReplyMarkup                          => 'Update',
    updateChatRevenueAmount                        => 'Update',
    updateChatTheme                                => 'Update',
    updateChatTitle                                => 'Update',
    updateChatUnreadMentionCount                   => 'Update',
    updateChatUnreadPollVoteCount                  => 'Update',
    updateChatUnreadReactionCount                  => 'Update',
    updateChatVideoChat                            => 'Update',
    updateChatViewAsTopics                         => 'Update',
    updateCommunity                                => 'Update',
    updateConnectionState                          => 'Update',
    updateContactCloseBirthdays                    => 'Update',
    updateDefaultBackground                        => 'Update',
    updateDefaultPaidReactionType                  => 'Update',
    updateDefaultReactionType                      => 'Update',
    updateDeleteMessages                           => 'Update',
    updateDiceEmojis                               => 'Update',
    updateDirectMessagesChatTopic                  => 'Update',
    updateEmojiChatThemes                          => 'Update',
    updateFavoriteStickers                         => 'Update',
    updateFile                                     => 'Update',
    updateFileAddedToDownloads                     => 'Update',
    updateFileDownload                             => 'Update',
    updateFileDownloads                            => 'Update',
    updateFileGenerationStart                      => 'Update',
    updateFileGenerationStop                       => 'Update',
    updateFileRemovedFromDownloads                 => 'Update',
    updateForumTopic                               => 'Update',
    updateForumTopicInfo                           => 'Update',
    updateFreezeState                              => 'Update',
    updateGiftAuctionState                         => 'Update',
    updateGramRevenueStatus                        => 'Update',
    updateGroupCall                                => 'Update',
    updateGroupCallMessageLevels                   => 'Update',
    updateGroupCallMessageSendFailed               => 'Update',
    updateGroupCallMessagesDeleted                 => 'Update',
    updateGroupCallParticipant                     => 'Update',
    updateGroupCallParticipants                    => 'Update',
    updateGroupCallVerificationState               => 'Update',
    updateHavePendingNotifications                 => 'Update',
    updateInstalledStickerSets                     => 'Update',
    updateLanguagePackStrings                      => 'Update',
    updateLiveStoryTopDonors                       => 'Update',
    updateManagedBot                               => 'Update',
    updateMessageContainsUnreadPollVotes           => 'Update',
    updateMessageContent                           => 'Update',
    updateMessageContentOpened                     => 'Update',
    updateMessageEdited                            => 'Update',
    updateMessageFactCheck                         => 'Update',
    updateMessageInteractionInfo                   => 'Update',
    updateMessageIsPinned                          => 'Update',
    updateMessageLiveLocationViewed                => 'Update',
    updateMessageMentionRead                       => 'Update',
    updateMessageReaction                          => 'Update',
    updateMessageReactions                         => 'Update',
    updateMessageSendAcknowledged                  => 'Update',
    updateMessageSendFailed                        => 'Update',
    updateMessageSendSucceeded                     => 'Update',
    updateMessageSuggestedPostInfo                 => 'Update',
    updateMessageUnreadReactions                   => 'Update',
    updateNewBusinessCallbackQuery                 => 'Update',
    updateNewBusinessMessage                       => 'Update',
    updateNewCallSignalingData                     => 'Update',
    updateNewCallbackQuery                         => 'Update',
    updateNewChat                                  => 'Update',
    updateNewChatJoinRequest                       => 'Update',
    updateNewChosenInlineResult                    => 'Update',
    updateNewCustomEvent                           => 'Update',
    updateNewCustomQuery                           => 'Update',
    updateNewGroupCallMessage                      => 'Update',
    updateNewGroupCallPaidReaction                 => 'Update',
    updateNewGuestQuery                            => 'Update',
    updateNewInlineCallbackQuery                   => 'Update',
    updateNewInlineQuery                           => 'Update',
    updateNewMessage                               => 'Update',
    updateNewOauthRequest                          => 'Update',
    updateNewPreCheckoutQuery                      => 'Update',
    updateNewShippingQuery                         => 'Update',
    updateNotification                             => 'Update',
    updateNotificationGroup                        => 'Update',
    updateOption                                   => 'Update',
    updateOwnedGramCount                           => 'Update',
    updateOwnedStarCount                           => 'Update',
    updatePaidMediaPurchased                       => 'Update',
    updatePendingMessage                           => 'Update',
    updatePoll                                     => 'Update',
    updatePollAnswer                               => 'Update',
    updateProfileAccentColors                      => 'Update',
    updateQuickReplyShortcut                       => 'Update',
    updateQuickReplyShortcutDeleted                => 'Update',
    updateQuickReplyShortcutMessages               => 'Update',
    updateQuickReplyShortcuts                      => 'Update',
    updateReactionNotificationSettings             => 'Update',
    updateRecentStickers                           => 'Update',
    updateSavedAnimations                          => 'Update',
    updateSavedMessagesTags                        => 'Update',
    updateSavedMessagesTopic                       => 'Update',
    updateSavedMessagesTopicCount                  => 'Update',
    updateSavedNotificationSounds                  => 'Update',
    updateScopeNotificationSettings                => 'Update',
    updateSecretChat                               => 'Update',
    updateServiceNotification                      => 'Update',
    updateSpeechRecognitionTrial                   => 'Update',
    updateSpeedLimitNotification                   => 'Update',
    updateStakeDiceState                           => 'Update',
    updateStarRevenueStatus                        => 'Update',
    updateStickerSet                               => 'Update',
    updateStory                                    => 'Update',
    updateStoryDeleted                             => 'Update',
    updateStoryListChatCount                       => 'Update',
    updateStoryPostFailed                          => 'Update',
    updateStoryPostSucceeded                       => 'Update',
    updateStoryStealthMode                         => 'Update',
    updateSuggestedActions                         => 'Update',
    updateSupergroup                               => 'Update',
    updateSupergroupFullInfo                       => 'Update',
    updateTermsOfService                           => 'Update',
    updateTextCompositionStyles                    => 'Update',
    updateTopicMessageCount                        => 'Update',
    updateTrendingStickerSets                      => 'Update',
    updateTrustedMiniAppBots                       => 'Update',
    updateUnconfirmedSession                       => 'Update',
    updateUnreadChatCount                          => 'Update',
    updateUnreadMessageCount                       => 'Update',
    updateUser                                     => 'Update',
    updateUserFullInfo                             => 'Update',
    updateUserPrivacySettingRules                  => 'Update',
    updateUserStatus                               => 'Update',
    updateUserSubscription                         => 'Update',
    updateVideoPublished                           => 'Update',
    updateWebAppMessageSent                        => 'Update',
    updateWebBrowserSettings                       => 'Update',
    updates                                        => 'Object',
    upgradeBasicGroupChatToSupergroupChat          => 'Function',
    upgradeGift                                    => 'Function',
    upgradeGiftResult                              => 'Object',
    upgradedGift                                   => 'Object',
    upgradedGiftAttributeIdBackdrop                => 'UpgradedGiftAttributeId',
    upgradedGiftAttributeIdModel                   => 'UpgradedGiftAttributeId',
    upgradedGiftAttributeIdSymbol                  => 'UpgradedGiftAttributeId',
    upgradedGiftAttributeRarityEpic                => 'UpgradedGiftAttributeRarity',
    upgradedGiftAttributeRarityLegendary           => 'UpgradedGiftAttributeRarity',
    upgradedGiftAttributeRarityPerMille            => 'UpgradedGiftAttributeRarity',
    upgradedGiftAttributeRarityRare                => 'UpgradedGiftAttributeRarity',
    upgradedGiftAttributeRarityUncommon            => 'UpgradedGiftAttributeRarity',
    upgradedGiftBackdrop                           => 'Object',
    upgradedGiftBackdropColors                     => 'Object',
    upgradedGiftBackdropCount                      => 'Object',
    upgradedGiftColors                             => 'Object',
    upgradedGiftModel                              => 'Object',
    upgradedGiftModelCount                         => 'Object',
    upgradedGiftOriginBlockchain                   => 'UpgradedGiftOrigin',
    upgradedGiftOriginCraft                        => 'UpgradedGiftOrigin',
    upgradedGiftOriginOffer                        => 'UpgradedGiftOrigin',
    upgradedGiftOriginPrepaidUpgrade               => 'UpgradedGiftOrigin',
    upgradedGiftOriginResale                       => 'UpgradedGiftOrigin',
    upgradedGiftOriginTransfer                     => 'UpgradedGiftOrigin',
    upgradedGiftOriginUpgrade                      => 'UpgradedGiftOrigin',
    upgradedGiftOriginalDetails                    => 'Object',
    upgradedGiftSymbol                             => 'Object',
    upgradedGiftSymbolCount                        => 'Object',
    upgradedGiftValueInfo                          => 'Object',
    uploadStickerFile                              => 'Function',
    user                                           => 'Object',
    userAuctionBid                                 => 'Object',
    userFullInfo                                   => 'Object',
    userLink                                       => 'Object',
    userPrivacySettingAllowCalls                   => 'UserPrivacySetting',
    userPrivacySettingAllowChatInvites             => 'UserPrivacySetting',
    userPrivacySettingAllowFindingByPhoneNumber    => 'UserPrivacySetting',
    userPrivacySettingAllowPeerToPeerCalls         => 'UserPrivacySetting',
    userPrivacySettingAllowPrivateVoiceAndVideoNoteMessages => 'UserPrivacySetting',
    userPrivacySettingAllowUnpaidMessages          => 'UserPrivacySetting',
    userPrivacySettingAutosaveGifts                => 'UserPrivacySetting',
    userPrivacySettingRuleAllowAll                 => 'UserPrivacySettingRule',
    userPrivacySettingRuleAllowBots                => 'UserPrivacySettingRule',
    userPrivacySettingRuleAllowChatMembers         => 'UserPrivacySettingRule',
    userPrivacySettingRuleAllowContacts            => 'UserPrivacySettingRule',
    userPrivacySettingRuleAllowPremiumUsers        => 'UserPrivacySettingRule',
    userPrivacySettingRuleAllowUsers               => 'UserPrivacySettingRule',
    userPrivacySettingRuleRestrictAll              => 'UserPrivacySettingRule',
    userPrivacySettingRuleRestrictBots             => 'UserPrivacySettingRule',
    userPrivacySettingRuleRestrictChatMembers      => 'UserPrivacySettingRule',
    userPrivacySettingRuleRestrictContacts         => 'UserPrivacySettingRule',
    userPrivacySettingRuleRestrictUsers            => 'UserPrivacySettingRule',
    userPrivacySettingRules                        => 'Object',
    userPrivacySettingShowBio                      => 'UserPrivacySetting',
    userPrivacySettingShowBirthdate                => 'UserPrivacySetting',
    userPrivacySettingShowLinkInForwardedMessages  => 'UserPrivacySetting',
    userPrivacySettingShowPhoneNumber              => 'UserPrivacySetting',
    userPrivacySettingShowProfileAudio             => 'UserPrivacySetting',
    userPrivacySettingShowProfilePhoto             => 'UserPrivacySetting',
    userPrivacySettingShowStatus                   => 'UserPrivacySetting',
    userRating                                     => 'Object',
    userStatusEmpty                                => 'UserStatus',
    userStatusLastMonth                            => 'UserStatus',
    userStatusLastWeek                             => 'UserStatus',
    userStatusOffline                              => 'UserStatus',
    userStatusOnline                               => 'UserStatus',
    userStatusRecently                             => 'UserStatus',
    userSupportInfo                                => 'Object',
    userTypeBot                                    => 'UserType',
    userTypeDeleted                                => 'UserType',
    userTypeRegular                                => 'UserType',
    userTypeUnknown                                => 'UserType',
    usernames                                      => 'Object',
    users                                          => 'Object',
    validateOrderInfo                              => 'Function',
    validatedOrderInfo                             => 'Object',
    vectorPathCommandCubicBezierCurve              => 'VectorPathCommand',
    vectorPathCommandLine                          => 'VectorPathCommand',
    venue                                          => 'Object',
    verificationStatus                             => 'Object',
    video                                          => 'Object',
    videoChat                                      => 'Object',
    videoMessageAdvertisement                      => 'Object',
    videoMessageAdvertisements                     => 'Object',
    videoNote                                      => 'Object',
    videoStoryboard                                => 'Object',
    viewMessages                                   => 'Function',
    viewPremiumFeature                             => 'Function',
    viewSponsoredChat                              => 'Function',
    viewTrendingStickerSets                        => 'Function',
    viewVideoMessageAdvertisement                  => 'Function',
    voiceNote                                      => 'Object',
    webApp                                         => 'Object',
    webAppInfo                                     => 'Object',
    webAppOpenModeCompact                          => 'WebAppOpenMode',
    webAppOpenModeFullScreen                       => 'WebAppOpenMode',
    webAppOpenModeFullSize                         => 'WebAppOpenMode',
    webAppOpenParameters                           => 'Object',
    webAppUrl                                      => 'Object',
    webBrowserSettings                             => 'Object',
    webBrowserTypeExternal                         => 'WebBrowserType',
    webBrowserTypeInApp                            => 'WebBrowserType',
    webDomainException                             => 'Object',
    webPageInstantView                             => 'Object',
    writeGeneratedFilePart                         => 'Function',
);

our %CLASS_SLOTS = (
    acceptCall                                     => { protocol => 'callProtocol' },
    addBotMediaPreview                             => { content => 'InputStoryContent' },
    addChatToList                                  => { chat_list => 'ChatList' },
    addChecklistTasks                              => { tasks => 'array:inputChecklistTask' },
    addContact                                     => { contact => 'importedContact' },
    addFavoriteSticker                             => { sticker => 'InputFile' },
    addGiftCollectionGifts                         => { owner_id => 'MessageSender' },
    addLocalMessage                                => { input_message_content => 'InputMessageContent', reply_to => 'InputMessageReplyTo', sender_id => 'MessageSender' },
    addMessageReaction                             => { reaction_type => 'ReactionType' },
    addNetworkStatistics                           => { entry => 'NetworkStatisticsEntry' },
    addOffer                                       => { options => 'messageSendOptions' },
    addPendingPaidMessageReaction                  => { type => 'PaidReactionType' },
    addPollOption                                  => { option => 'inputPollOption' },
    addProfileAudio                                => { audio => 'inputAudio' },
    addProxy                                       => { proxy => 'proxy' },
    addQuickReplyShortcutMessage                   => { input_message_content => 'InputMessageContent' },
    addQuickReplyShortcutMessageAlbum              => { input_message_contents => 'array:InputMessageContent' },
    addRecentSticker                               => { sticker => 'InputFile' },
    addSavedAnimation                              => { animation => 'InputFile' },
    addSavedNotificationSound                      => { sound => 'InputFile' },
    addStickerToSet                                => { sticker => 'newSticker' },
    addedProxies                                   => { proxies => 'array:addedProxy' },
    addedProxy                                     => { proxy => 'proxy' },
    addedReaction                                  => { sender_id => 'MessageSender', type => 'ReactionType' },
    addedReactions                                 => { reactions => 'array:addedReaction' },
    advertisementSponsor                           => { photo => 'photo' },
    affiliateInfo                                  => { star_amount => 'starAmount' },
    affiliateProgramInfo                           => { daily_revenue_per_user_amount => 'starAmount', parameters => 'affiliateProgramParameters' },
    alternativeVideo                               => { hls_file => 'file', video => 'file' },
    animatedChatPhoto                              => { file => 'file' },
    animatedEmoji                                  => { sound => 'file', sticker => 'sticker' },
    animation                                      => { animation => 'file', minithumbnail => 'minithumbnail', thumbnail => 'thumbnail' },
    animations                                     => { animations => 'array:animation' },
    answerChatJoinRequestQuery                     => { result => 'ChatJoinRequestResult' },
    answerGuestQuery                               => { result => 'InputInlineQueryResult' },
    answerInlineQuery                              => { button => 'inlineQueryResultsButton', results => 'array:InputInlineQueryResult' },
    answerShippingQuery                            => { shipping_options => 'array:shippingOption' },
    answerWebAppQuery                              => { result => 'InputInlineQueryResult' },
    assignStoreTransaction                         => { purpose => 'StorePaymentPurpose', transaction => 'StoreTransaction' },
    attachmentMenuBot                              => { android_icon => 'file', android_side_menu_icon => 'file', default_icon => 'file', icon_color => 'attachmentMenuBotColor', ios_animated_icon => 'file', ios_side_menu_icon => 'file', ios_static_icon => 'file', macos_icon => 'file', macos_side_menu_icon => 'file', name_color => 'attachmentMenuBotColor', web_app_placeholder => 'file' },
    auctionStateActive                             => { bid_levels => 'array:auctionBid', rounds => 'array:auctionRound', user_bid => 'userAuctionBid' },
    audio                                          => { album_cover_minithumbnail => 'minithumbnail', album_cover_thumbnail => 'thumbnail', audio => 'file', external_album_covers => 'array:thumbnail' },
    audios                                         => { audios => 'array:audio' },
    authenticationCodeInfo                         => { next_type => 'AuthenticationCodeType', type => 'AuthenticationCodeType' },
    authenticationCodeTypeFirebaseAndroid          => { device_verification_parameters => 'FirebaseDeviceVerificationParameters' },
    authorizationStateWaitCode                     => { code_info => 'authenticationCodeInfo' },
    authorizationStateWaitEmailCode                => { code_info => 'emailAddressAuthenticationCodeInfo', email_address_reset_state => 'EmailAddressResetState' },
    authorizationStateWaitRegistration             => { terms_of_service => 'termsOfService' },
    autoDownloadSettingsPresets                    => { high => 'autoDownloadSettings', low => 'autoDownloadSettings', medium => 'autoDownloadSettings' },
    autosaveSettings                               => { channel_settings => 'scopeAutosaveSettings', exceptions => 'array:autosaveSettingsException', group_settings => 'scopeAutosaveSettings', private_chat_settings => 'scopeAutosaveSettings' },
    autosaveSettingsException                      => { settings => 'scopeAutosaveSettings' },
    availableGift                                  => { gift => 'gift' },
    availableGifts                                 => { gifts => 'array:availableGift' },
    availableReaction                              => { type => 'ReactionType' },
    availableReactions                             => { popular_reactions => 'array:availableReaction', recent_reactions => 'array:availableReaction', top_reactions => 'array:availableReaction', unavailability_reason => 'ReactionUnavailabilityReason' },
    background                                     => { document => 'document', type => 'BackgroundType' },
    backgroundTypeFill                             => { fill => 'BackgroundFill' },
    backgroundTypePattern                          => { fill => 'BackgroundFill' },
    backgrounds                                    => { backgrounds => 'array:background' },
    banChatMember                                  => { member_id => 'MessageSender' },
    bankCardInfo                                   => { actions => 'array:bankCardActionOpenUrl' },
    basicGroup                                     => { status => 'ChatMemberStatus' },
    basicGroupFullInfo                             => { bot_commands => 'array:botCommands', invite_link => 'chatInviteLink', members => 'array:chatMember', photo => 'chatPhoto' },
    botCommands                                    => { commands => 'array:botCommand' },
    botInfo                                        => { affiliate_program => 'affiliateProgramInfo', animation => 'animation', commands => 'array:botCommand', default_channel_administrator_rights => 'chatAdministratorRights', default_group_administrator_rights => 'chatAdministratorRights', edit_commands_link => 'InternalLinkType', edit_description_link => 'InternalLinkType', edit_description_media_link => 'InternalLinkType', edit_settings_link => 'InternalLinkType', menu_button => 'botMenuButton', photo => 'photo', verification_parameters => 'botVerificationParameters' },
    botMediaPreview                                => { content => 'StoryContent' },
    botMediaPreviewInfo                            => { previews => 'array:botMediaPreview' },
    botMediaPreviews                               => { previews => 'array:botMediaPreview' },
    botVerification                                => { custom_description => 'formattedText' },
    botVerificationParameters                      => { default_custom_description => 'formattedText' },
    botWriteAccessAllowReasonLaunchedWebApp        => { web_app => 'webApp' },
    businessAwayMessageSettings                    => { recipients => 'businessRecipients', schedule => 'BusinessAwayMessageSchedule' },
    businessChatLink                               => { text => 'formattedText' },
    businessChatLinkInfo                           => { text => 'formattedText' },
    businessChatLinks                              => { links => 'array:businessChatLink' },
    businessConnectedBot                           => { recipients => 'businessRecipients', rights => 'businessBotRights' },
    businessConnectedBotInfo                       => { bot => 'businessConnectedBot' },
    businessConnection                             => { rights => 'businessBotRights' },
    businessFeaturePromotionAnimation              => { animation => 'animation', feature => 'BusinessFeature' },
    businessFeatures                               => { features => 'array:BusinessFeature' },
    businessGreetingMessageSettings                => { recipients => 'businessRecipients' },
    businessInfo                                   => { away_message_settings => 'businessAwayMessageSettings', greeting_message_settings => 'businessGreetingMessageSettings', local_opening_hours => 'businessOpeningHours', location => 'businessLocation', opening_hours => 'businessOpeningHours', start_page => 'businessStartPage' },
    businessLocation                               => { location => 'location' },
    businessMessage                                => { message => 'message', reply_to_message => 'message' },
    businessMessages                               => { messages => 'array:businessMessage' },
    businessOpeningHours                           => { opening_hours => 'array:businessOpeningHoursInterval' },
    businessStartPage                              => { sticker => 'sticker' },
    buyGiftUpgrade                                 => { owner_id => 'MessageSender' },
    call                                           => { state => 'CallState' },
    callServer                                     => { type => 'CallServerType' },
    callStateDiscarded                             => { reason => 'CallDiscardReason' },
    callStateError                                 => { error => 'error' },
    callStateReady                                 => { protocol => 'callProtocol', servers => 'array:callServer' },
    canPurchaseFromStore                           => { purpose => 'StorePaymentPurpose' },
    canSendGiftResultFail                          => { reason => 'formattedText' },
    changeImportedContacts                         => { contacts => 'array:importedContact' },
    chat                                           => { action_bar => 'ChatActionBar', available_reactions => 'ChatAvailableReactions', background => 'chatBackground', block_list => 'BlockList', business_bot_manage_bar => 'businessBotManageBar', chat_lists => 'array:ChatList', draft_message => 'draftMessage', emoji_status => 'emojiStatus', last_message => 'message', message_sender_id => 'MessageSender', notification_settings => 'chatNotificationSettings', pending_join_requests => 'chatJoinRequestsInfo', permissions => 'chatPermissions', photo => 'chatPhotoInfo', positions => 'array:chatPosition', theme => 'ChatTheme', type => 'ChatType', upgraded_gift_colors => 'upgradedGiftColors', video_chat => 'videoChat' },
    chatActionBarReportAddBlock                    => { account_info => 'accountInfo' },
    chatActiveStories                              => { list => 'StoryList', stories => 'array:storyInfo' },
    chatAdministrators                             => { administrators => 'array:chatAdministrator' },
    chatAvailableReactionsSome                     => { reactions => 'array:ReactionType' },
    chatBackground                                 => { background => 'background' },
    chatBoost                                      => { source => 'ChatBoostSource' },
    chatBoostFeatures                              => { features => 'array:chatBoostLevelFeatures' },
    chatBoostSlots                                 => { slots => 'array:chatBoostSlot' },
    chatBoostStatus                                => { prepaid_giveaways => 'array:prepaidGiveaway' },
    chatEvent                                      => { action => 'ChatEventAction', member_id => 'MessageSender' },
    chatEventAvailableReactionsChanged             => { new_available_reactions => 'ChatAvailableReactions', old_available_reactions => 'ChatAvailableReactions' },
    chatEventBackgroundChanged                     => { new_background => 'chatBackground', old_background => 'chatBackground' },
    chatEventEmojiStatusChanged                    => { new_emoji_status => 'emojiStatus', old_emoji_status => 'emojiStatus' },
    chatEventForumTopicCreated                     => { topic_info => 'forumTopicInfo' },
    chatEventForumTopicDeleted                     => { topic_info => 'forumTopicInfo' },
    chatEventForumTopicEdited                      => { new_topic_info => 'forumTopicInfo', old_topic_info => 'forumTopicInfo' },
    chatEventForumTopicPinned                      => { new_topic_info => 'forumTopicInfo', old_topic_info => 'forumTopicInfo' },
    chatEventForumTopicToggleIsClosed              => { topic_info => 'forumTopicInfo' },
    chatEventForumTopicToggleIsHidden              => { topic_info => 'forumTopicInfo' },
    chatEventInviteLinkDeleted                     => { invite_link => 'chatInviteLink' },
    chatEventInviteLinkEdited                      => { new_invite_link => 'chatInviteLink', old_invite_link => 'chatInviteLink' },
    chatEventInviteLinkRevoked                     => { invite_link => 'chatInviteLink' },
    chatEventLocationChanged                       => { new_location => 'chatLocation', old_location => 'chatLocation' },
    chatEventMemberInvited                         => { status => 'ChatMemberStatus' },
    chatEventMemberJoinedByInviteLink              => { invite_link => 'chatInviteLink' },
    chatEventMemberJoinedByRequest                 => { invite_link => 'chatInviteLink' },
    chatEventMemberPromoted                        => { new_status => 'ChatMemberStatus', old_status => 'ChatMemberStatus' },
    chatEventMemberRestricted                      => { member_id => 'MessageSender', new_status => 'ChatMemberStatus', old_status => 'ChatMemberStatus' },
    chatEventMemberSubscriptionExtended            => { new_status => 'ChatMemberStatus', old_status => 'ChatMemberStatus' },
    chatEventMessageDeleted                        => { message => 'message' },
    chatEventMessageEdited                         => { new_message => 'message', old_message => 'message' },
    chatEventMessagePinned                         => { message => 'message' },
    chatEventMessageUnpinned                       => { message => 'message' },
    chatEventPermissionsChanged                    => { new_permissions => 'chatPermissions', old_permissions => 'chatPermissions' },
    chatEventPhotoChanged                          => { new_photo => 'chatPhoto', old_photo => 'chatPhoto' },
    chatEventPollStopped                           => { message => 'message' },
    chatEventVideoChatParticipantIsMutedToggled    => { participant_id => 'MessageSender' },
    chatEventVideoChatParticipantVolumeLevelChanged => { participant_id => 'MessageSender' },
    chatEvents                                     => { events => 'array:chatEvent' },
    chatFolder                                     => { icon => 'chatFolderIcon', name => 'chatFolderName' },
    chatFolderInfo                                 => { icon => 'chatFolderIcon', name => 'chatFolderName' },
    chatFolderInviteLinkInfo                       => { chat_folder_info => 'chatFolderInfo' },
    chatFolderInviteLinks                          => { invite_links => 'array:chatFolderInviteLink' },
    chatFolderName                                 => { text => 'formattedText' },
    chatInviteLink                                 => { subscription_pricing => 'starSubscriptionPricing' },
    chatInviteLinkCounts                           => { invite_link_counts => 'array:chatInviteLinkCount' },
    chatInviteLinkInfo                             => { photo => 'chatPhotoInfo', subscription_info => 'chatInviteLinkSubscriptionInfo', type => 'InviteLinkChatType', verification_status => 'verificationStatus' },
    chatInviteLinkMembers                          => { members => 'array:chatInviteLinkMember' },
    chatInviteLinkSubscriptionInfo                 => { pricing => 'starSubscriptionPricing' },
    chatInviteLinks                                => { invite_links => 'array:chatInviteLink' },
    chatJoinRequests                               => { requests => 'array:chatJoinRequest' },
    chatLists                                      => { chat_lists => 'array:ChatList' },
    chatLocation                                   => { location => 'location' },
    chatMember                                     => { member_id => 'MessageSender', status => 'ChatMemberStatus' },
    chatMemberStatusAdministrator                  => { rights => 'chatAdministratorRights' },
    chatMemberStatusRestricted                     => { permissions => 'chatPermissions' },
    chatMembers                                    => { members => 'array:chatMember' },
    chatMembersFilterMention                       => { topic_id => 'MessageTopic' },
    chatMessageSender                              => { sender => 'MessageSender' },
    chatMessageSenders                             => { senders => 'array:chatMessageSender' },
    chatPhoto                                      => { animation => 'animatedChatPhoto', minithumbnail => 'minithumbnail', sizes => 'array:photoSize', small_animation => 'animatedChatPhoto', sticker => 'chatPhotoSticker' },
    chatPhotoInfo                                  => { big => 'file', minithumbnail => 'minithumbnail', small => 'file' },
    chatPhotoSticker                               => { background_fill => 'BackgroundFill', type => 'ChatPhotoStickerType' },
    chatPhotos                                     => { photos => 'array:chatPhoto' },
    chatPosition                                   => { list => 'ChatList', source => 'ChatSource' },
    chatRevenueStatistics                          => { revenue_amount => 'chatRevenueAmount', revenue_by_hour_graph => 'StatisticalGraph', revenue_graph => 'StatisticalGraph' },
    chatRevenueTransaction                         => { type => 'ChatRevenueTransactionType' },
    chatRevenueTransactionTypeFragmentWithdrawal   => { state => 'RevenueWithdrawalState' },
    chatRevenueTransactions                        => { transactions => 'array:chatRevenueTransaction' },
    chatStatisticsChannel                          => { instant_view_interaction_graph => 'StatisticalGraph', join_by_source_graph => 'StatisticalGraph', join_graph => 'StatisticalGraph', language_graph => 'StatisticalGraph', mean_message_reaction_count => 'statisticalValue', mean_message_share_count => 'statisticalValue', mean_message_view_count => 'statisticalValue', mean_story_reaction_count => 'statisticalValue', mean_story_share_count => 'statisticalValue', mean_story_view_count => 'statisticalValue', member_count => 'statisticalValue', member_count_graph => 'StatisticalGraph', message_interaction_graph => 'StatisticalGraph', message_reaction_graph => 'StatisticalGraph', mute_graph => 'StatisticalGraph', period => 'dateRange', recent_interactions => 'array:chatStatisticsInteractionInfo', story_interaction_graph => 'StatisticalGraph', story_reaction_graph => 'StatisticalGraph', view_count_by_hour_graph => 'StatisticalGraph', view_count_by_source_graph => 'StatisticalGraph' },
    chatStatisticsInteractionInfo                  => { object_type => 'ChatStatisticsObjectType' },
    chatStatisticsSupergroup                       => { action_graph => 'StatisticalGraph', day_graph => 'StatisticalGraph', join_by_source_graph => 'StatisticalGraph', join_graph => 'StatisticalGraph', language_graph => 'StatisticalGraph', member_count => 'statisticalValue', member_count_graph => 'StatisticalGraph', message_content_graph => 'StatisticalGraph', message_count => 'statisticalValue', period => 'dateRange', sender_count => 'statisticalValue', top_administrators => 'array:chatStatisticsAdministratorActionsInfo', top_inviters => 'array:chatStatisticsInviterInfo', top_senders => 'array:chatStatisticsMessageSenderInfo', viewer_count => 'statisticalValue', week_graph => 'StatisticalGraph' },
    chatThemeGift                                  => { gift_theme => 'giftChatTheme' },
    checkAuthenticationEmailCode                   => { code => 'EmailAddressAuthentication' },
    checkCreatedPublicChatsLimit                   => { type => 'PublicChatType' },
    checkLoginEmailAddressCode                     => { code => 'EmailAddressAuthentication' },
    checklist                                      => { tasks => 'array:checklistTask', title => 'formattedText' },
    checklistTask                                  => { completed_by => 'MessageSender', text => 'formattedText' },
    closeBirthdayUser                              => { birthdate => 'birthdate' },
    closedVectorPath                               => { commands => 'array:VectorPathCommand' },
    community                                      => { permissions => 'communityPermissions', photo => 'chatPhotoInfo', status => 'CommunityMemberStatus' },
    communityMemberStatusAdministrator             => { rights => 'communityAdministratorRights' },
    composeRichMessageWithAi                       => { message => 'inputRichMessage' },
    composeTextWithAi                              => { text => 'formattedText' },
    connectAffiliateProgram                        => { affiliate => 'AffiliateType' },
    connectedAffiliateProgram                      => { parameters => 'affiliateProgramParameters' },
    connectedAffiliatePrograms                     => { programs => 'array:connectedAffiliateProgram' },
    connectedWebsites                              => { websites => 'array:connectedWebsite' },
    countries                                      => { countries => 'array:countryInfo' },
    craftGiftResultSuccess                         => { gift => 'upgradedGift' },
    createBusinessChatLink                         => { link_info => 'inputBusinessChatLink' },
    createCall                                     => { protocol => 'callProtocol' },
    createChatFolder                               => { folder => 'chatFolder' },
    createChatSubscriptionInviteLink               => { subscription_pricing => 'starSubscriptionPricing' },
    createForumTopic                               => { icon => 'forumTopicIcon' },
    createGiftCollection                           => { owner_id => 'MessageSender' },
    createGroupCall                                => { join_parameters => 'groupCallJoinParameters' },
    createInvoiceLink                              => { invoice => 'InputMessageContent' },
    createNewStickerSet                            => { sticker_type => 'StickerType', stickers => 'array:newSticker' },
    createNewSupergroupChat                        => { location => 'chatLocation' },
    createdBasicGroupChat                          => { failed_to_add_members => 'failedToAddMembers' },
    dateTimeFormattingTypeAbsolute                 => { date_precision => 'DateTimePartPrecision', time_precision => 'DateTimePartPrecision' },
    datedFile                                      => { file => 'file' },
    decryptGroupCallData                           => { data_channel => 'GroupCallDataChannel', participant_id => 'MessageSender' },
    deepLinkInfo                                   => { text => 'formattedText' },
    deleteAllRecentMessageReactionsFromSender      => { sender_id => 'MessageSender' },
    deleteChatMessagesBySender                     => { sender_id => 'MessageSender' },
    deleteCommands                                 => { scope => 'BotCommandScope' },
    deleteGiftCollection                           => { owner_id => 'MessageSender' },
    deleteGroupCallMessagesBySender                => { sender_id => 'MessageSender' },
    deleteMessageReactionsFromSender               => { sender_id => 'MessageSender' },
    deletePassportElement                          => { type => 'PassportElementType' },
    diceStickersRegular                            => { sticker => 'sticker' },
    diceStickersSlotMachine                        => { background => 'sticker', center_reel => 'sticker', left_reel => 'sticker', lever => 'sticker', right_reel => 'sticker' },
    diffEntity                                     => { type => 'DiffEntityType' },
    diffText                                       => { entities => 'array:diffEntity' },
    directMessagesChatTopic                        => { draft_message => 'draftMessage', last_message => 'message', sender_id => 'MessageSender' },
    disconnectAffiliateProgram                     => { affiliate => 'AffiliateType' },
    document                                       => { document => 'file', minithumbnail => 'minithumbnail', thumbnail => 'thumbnail' },
    draftMessage                                   => { content => 'DraftMessageContent', reply_to => 'InputMessageReplyTo', suggested_post_info => 'inputSuggestedPostInfo' },
    draftMessageContentRichMessage                 => { message => 'richMessage' },
    draftMessageContentText                        => { link_preview_options => 'linkPreviewOptions', text => 'formattedText' },
    draftMessageContentVideoNote                   => { self_destruct_type => 'MessageSelfDestructType' },
    draftMessageContentVoiceNote                   => { self_destruct_type => 'MessageSelfDestructType' },
    editBotMediaPreview                            => { content => 'InputStoryContent' },
    editBusinessChatLink                           => { link_info => 'inputBusinessChatLink' },
    editBusinessMessageCaption                     => { caption => 'formattedText', reply_markup => 'ReplyMarkup' },
    editBusinessMessageChecklist                   => { checklist => 'inputChecklist', reply_markup => 'ReplyMarkup' },
    editBusinessMessageLiveLocation                => { location => 'liveLocation', reply_markup => 'ReplyMarkup' },
    editBusinessMessageMedia                       => { input_message_content => 'InputMessageContent', reply_markup => 'ReplyMarkup' },
    editBusinessMessageReplyMarkup                 => { reply_markup => 'ReplyMarkup' },
    editBusinessMessageText                        => { input_message_content => 'InputMessageContent', reply_markup => 'ReplyMarkup' },
    editBusinessStory                              => { areas => 'inputStoryAreas', caption => 'formattedText', content => 'InputStoryContent', privacy_settings => 'StoryPrivacySettings' },
    editChatFolder                                 => { folder => 'chatFolder' },
    editCustomLanguagePackInfo                     => { info => 'languagePackInfo' },
    editEphemeralMessage                           => { input_message_content => 'InputMessageContent', reply_markup => 'ReplyMarkup' },
    editInlineMessageCaption                       => { caption => 'formattedText', reply_markup => 'ReplyMarkup' },
    editInlineMessageLiveLocation                  => { location => 'liveLocation', reply_markup => 'ReplyMarkup' },
    editInlineMessageMedia                         => { input_message_content => 'InputMessageContent', reply_markup => 'ReplyMarkup' },
    editInlineMessageReplyMarkup                   => { reply_markup => 'ReplyMarkup' },
    editInlineMessageText                          => { input_message_content => 'InputMessageContent', reply_markup => 'ReplyMarkup' },
    editMessageCaption                             => { caption => 'formattedText', reply_markup => 'ReplyMarkup' },
    editMessageChecklist                           => { checklist => 'inputChecklist', reply_markup => 'ReplyMarkup' },
    editMessageLiveLocation                        => { location => 'liveLocation', reply_markup => 'ReplyMarkup' },
    editMessageMedia                               => { input_message_content => 'InputMessageContent', reply_markup => 'ReplyMarkup' },
    editMessageReplyMarkup                         => { reply_markup => 'ReplyMarkup' },
    editMessageSchedulingState                     => { scheduling_state => 'MessageSchedulingState' },
    editMessageText                                => { input_message_content => 'InputMessageContent', reply_markup => 'ReplyMarkup' },
    editProxy                                      => { proxy => 'proxy' },
    editQuickReplyMessage                          => { input_message_content => 'InputMessageContent' },
    editStory                                      => { areas => 'inputStoryAreas', caption => 'formattedText', content => 'InputStoryContent' },
    emojiCategories                                => { categories => 'array:emojiCategory' },
    emojiCategory                                  => { icon => 'sticker', source => 'EmojiCategorySource' },
    emojiChatTheme                                 => { dark_settings => 'themeSettings', light_settings => 'themeSettings' },
    emojiKeywords                                  => { emoji_keywords => 'array:emojiKeyword' },
    emojiReaction                                  => { activate_animation => 'sticker', appear_animation => 'sticker', around_animation => 'sticker', center_animation => 'sticker', effect_animation => 'sticker', select_animation => 'sticker', static_icon => 'sticker' },
    emojiStatus                                    => { type => 'EmojiStatusType' },
    emojiStatusTypeUpgradedGift                    => { backdrop_colors => 'upgradedGiftBackdropColors' },
    emojiStatuses                                  => { emoji_statuses => 'array:emojiStatus' },
    encryptGroupCallData                           => { data_channel => 'GroupCallDataChannel' },
    encryptedPassportElement                       => { files => 'array:datedFile', front_side => 'datedFile', reverse_side => 'datedFile', selfie => 'datedFile', translation => 'array:datedFile', type => 'PassportElementType' },
    factCheck                                      => { text => 'formattedText' },
    failedToAddMembers                             => { failed_to_add_members => 'array:failedToAddMember' },
    file                                           => { local => 'localFile', remote => 'remoteFile' },
    fileDownload                                   => { message => 'message' },
    finishFileGeneration                           => { error => 'error' },
    fixRichMessageWithAi                           => { message => 'inputRichMessage' },
    fixTextWithAi                                  => { text => 'formattedText' },
    fixedText                                      => { diff_text => 'diffText', text => 'formattedText' },
    formattedText                                  => { entities => 'array:textEntity' },
    forumTopic                                     => { draft_message => 'draftMessage', info => 'forumTopicInfo', last_message => 'message', notification_settings => 'chatNotificationSettings' },
    forumTopicInfo                                 => { creator_id => 'MessageSender', icon => 'forumTopicIcon' },
    forumTopics                                    => { topics => 'array:forumTopic' },
    forwardMessages                                => { options => 'messageSendOptions', topic_id => 'MessageTopic' },
    forwardSource                                  => { sender_id => 'MessageSender' },
    foundAffiliateProgram                          => { info => 'affiliateProgramInfo' },
    foundAffiliatePrograms                         => { programs => 'array:foundAffiliateProgram' },
    foundChatBoosts                                => { boosts => 'array:chatBoost' },
    foundChatMessages                              => { messages => 'array:message' },
    foundFileDownloads                             => { files => 'array:fileDownload', total_counts => 'downloadedFileCounts' },
    foundMessages                                  => { messages => 'array:message' },
    foundPublicPosts                               => { messages => 'array:message', search_limits => 'publicPostSearchLimits' },
    foundStories                                   => { stories => 'array:story' },
    foundWebApp                                    => { web_app => 'webApp' },
    game                                           => { animation => 'animation', photo => 'photo', text => 'formattedText' },
    gameHighScores                                 => { scores => 'array:gameHighScore' },
    getAllStickerEmojis                            => { sticker_type => 'StickerType' },
    getArchivedStickerSets                         => { sticker_type => 'StickerType' },
    getBackgroundUrl                               => { type => 'BackgroundType' },
    getBlockedMessageSenders                       => { block_list => 'BlockList' },
    getBusinessFeatures                            => { source => 'BusinessFeature' },
    getCallbackQueryAnswer                         => { payload => 'CallbackQueryPayload' },
    getChatEventLog                                => { filters => 'chatEventLogFilters' },
    getChatFolderChatCount                         => { folder => 'chatFolder' },
    getChatFolderDefaultIconName                   => { folder => 'chatFolder' },
    getChatInviteLinkMembers                       => { offset_member => 'chatInviteLinkMember' },
    getChatJoinRequests                            => { offset_request => 'chatJoinRequest' },
    getChatMember                                  => { member_id => 'MessageSender' },
    getChatMessageCalendar                         => { filter => 'SearchMessagesFilter', topic_id => 'MessageTopic' },
    getChatMessageCount                            => { filter => 'SearchMessagesFilter', topic_id => 'MessageTopic' },
    getChatMessagePosition                         => { filter => 'SearchMessagesFilter', topic_id => 'MessageTopic' },
    getChatNotificationSettingsExceptions          => { scope => 'NotificationSettingsScope' },
    getChatSparseMessagePositions                  => { filter => 'SearchMessagesFilter' },
    getChatStoryInteractions                       => { reaction_type => 'ReactionType' },
    getChats                                       => { chat_list => 'ChatList' },
    getCollectibleItemInfo                         => { type => 'CollectibleItemType' },
    getCommands                                    => { scope => 'BotCommandScope' },
    getConnectedAffiliateProgram                   => { affiliate => 'AffiliateType' },
    getConnectedAffiliatePrograms                  => { affiliate => 'AffiliateType' },
    getCreatedPublicChats                          => { type => 'PublicChatType' },
    getCurrentWeather                              => { location => 'location' },
    getEmojiCategories                             => { type => 'EmojiCategoryType' },
    getGiftCollections                             => { owner_id => 'MessageSender' },
    getGroupCallParticipants                       => { input_group_call => 'InputGroupCall' },
    getGroupCallStreamSegment                      => { video_quality => 'GroupCallVideoQuality' },
    getGuardBotWebAppUrl                           => { parameters => 'webAppOpenParameters' },
    getInlineQueryResults                          => { user_location => 'location' },
    getInstalledStickerSets                        => { sticker_type => 'StickerType' },
    getInternalLink                                => { type => 'InternalLinkType' },
    getJsonString                                  => { json_value => 'JsonValue' },
    getLinkPreview                                 => { link_preview_options => 'linkPreviewOptions', text => 'formattedText' },
    getMainWebApp                                  => { parameters => 'webAppOpenParameters' },
    getMapThumbnailFile                            => { location => 'location' },
    getMarkdownText                                => { text => 'formattedText' },
    getMessageAddedReactions                       => { reaction_type => 'ReactionType' },
    getPassportElement                             => { type => 'PassportElementType' },
    getPaymentForm                                 => { input_invoice => 'InputInvoice', theme => 'themeParameters' },
    getPremiumFeatures                             => { source => 'PremiumSource' },
    getPremiumLimit                                => { limit_type => 'PremiumLimitType' },
    getReceivedGifts                               => { owner_id => 'MessageSender' },
    getRemoteFile                                  => { file_type => 'FileType' },
    getScopeNotificationSettings                   => { scope => 'NotificationSettingsScope' },
    getStarAdAccountUrl                            => { owner_id => 'MessageSender' },
    getStarRevenueStatistics                       => { owner_id => 'MessageSender' },
    getStarTransactions                            => { direction => 'TransactionDirection', owner_id => 'MessageSender' },
    getStarWithdrawalUrl                           => { owner_id => 'MessageSender' },
    getStickerEmojis                               => { sticker => 'InputFile' },
    getStickers                                    => { sticker_type => 'StickerType' },
    getSupergroupMembers                           => { filter => 'SupergroupMembersFilter' },
    getThemeParametersJsonString                   => { theme => 'themeParameters' },
    getTonTransactions                             => { direction => 'TransactionDirection' },
    getTopChats                                    => { category => 'TopChatCategory' },
    getTrendingStickerSets                         => { sticker_type => 'StickerType' },
    getUserPrivacySettingRules                     => { setting => 'UserPrivacySetting' },
    getWebAppLinkUrl                               => { parameters => 'webAppOpenParameters' },
    getWebAppUrl                                   => { parameters => 'webAppOpenParameters' },
    gift                                           => { auction_info => 'giftAuction', background => 'giftBackground', overall_limits => 'giftPurchaseLimits', sticker => 'sticker', user_limits => 'giftPurchaseLimits' },
    giftAuctionAcquiredGift                        => { receiver_id => 'MessageSender', text => 'formattedText' },
    giftAuctionAcquiredGifts                       => { gifts => 'array:giftAuctionAcquiredGift' },
    giftAuctionState                               => { gift => 'gift', state => 'AuctionState' },
    giftChatTheme                                  => { dark_settings => 'themeSettings', gift => 'upgradedGift', light_settings => 'themeSettings' },
    giftChatThemes                                 => { themes => 'array:giftChatTheme' },
    giftCollection                                 => { icon => 'sticker' },
    giftCollections                                => { collections => 'array:giftCollection' },
    giftForResale                                  => { gift => 'upgradedGift' },
    giftPremiumWithStars                           => { text => 'formattedText' },
    giftResaleResultPriceIncreased                 => { price => 'GiftResalePrice' },
    giftSettings                                   => { accepted_gift_types => 'acceptedGiftTypes' },
    giftUpgradePreview                             => { backdrops => 'array:upgradedGiftBackdrop', models => 'array:upgradedGiftModel', next_prices => 'array:giftUpgradePrice', prices => 'array:giftUpgradePrice', symbols => 'array:upgradedGiftSymbol' },
    giftUpgradeVariants                            => { backdrops => 'array:upgradedGiftBackdrop', models => 'array:upgradedGiftModel', symbols => 'array:upgradedGiftSymbol' },
    giftsForCrafting                               => { attribute_persistence_probabilities => 'array:attributeCraftPersistenceProbability', gifts => 'array:receivedGift' },
    giftsForResale                                 => { backdrops => 'array:upgradedGiftBackdropCount', gifts => 'array:giftForResale', models => 'array:upgradedGiftModelCount', symbols => 'array:upgradedGiftSymbolCount' },
    giveawayInfoOngoing                            => { status => 'GiveawayParticipantStatus' },
    gramRevenueStatistics                          => { revenue_by_day_graph => 'StatisticalGraph', status => 'gramRevenueStatus' },
    groupCall                                      => { message_sender_id => 'MessageSender', recent_speakers => 'array:groupCallRecentSpeaker' },
    groupCallMessage                               => { sender_id => 'MessageSender', text => 'formattedText' },
    groupCallParticipant                           => { participant_id => 'MessageSender', screen_sharing_video_info => 'groupCallParticipantVideoInfo', video_info => 'groupCallParticipantVideoInfo' },
    groupCallParticipantVideoInfo                  => { source_groups => 'array:groupCallVideoSourceGroup' },
    groupCallParticipants                          => { participant_ids => 'array:MessageSender' },
    groupCallRecentSpeaker                         => { participant_id => 'MessageSender' },
    groupCallStreams                               => { streams => 'array:groupCallStream' },
    hideSuggestedAction                            => { action => 'SuggestedAction' },
    identityDocument                               => { expiration_date => 'date', front_side => 'datedFile', reverse_side => 'datedFile', selfie => 'datedFile', translation => 'array:datedFile' },
    importContacts                                 => { contacts => 'array:importedContact' },
    importMessages                                 => { attached_files => 'array:InputFile', message_file => 'InputFile' },
    importedContact                                => { note => 'formattedText' },
    inlineKeyboardButton                           => { style => 'ButtonStyle', type => 'InlineKeyboardButtonType' },
    inlineKeyboardButtonTypeSwitchInline           => { target_chat => 'TargetChat' },
    inlineQueryResultAnimation                     => { animation => 'animation' },
    inlineQueryResultArticle                       => { thumbnail => 'thumbnail' },
    inlineQueryResultAudio                         => { audio => 'audio' },
    inlineQueryResultContact                       => { contact => 'contact', thumbnail => 'thumbnail' },
    inlineQueryResultDocument                      => { document => 'document' },
    inlineQueryResultGame                          => { game => 'game' },
    inlineQueryResultLocation                      => { location => 'location', thumbnail => 'thumbnail' },
    inlineQueryResultPhoto                         => { photo => 'photo' },
    inlineQueryResultSticker                       => { sticker => 'sticker' },
    inlineQueryResultVenue                         => { thumbnail => 'thumbnail', venue => 'venue' },
    inlineQueryResultVideo                         => { video => 'video' },
    inlineQueryResultVoiceNote                     => { voice_note => 'voiceNote' },
    inlineQueryResults                             => { button => 'inlineQueryResultsButton', results => 'array:InlineQueryResult' },
    inlineQueryResultsButton                       => { type => 'InlineQueryResultsButtonType' },
    inputAnimation                                 => { animation => 'InputFile', thumbnail => 'inputThumbnail' },
    inputAudio                                     => { album_cover_thumbnail => 'inputThumbnail', audio => 'InputFile' },
    inputBackgroundLocal                           => { background => 'InputFile' },
    inputBusinessChatLink                          => { text => 'formattedText' },
    inputBusinessStartPage                         => { sticker => 'InputFile' },
    inputChatPhotoAnimation                        => { animation => 'InputFile' },
    inputChatPhotoStatic                           => { photo => 'InputFile' },
    inputChatPhotoSticker                          => { sticker => 'chatPhotoSticker' },
    inputChecklist                                 => { tasks => 'array:inputChecklistTask', title => 'formattedText' },
    inputChecklistTask                             => { text => 'formattedText' },
    inputDocument                                  => { document => 'InputFile', thumbnail => 'inputThumbnail' },
    inputIdentityDocument                          => { expiration_date => 'date', front_side => 'InputFile', reverse_side => 'InputFile', selfie => 'InputFile', translation => 'array:InputFile' },
    inputInlineQueryResultAnimation                => { input_message_content => 'InputMessageContent', reply_markup => 'ReplyMarkup' },
    inputInlineQueryResultArticle                  => { input_message_content => 'InputMessageContent', reply_markup => 'ReplyMarkup' },
    inputInlineQueryResultAudio                    => { input_message_content => 'InputMessageContent', reply_markup => 'ReplyMarkup' },
    inputInlineQueryResultContact                  => { contact => 'contact', input_message_content => 'InputMessageContent', reply_markup => 'ReplyMarkup' },
    inputInlineQueryResultDocument                 => { input_message_content => 'InputMessageContent', reply_markup => 'ReplyMarkup' },
    inputInlineQueryResultGame                     => { reply_markup => 'ReplyMarkup' },
    inputInlineQueryResultLocation                 => { input_message_content => 'InputMessageContent', location => 'location', reply_markup => 'ReplyMarkup' },
    inputInlineQueryResultPhoto                    => { input_message_content => 'InputMessageContent', reply_markup => 'ReplyMarkup' },
    inputInlineQueryResultSticker                  => { input_message_content => 'InputMessageContent', reply_markup => 'ReplyMarkup' },
    inputInlineQueryResultVenue                    => { input_message_content => 'InputMessageContent', reply_markup => 'ReplyMarkup', venue => 'venue' },
    inputInlineQueryResultVideo                    => { input_message_content => 'InputMessageContent', reply_markup => 'ReplyMarkup' },
    inputInlineQueryResultVoiceNote                => { input_message_content => 'InputMessageContent', reply_markup => 'ReplyMarkup' },
    inputInvoiceTelegram                           => { purpose => 'TelegramPaymentPurpose' },
    inputMessageAnimation                          => { animation => 'inputAnimation', caption => 'formattedText' },
    inputMessageAudio                              => { audio => 'inputAudio', caption => 'formattedText' },
    inputMessageChecklist                          => { checklist => 'inputChecklist' },
    inputMessageContact                            => { contact => 'contact' },
    inputMessageDocument                           => { caption => 'formattedText', document => 'inputDocument' },
    inputMessageForwarded                          => { copy_options => 'messageCopyOptions' },
    inputMessageInvoice                            => { invoice => 'invoice', paid_media => 'inputPaidMedia', paid_media_caption => 'formattedText' },
    inputMessageLiveLocation                       => { location => 'liveLocation' },
    inputMessageLocation                           => { location => 'location' },
    inputMessagePaidMedia                          => { caption => 'formattedText', paid_media => 'array:inputPaidMedia' },
    inputMessagePhoto                              => { caption => 'formattedText', photo => 'inputPhoto', self_destruct_type => 'MessageSelfDestructType' },
    inputMessagePoll                               => { description => 'formattedText', media => 'InputPollMedia', options => 'array:inputPollOption', question => 'formattedText', type => 'InputPollType' },
    inputMessageReplyToExternalMessage             => { quote => 'inputTextQuote' },
    inputMessageReplyToMessage                     => { quote => 'inputTextQuote' },
    inputMessageRichMessage                        => { message => 'inputRichMessage' },
    inputMessageSticker                            => { sticker => 'inputSticker' },
    inputMessageText                               => { link_preview_options => 'linkPreviewOptions', text => 'formattedText' },
    inputMessageVenue                              => { venue => 'venue' },
    inputMessageVideo                              => { caption => 'formattedText', self_destruct_type => 'MessageSelfDestructType', video => 'inputVideo' },
    inputMessageVideoNote                          => { self_destruct_type => 'MessageSelfDestructType', video_note => 'inputVideoNote' },
    inputMessageVoiceNote                          => { caption => 'formattedText', self_destruct_type => 'MessageSelfDestructType', voice_note => 'inputVoiceNote' },
    inputPageBlockAnimation                        => { animation => 'inputAnimation', caption => 'pageBlockCaption' },
    inputPageBlockAudio                            => { audio => 'inputAudio', caption => 'pageBlockCaption' },
    inputPageBlockBlockQuote                       => { blocks => 'array:InputPageBlock', credit => 'RichText' },
    inputPageBlockCollage                          => { blocks => 'array:InputPageBlock', caption => 'pageBlockCaption' },
    inputPageBlockDetails                          => { blocks => 'array:InputPageBlock', header => 'RichText' },
    inputPageBlockFooter                           => { footer => 'RichText' },
    inputPageBlockList                             => { items => 'array:inputPageBlockListItem' },
    inputPageBlockListItem                         => { blocks => 'array:InputPageBlock' },
    inputPageBlockMap                              => { caption => 'pageBlockCaption', location => 'location' },
    inputPageBlockParagraph                        => { text => 'RichText' },
    inputPageBlockPhoto                            => { caption => 'pageBlockCaption', photo => 'inputPhoto' },
    inputPageBlockPreformatted                     => { text => 'RichText' },
    inputPageBlockPullQuote                        => { credit => 'RichText', text => 'RichText' },
    inputPageBlockSectionHeading                   => { text => 'RichText' },
    inputPageBlockSlideshow                        => { blocks => 'array:InputPageBlock', caption => 'pageBlockCaption' },
    inputPageBlockTable                            => { caption => 'RichText' },
    inputPageBlockThinking                         => { text => 'RichText' },
    inputPageBlockVideo                            => { caption => 'pageBlockCaption', video => 'inputVideo' },
    inputPageBlockVoiceNote                        => { caption => 'pageBlockCaption', voice_note => 'inputVoiceNote' },
    inputPaidMedia                                 => { media => 'InputFile', thumbnail => 'inputThumbnail', type => 'InputPaidMediaType' },
    inputPaidMediaTypePhoto                        => { video => 'InputFile' },
    inputPaidMediaTypeVideo                        => { cover => 'InputFile' },
    inputPassportElementAddress                    => { address => 'address' },
    inputPassportElementBankStatement              => { bank_statement => 'inputPersonalDocument' },
    inputPassportElementDriverLicense              => { driver_license => 'inputIdentityDocument' },
    inputPassportElementError                      => { source => 'InputPassportElementErrorSource', type => 'PassportElementType' },
    inputPassportElementIdentityCard               => { identity_card => 'inputIdentityDocument' },
    inputPassportElementInternalPassport           => { internal_passport => 'inputIdentityDocument' },
    inputPassportElementPassport                   => { passport => 'inputIdentityDocument' },
    inputPassportElementPassportRegistration       => { passport_registration => 'inputPersonalDocument' },
    inputPassportElementPersonalDetails            => { personal_details => 'personalDetails' },
    inputPassportElementRentalAgreement            => { rental_agreement => 'inputPersonalDocument' },
    inputPassportElementTemporaryRegistration      => { temporary_registration => 'inputPersonalDocument' },
    inputPassportElementUtilityBill                => { utility_bill => 'inputPersonalDocument' },
    inputPersonalDocument                          => { files => 'array:InputFile', translation => 'array:InputFile' },
    inputPhoto                                     => { photo => 'InputFile', thumbnail => 'inputThumbnail', video => 'InputFile' },
    inputPollMediaAnimation                        => { animation => 'inputAnimation' },
    inputPollMediaAudio                            => { audio => 'inputAudio' },
    inputPollMediaDocument                         => { document => 'inputDocument' },
    inputPollMediaLocation                         => { location => 'location' },
    inputPollMediaPhoto                            => { photo => 'inputPhoto' },
    inputPollMediaSticker                          => { sticker => 'inputSticker' },
    inputPollMediaVenue                            => { venue => 'venue' },
    inputPollMediaVideo                            => { video => 'inputVideo' },
    inputPollOption                                => { media => 'InputPollMedia', text => 'formattedText' },
    inputPollTypeQuiz                              => { explanation => 'formattedText', explanation_media => 'InputPollMedia' },
    inputRichMessage                               => { source => 'RichMessageSource' },
    inputRichMessageMedia                          => { media => 'InputMessageContent' },
    inputSticker                                   => { sticker => 'InputFile', thumbnail => 'inputThumbnail' },
    inputStoryArea                                 => { position => 'storyAreaPosition', type => 'InputStoryAreaType' },
    inputStoryAreaTypeLocation                     => { address => 'locationAddress', location => 'location' },
    inputStoryAreaTypeSuggestedReaction            => { reaction_type => 'ReactionType' },
    inputStoryAreas                                => { areas => 'array:inputStoryArea' },
    inputStoryContentPhoto                         => { photo => 'InputFile' },
    inputStoryContentVideo                         => { video => 'InputFile' },
    inputSuggestedPostInfo                         => { price => 'SuggestedPostPrice' },
    inputTextQuote                                 => { text => 'formattedText' },
    inputThumbnail                                 => { thumbnail => 'InputFile' },
    inputVideo                                     => { cover => 'InputFile', thumbnail => 'inputThumbnail', video => 'InputFile' },
    inputVideoNote                                 => { thumbnail => 'inputThumbnail', video_note => 'InputFile' },
    inputVoiceNote                                 => { voice_note => 'InputFile' },
    internalLinkTypeAttachmentMenuBot              => { target_chat => 'TargetChat' },
    internalLinkTypeBotAddToChannel                => { administrator_rights => 'chatAdministratorRights' },
    internalLinkTypeBotStartInGroup                => { administrator_rights => 'chatAdministratorRights' },
    internalLinkTypeMainWebApp                     => { mode => 'WebAppOpenMode' },
    internalLinkTypeMessageDraft                   => { text => 'formattedText' },
    internalLinkTypeNewStory                       => { content_type => 'StoryContentType' },
    internalLinkTypeProxy                          => { proxy => 'proxy' },
    internalLinkTypeSettings                       => { section => 'SettingsSection' },
    internalLinkTypeWebApp                         => { mode => 'WebAppOpenMode' },
    invoice                                        => { price_parts => 'array:labeledPricePart' },
    joinGroupCall                                  => { input_group_call => 'InputGroupCall', join_parameters => 'groupCallJoinParameters' },
    joinLiveStory                                  => { join_parameters => 'groupCallJoinParameters' },
    joinVideoChat                                  => { join_parameters => 'groupCallJoinParameters', participant_id => 'MessageSender' },
    jsonObjectMember                               => { value => 'JsonValue' },
    jsonValueArray                                 => { values => 'array:JsonValue' },
    jsonValueObject                                => { members => 'array:jsonObjectMember' },
    keyboardButton                                 => { style => 'ButtonStyle', type => 'KeyboardButtonType' },
    keyboardButtonTypeRequestChat                  => { bot_administrator_rights => 'chatAdministratorRights', user_administrator_rights => 'chatAdministratorRights' },
    languagePackString                             => { value => 'LanguagePackStringValue' },
    languagePackStrings                            => { strings => 'array:languagePackString' },
    launchPrepaidGiveaway                          => { parameters => 'giveawayParameters' },
    linkPreview                                    => { description => 'formattedText', type => 'LinkPreviewType' },
    linkPreviewAlbumMediaPhoto                     => { photo => 'photo' },
    linkPreviewAlbumMediaVideo                     => { video => 'video' },
    linkPreviewTypeAlbum                           => { media => 'array:LinkPreviewAlbumMedia' },
    linkPreviewTypeAnimation                       => { animation => 'animation' },
    linkPreviewTypeApp                             => { photo => 'photo' },
    linkPreviewTypeArticle                         => { photo => 'photo' },
    linkPreviewTypeAudio                           => { audio => 'audio' },
    linkPreviewTypeBackground                      => { background_type => 'BackgroundType', document => 'document' },
    linkPreviewTypeChannelBoost                    => { photo => 'chatPhoto' },
    linkPreviewTypeChat                            => { photo => 'chatPhoto', type => 'InviteLinkChatType' },
    linkPreviewTypeDirectMessagesChat              => { photo => 'chatPhoto' },
    linkPreviewTypeDocument                        => { document => 'document' },
    linkPreviewTypeEmbeddedAnimationPlayer         => { animation => 'animation', thumbnail => 'photo' },
    linkPreviewTypeEmbeddedAudioPlayer             => { audio => 'audio', thumbnail => 'photo' },
    linkPreviewTypeEmbeddedVideoPlayer             => { thumbnail => 'photo', video => 'video' },
    linkPreviewTypeGiftAuction                     => { gift => 'gift' },
    linkPreviewTypeGiftCollection                  => { icons => 'array:sticker' },
    linkPreviewTypePhoto                           => { photo => 'photo' },
    linkPreviewTypeSticker                         => { sticker => 'sticker' },
    linkPreviewTypeStickerSet                      => { stickers => 'array:sticker' },
    linkPreviewTypeStoryAlbum                      => { photo_icon => 'photo', video_icon => 'video' },
    linkPreviewTypeSupergroupBoost                 => { photo => 'chatPhoto' },
    linkPreviewTypeTheme                           => { documents => 'array:document', settings => 'themeSettings' },
    linkPreviewTypeUpgradedGift                    => { gift => 'upgradedGift' },
    linkPreviewTypeUser                            => { photo => 'chatPhoto' },
    linkPreviewTypeVideo                           => { cover => 'photo', video => 'video' },
    linkPreviewTypeVideoChat                       => { photo => 'chatPhoto' },
    linkPreviewTypeVideoNote                       => { video_note => 'videoNote' },
    linkPreviewTypeVoiceNote                       => { voice_note => 'voiceNote' },
    linkPreviewTypeWebApp                          => { photo => 'photo' },
    liveLocation                                   => { location => 'location' },
    liveStoryDonors                                => { top_donors => 'array:paidReactor' },
    loadActiveStories                              => { story_list => 'StoryList' },
    loadChats                                      => { chat_list => 'ChatList' },
    localizationTargetInfo                         => { language_packs => 'array:languagePackInfo' },
    mainWebApp                                     => { mode => 'WebAppOpenMode', url => 'webAppUrl' },
    maskPosition                                   => { point => 'MaskPoint' },
    message                                        => { content => 'MessageContent', fact_check => 'factCheck', forward_info => 'messageForwardInfo', guest_bot_caller_id => 'MessageSender', import_info => 'messageImportInfo', interaction_info => 'messageInteractionInfo', receiver_id => 'MessageSender', reply_markup => 'ReplyMarkup', reply_to => 'MessageReplyTo', restriction_info => 'restrictionInfo', scheduling_state => 'MessageSchedulingState', self_destruct_type => 'MessageSelfDestructType', sender_id => 'MessageSender', sending_state => 'MessageSendingState', suggested_post_info => 'suggestedPostInfo', topic_id => 'MessageTopic', unread_reactions => 'array:unreadReaction' },
    messageAnimatedEmoji                           => { animated_emoji => 'animatedEmoji' },
    messageAnimation                               => { animation => 'animation', caption => 'formattedText' },
    messageAudio                                   => { audio => 'audio', caption => 'formattedText' },
    messageBotWriteAccessAllowed                   => { reason => 'BotWriteAccessAllowReason' },
    messageCalendar                                => { days => 'array:messageCalendarDay' },
    messageCalendarDay                             => { message => 'message' },
    messageCall                                    => { discard_reason => 'CallDiscardReason' },
    messageChatChangePhoto                         => { photo => 'chatPhoto' },
    messageChatSetBackground                       => { background => 'chatBackground' },
    messageChatSetTheme                            => { theme => 'ChatTheme' },
    messageChatShared                              => { chat => 'sharedChat' },
    messageChecklist                               => { list => 'checklist' },
    messageChecklistTasksAdded                     => { tasks => 'array:checklistTask' },
    messageContact                                 => { contact => 'contact' },
    messageCopyOptions                             => { new_caption => 'formattedText' },
    messageDice                                    => { final_state => 'DiceStickers', initial_state => 'DiceStickers' },
    messageDocument                                => { caption => 'formattedText', document => 'document' },
    messageEffect                                  => { static_icon => 'sticker', type => 'MessageEffectType' },
    messageEffectTypeEmojiReaction                 => { effect_animation => 'sticker', select_animation => 'sticker' },
    messageEffectTypePremiumSticker                => { sticker => 'sticker' },
    messageForumTopicCreated                       => { icon => 'forumTopicIcon' },
    messageForwardInfo                             => { origin => 'MessageOrigin', source => 'forwardSource' },
    messageGame                                    => { game => 'game' },
    messageGift                                    => { gift => 'gift', receiver_id => 'MessageSender', sender_id => 'MessageSender', text => 'formattedText' },
    messageGiftedPremium                           => { sticker => 'sticker', text => 'formattedText' },
    messageGiftedStars                             => { sticker => 'sticker' },
    messageGiftedTon                               => { sticker => 'sticker' },
    messageGiveaway                                => { parameters => 'giveawayParameters', prize => 'GiveawayPrize', sticker => 'sticker' },
    messageGiveawayPrizeStars                      => { sticker => 'sticker' },
    messageGiveawayWinners                         => { prize => 'GiveawayPrize' },
    messageGroupCall                               => { other_participant_ids => 'array:MessageSender' },
    messageInteractionInfo                         => { reactions => 'messageReactions', reply_info => 'messageReplyInfo' },
    messageInvoice                                 => { paid_media => 'PaidMedia', paid_media_caption => 'formattedText', product_info => 'productInfo' },
    messageLinkInfo                                => { message => 'message', topic_id => 'MessageTopic' },
    messageLiveLocation                            => { location => 'liveLocation' },
    messageLocation                                => { location => 'location' },
    messagePaidMedia                               => { caption => 'formattedText', media => 'array:PaidMedia' },
    messagePassportDataReceived                    => { credentials => 'encryptedCredentials', elements => 'array:encryptedPassportElement' },
    messagePassportDataSent                        => { types => 'array:PassportElementType' },
    messagePaymentRefunded                         => { owner_id => 'MessageSender' },
    messagePaymentSuccessfulBot                    => { order_info => 'orderInfo' },
    messagePhoto                                   => { caption => 'formattedText', photo => 'photo', video => 'video' },
    messagePoll                                    => { description => 'formattedText', media => 'PollMedia', poll => 'poll' },
    messagePollOptionAdded                         => { text => 'formattedText' },
    messagePollOptionDeleted                       => { text => 'formattedText' },
    messagePositions                               => { positions => 'array:messagePosition' },
    messagePremiumGiftCode                         => { creator_id => 'MessageSender', sticker => 'sticker', text => 'formattedText' },
    messageProximityAlertTriggered                 => { traveler_id => 'MessageSender', watcher_id => 'MessageSender' },
    messageReaction                                => { recent_sender_ids => 'array:MessageSender', type => 'ReactionType', used_sender_id => 'MessageSender' },
    messageReactions                               => { paid_reactors => 'array:paidReactor', reactions => 'array:messageReaction' },
    messageRefundedUpgradedGift                    => { gift => 'gift', origin => 'UpgradedGiftOrigin', receiver_id => 'MessageSender', sender_id => 'MessageSender' },
    messageReplyInfo                               => { recent_replier_ids => 'array:MessageSender' },
    messageReplyToMessage                          => { content => 'MessageContent', origin => 'MessageOrigin', quote => 'textQuote' },
    messageRichMessage                             => { message => 'richMessage' },
    messageSendOptions                             => { scheduling_state => 'MessageSchedulingState', suggested_post_info => 'inputSuggestedPostInfo' },
    messageSenders                                 => { senders => 'array:MessageSender' },
    messageSendingStateFailed                      => { error => 'error' },
    messageStakeDice                               => { final_state => 'DiceStickers', initial_state => 'DiceStickers' },
    messageStatistics                              => { message_interaction_graph => 'StatisticalGraph', message_reaction_graph => 'StatisticalGraph' },
    messageSticker                                 => { sticker => 'sticker' },
    messageSuggestBirthdate                        => { birthdate => 'birthdate' },
    messageSuggestProfilePhoto                     => { photo => 'chatPhoto' },
    messageSuggestedPostApprovalFailed             => { price => 'SuggestedPostPrice' },
    messageSuggestedPostApproved                   => { price => 'SuggestedPostPrice' },
    messageSuggestedPostPaid                       => { star_amount => 'starAmount' },
    messageSuggestedPostRefunded                   => { reason => 'SuggestedPostRefundReason' },
    messageText                                    => { link_preview => 'linkPreview', link_preview_options => 'linkPreviewOptions', text => 'formattedText' },
    messageThreadInfo                              => { draft_message => 'draftMessage', messages => 'array:message', reply_info => 'messageReplyInfo' },
    messageUpgradedGift                            => { gift => 'upgradedGift', origin => 'UpgradedGiftOrigin', receiver_id => 'MessageSender', sender_id => 'MessageSender' },
    messageUpgradedGiftPurchaseOffer               => { gift => 'upgradedGift', price => 'GiftResalePrice', state => 'GiftPurchaseOfferState' },
    messageUpgradedGiftPurchaseOfferRejected       => { gift => 'upgradedGift', price => 'GiftResalePrice' },
    messageUsersShared                             => { users => 'array:sharedUser' },
    messageVenue                                   => { venue => 'venue' },
    messageVideo                                   => { alternative_videos => 'array:alternativeVideo', caption => 'formattedText', cover => 'photo', storyboards => 'array:videoStoryboard', video => 'video' },
    messageVideoNote                               => { video_note => 'videoNote' },
    messageViewers                                 => { viewers => 'array:messageViewer' },
    messageVoiceNote                               => { caption => 'formattedText', voice_note => 'voiceNote' },
    messages                                       => { messages => 'array:message' },
    networkStatistics                              => { entries => 'array:NetworkStatisticsEntry' },
    networkStatisticsEntryCall                     => { network_type => 'NetworkType' },
    networkStatisticsEntryFile                     => { file_type => 'FileType', network_type => 'NetworkType' },
    newSticker                                     => { format => 'StickerFormat', mask_position => 'maskPosition', sticker => 'InputFile' },
    notification                                   => { type => 'NotificationType' },
    notificationGroup                              => { notifications => 'array:notification', type => 'NotificationGroupType' },
    notificationSound                              => { sound => 'file' },
    notificationSounds                             => { notification_sounds => 'array:notificationSound' },
    notificationTypeNewMessage                     => { message => 'message' },
    notificationTypeNewPushMessage                 => { content => 'PushMessageContent', sender_id => 'MessageSender' },
    openWebApp                                     => { parameters => 'webAppOpenParameters', reply_to => 'InputMessageReplyTo', topic_id => 'MessageTopic' },
    optimizeStorage                                => { file_types => 'array:FileType' },
    orderInfo                                      => { shipping_address => 'address' },
    outline                                        => { paths => 'array:closedVectorPath' },
    pageBlockAnimation                             => { animation => 'animation', caption => 'pageBlockCaption' },
    pageBlockAudio                                 => { audio => 'audio', caption => 'pageBlockCaption' },
    pageBlockAuthorDate                            => { author => 'RichText' },
    pageBlockBlockQuote                            => { blocks => 'array:PageBlock', credit => 'RichText' },
    pageBlockCaption                               => { credit => 'RichText', text => 'RichText' },
    pageBlockChatLink                              => { photo => 'chatPhotoInfo' },
    pageBlockCollage                               => { blocks => 'array:PageBlock', caption => 'pageBlockCaption' },
    pageBlockCover                                 => { cover => 'PageBlock' },
    pageBlockDetails                               => { blocks => 'array:PageBlock', header => 'RichText' },
    pageBlockEmbedded                              => { caption => 'pageBlockCaption', poster_photo => 'photo' },
    pageBlockEmbeddedPost                          => { author_photo => 'photo', blocks => 'array:PageBlock', caption => 'pageBlockCaption' },
    pageBlockFooter                                => { footer => 'RichText' },
    pageBlockHeader                                => { header => 'RichText' },
    pageBlockKicker                                => { kicker => 'RichText' },
    pageBlockList                                  => { items => 'array:pageBlockListItem' },
    pageBlockListItem                              => { blocks => 'array:PageBlock' },
    pageBlockMap                                   => { caption => 'pageBlockCaption', location => 'location' },
    pageBlockParagraph                             => { text => 'RichText' },
    pageBlockPhoto                                 => { caption => 'pageBlockCaption', photo => 'photo' },
    pageBlockPreformatted                          => { text => 'RichText' },
    pageBlockPullQuote                             => { credit => 'RichText', text => 'RichText' },
    pageBlockRelatedArticle                        => { photo => 'photo' },
    pageBlockRelatedArticles                       => { articles => 'array:pageBlockRelatedArticle', header => 'RichText' },
    pageBlockSectionHeading                        => { text => 'RichText' },
    pageBlockSlideshow                             => { blocks => 'array:PageBlock', caption => 'pageBlockCaption' },
    pageBlockSubheader                             => { subheader => 'RichText' },
    pageBlockSubtitle                              => { subtitle => 'RichText' },
    pageBlockTable                                 => { caption => 'RichText' },
    pageBlockTableCell                             => { align => 'PageBlockHorizontalAlignment', text => 'RichText', valign => 'PageBlockVerticalAlignment' },
    pageBlockThinking                              => { text => 'RichText' },
    pageBlockTitle                                 => { title => 'RichText' },
    pageBlockVideo                                 => { caption => 'pageBlockCaption', video => 'video' },
    pageBlockVoiceNote                             => { caption => 'pageBlockCaption', voice_note => 'voiceNote' },
    paidMediaPhoto                                 => { photo => 'photo', video => 'video' },
    paidMediaPreview                               => { minithumbnail => 'minithumbnail' },
    paidMediaVideo                                 => { cover => 'photo', video => 'video' },
    paidReactor                                    => { sender_id => 'MessageSender' },
    parseMarkdown                                  => { text => 'formattedText' },
    parseTextEntities                              => { parse_mode => 'TextParseMode' },
    passkeys                                       => { passkeys => 'array:passkey' },
    passportAuthorizationForm                      => { required_elements => 'array:passportRequiredElement' },
    passportElementAddress                         => { address => 'address' },
    passportElementBankStatement                   => { bank_statement => 'personalDocument' },
    passportElementDriverLicense                   => { driver_license => 'identityDocument' },
    passportElementError                           => { source => 'PassportElementErrorSource', type => 'PassportElementType' },
    passportElementIdentityCard                    => { identity_card => 'identityDocument' },
    passportElementInternalPassport                => { internal_passport => 'identityDocument' },
    passportElementPassport                        => { passport => 'identityDocument' },
    passportElementPassportRegistration            => { passport_registration => 'personalDocument' },
    passportElementPersonalDetails                 => { personal_details => 'personalDetails' },
    passportElementRentalAgreement                 => { rental_agreement => 'personalDocument' },
    passportElementTemporaryRegistration           => { temporary_registration => 'personalDocument' },
    passportElementUtilityBill                     => { utility_bill => 'personalDocument' },
    passportElements                               => { elements => 'array:PassportElement' },
    passportElementsWithErrors                     => { elements => 'array:PassportElement', errors => 'array:passportElementError' },
    passportRequiredElement                        => { suitable_elements => 'array:passportSuitableElement' },
    passportSuitableElement                        => { type => 'PassportElementType' },
    passwordState                                  => { recovery_email_address_code_info => 'emailAddressAuthenticationCodeInfo' },
    paymentForm                                    => { product_info => 'productInfo', type => 'PaymentFormType' },
    paymentFormTypeRegular                         => { additional_payment_options => 'array:paymentOption', invoice => 'invoice', payment_provider => 'PaymentProvider', saved_credentials => 'array:savedCredentials', saved_order_info => 'orderInfo' },
    paymentFormTypeStarSubscription                => { pricing => 'starSubscriptionPricing' },
    paymentReceipt                                 => { product_info => 'productInfo', type => 'PaymentReceiptType' },
    paymentReceiptTypeRegular                      => { invoice => 'invoice', order_info => 'orderInfo', shipping_option => 'shippingOption' },
    personalDetails                                => { birthdate => 'date' },
    personalDocument                               => { files => 'array:datedFile', translation => 'array:datedFile' },
    phoneNumberAuthenticationSettings              => { firebase_authentication_settings => 'FirebaseAuthenticationSettings' },
    phoneNumberInfo                                => { country => 'countryInfo' },
    photo                                          => { minithumbnail => 'minithumbnail', sizes => 'array:photoSize' },
    photoSize                                      => { photo => 'file' },
    pingProxy                                      => { proxy => 'proxy' },
    placeGiftAuctionBid                            => { text => 'formattedText' },
    poll                                           => { options => 'array:pollOption', question => 'formattedText', recent_voter_ids => 'array:MessageSender', type => 'PollType', vote_restriction_reason => 'PollVoteRestrictionReason' },
    pollMediaAnimation                             => { animation => 'animation' },
    pollMediaAudio                                 => { audio => 'audio' },
    pollMediaDocument                              => { document => 'document' },
    pollMediaLink                                  => { link_preview => 'linkPreview' },
    pollMediaLocation                              => { location => 'location' },
    pollMediaPhoto                                 => { photo => 'photo', video => 'video' },
    pollMediaSticker                               => { sticker => 'sticker' },
    pollMediaVenue                                 => { venue => 'venue' },
    pollMediaVideo                                 => { alternative_videos => 'array:alternativeVideo', cover => 'photo', storyboards => 'array:videoStoryboard', video => 'video' },
    pollOption                                     => { author => 'MessageSender', media => 'PollMedia', recent_voter_ids => 'array:MessageSender', text => 'formattedText' },
    pollTypeQuiz                                   => { explanation => 'formattedText', explanation_media => 'PollMedia' },
    pollVoteStatistics                             => { vote_graph => 'StatisticalGraph' },
    pollVoter                                      => { voter_id => 'MessageSender' },
    pollVoters                                     => { voters => 'array:pollVoter' },
    postStory                                      => { areas => 'inputStoryAreas', caption => 'formattedText', content => 'InputStoryContent', from_story_full_id => 'storyFullId', privacy_settings => 'StoryPrivacySettings' },
    preliminaryUploadFile                          => { file => 'InputFile', file_type => 'FileType' },
    premiumFeaturePromotionAnimation               => { animation => 'animation', feature => 'PremiumFeature' },
    premiumFeatures                                => { features => 'array:PremiumFeature', limits => 'array:premiumLimit', payment_link => 'InternalLinkType' },
    premiumGiftCodeInfo                            => { creator_id => 'MessageSender' },
    premiumGiftPaymentOption                       => { sticker => 'sticker' },
    premiumGiftPaymentOptions                      => { options => 'array:premiumGiftPaymentOption' },
    premiumGiveawayPaymentOptions                  => { options => 'array:premiumGiveawayPaymentOption' },
    premiumLimit                                   => { type => 'PremiumLimitType' },
    premiumPaymentOption                           => { payment_link => 'InternalLinkType' },
    premiumSourceBusinessFeature                   => { feature => 'BusinessFeature' },
    premiumSourceFeature                           => { feature => 'PremiumFeature' },
    premiumSourceLimitExceeded                     => { limit_type => 'PremiumLimitType' },
    premiumSourceStoryFeature                      => { feature => 'PremiumStoryFeature' },
    premiumState                                   => { animations => 'array:premiumFeaturePromotionAnimation', business_animations => 'array:businessFeaturePromotionAnimation', payment_options => 'array:premiumStatePaymentOption', state => 'formattedText' },
    premiumStatePaymentOption                      => { payment_option => 'premiumPaymentOption' },
    prepaidGiveaway                                => { prize => 'GiveawayPrize' },
    preparedInlineMessage                          => { chat_types => 'targetChatTypes', result => 'InlineQueryResult' },
    productInfo                                    => { description => 'formattedText', photo => 'photo' },
    profileAccentColor                             => { dark_theme_colors => 'profileAccentColors', light_theme_colors => 'profileAccentColors' },
    profilePhoto                                   => { big => 'file', minithumbnail => 'minithumbnail', small => 'file' },
    proxy                                          => { type => 'ProxyType' },
    publicForwardMessage                           => { message => 'message' },
    publicForwardStory                             => { story => 'story' },
    publicForwards                                 => { forwards => 'array:PublicForward' },
    pushMessageContentAnimation                    => { animation => 'animation' },
    pushMessageContentAudio                        => { audio => 'audio' },
    pushMessageContentDocument                     => { document => 'document' },
    pushMessageContentGiveaway                     => { prize => 'GiveawayPrize' },
    pushMessageContentPhoto                        => { photo => 'photo' },
    pushMessageContentSticker                      => { sticker => 'sticker' },
    pushMessageContentVideo                        => { video => 'video' },
    pushMessageContentVideoNote                    => { video_note => 'videoNote' },
    pushMessageContentVoiceNote                    => { voice_note => 'voiceNote' },
    quickReplyMessage                              => { content => 'MessageContent', reply_markup => 'ReplyMarkup', sending_state => 'MessageSendingState' },
    quickReplyMessages                             => { messages => 'array:quickReplyMessage' },
    quickReplyShortcut                             => { first_message => 'quickReplyMessage' },
    reactionNotificationSettings                   => { message_reaction_source => 'ReactionNotificationSource', poll_vote_source => 'ReactionNotificationSource', story_reaction_source => 'ReactionNotificationSource' },
    readChatList                                   => { chat_list => 'ChatList' },
    receivedGift                                   => { gift => 'SentGift', sender_id => 'MessageSender', text => 'formattedText' },
    receivedGifts                                  => { gifts => 'array:receivedGift' },
    recommendedChatFolder                          => { folder => 'chatFolder' },
    recommendedChatFolders                         => { chat_folders => 'array:recommendedChatFolder' },
    registerDevice                                 => { device_token => 'DeviceToken' },
    removeFavoriteSticker                          => { sticker => 'InputFile' },
    removeGiftCollectionGifts                      => { owner_id => 'MessageSender' },
    removeMessageReaction                          => { reaction_type => 'ReactionType' },
    removeMessageSenderBotVerification             => { verified_id => 'MessageSender' },
    removeRecentSticker                            => { sticker => 'InputFile' },
    removeSavedAnimation                           => { animation => 'InputFile' },
    removeStickerFromSet                           => { sticker => 'InputFile' },
    removeTopChat                                  => { category => 'TopChatCategory' },
    reorderGiftCollectionGifts                     => { owner_id => 'MessageSender' },
    reorderGiftCollections                         => { owner_id => 'MessageSender' },
    reorderInstalledStickerSets                    => { sticker_type => 'StickerType' },
    replaceStickerInSet                            => { new_sticker => 'newSticker', old_sticker => 'InputFile' },
    reportChatPhoto                                => { reason => 'ReportReason' },
    reportChatResultOptionRequired                 => { options => 'array:reportOption' },
    reportMessageReactions                         => { sender_id => 'MessageSender' },
    reportSponsoredResultOptionRequired            => { options => 'array:reportOption' },
    reportStoryResultOptionRequired                => { options => 'array:reportOption' },
    resendAuthenticationCode                       => { reason => 'ResendCodeReason' },
    resendMessages                                 => { quote => 'inputTextQuote' },
    resendPhoneNumberCode                          => { reason => 'ResendCodeReason' },
    richMessage                                    => { blocks => 'array:PageBlock' },
    richMessageSourceBlocks                        => { blocks => 'array:InputPageBlock' },
    richMessageSourceHtml                          => { media => 'array:inputRichMessageMedia' },
    richMessageSourceMarkdown                      => { media => 'array:inputRichMessageMedia' },
    richTextAnchorLink                             => { text => 'RichText' },
    richTextBankCardNumber                         => { text => 'RichText' },
    richTextBold                                   => { text => 'RichText' },
    richTextBotCommand                             => { text => 'RichText' },
    richTextCashtag                                => { text => 'RichText' },
    richTextDateTime                               => { formatting_type => 'DateTimeFormattingType', text => 'RichText' },
    richTextDiff                                   => { old_text => 'RichText', text => 'RichText' },
    richTextEmailAddress                           => { text => 'RichText' },
    richTextFixed                                  => { text => 'RichText' },
    richTextHashtag                                => { text => 'RichText' },
    richTextIcon                                   => { document => 'document' },
    richTextItalic                                 => { text => 'RichText' },
    richTextMarked                                 => { text => 'RichText' },
    richTextMention                                => { text => 'RichText' },
    richTextMentionName                            => { text => 'RichText' },
    richTextPhoneNumber                            => { text => 'RichText' },
    richTextReference                              => { text => 'RichText' },
    richTextReferenceLink                          => { text => 'RichText' },
    richTextSpoiler                                => { text => 'RichText' },
    richTextStrikethrough                          => { text => 'RichText' },
    richTextSubscript                              => { text => 'RichText' },
    richTextSuperscript                            => { text => 'RichText' },
    richTextUnderline                              => { text => 'RichText' },
    richTextUrl                                    => { text => 'RichText' },
    richTexts                                      => { texts => 'array:RichText' },
    saveApplicationLogEvent                        => { data => 'JsonValue' },
    savePreparedInlineMessage                      => { chat_types => 'targetChatTypes', result => 'InputInlineQueryResult' },
    savePreparedKeyboardButton                     => { button => 'keyboardButton' },
    savedMessagesTag                               => { tag => 'ReactionType' },
    savedMessagesTags                              => { tags => 'array:savedMessagesTag' },
    savedMessagesTopic                             => { draft_message => 'draftMessage', last_message => 'message', type => 'SavedMessagesTopicType' },
    searchAffiliatePrograms                        => { affiliate => 'AffiliateType', sort_order => 'AffiliateProgramSortOrder' },
    searchChatMembers                              => { filter => 'ChatMembersFilter' },
    searchChatMessages                             => { filter => 'SearchMessagesFilter', sender_id => 'MessageSender', topic_id => 'MessageTopic' },
    searchChats                                    => { type_filter => 'SearchChatTypeFilter' },
    searchChatsOnServer                            => { type_filter => 'SearchChatTypeFilter' },
    searchGiftsForResale                           => { attributes => 'array:UpgradedGiftAttributeId', order => 'GiftForResaleOrder' },
    searchInstalledStickerSets                     => { sticker_type => 'StickerType' },
    searchMessages                                 => { chat_list => 'ChatList', chat_type_filter => 'SearchMessagesChatTypeFilter', filter => 'SearchMessagesFilter' },
    searchPublicChats                              => { type_filter => 'SearchChatTypeFilter' },
    searchPublicStoriesByLocation                  => { address => 'locationAddress' },
    searchQuote                                    => { quote => 'formattedText', text => 'formattedText' },
    searchRecentlyFoundChats                       => { type_filter => 'SearchChatTypeFilter' },
    searchSavedMessages                            => { tag => 'ReactionType' },
    searchSecretMessages                           => { filter => 'SearchMessagesFilter' },
    searchStickerSets                              => { sticker_type => 'StickerType' },
    searchStickers                                 => { sticker_type => 'StickerType' },
    secretChat                                     => { state => 'SecretChatState' },
    sendBusinessMessage                            => { input_message_content => 'InputMessageContent', reply_markup => 'ReplyMarkup', reply_to => 'InputMessageReplyTo' },
    sendBusinessMessageAlbum                       => { input_message_contents => 'array:InputMessageContent', reply_to => 'InputMessageReplyTo' },
    sendCallDebugInformation                       => { call_id => 'InputCall' },
    sendCallLog                                    => { call_id => 'InputCall', log_file => 'InputFile' },
    sendCallRating                                 => { call_id => 'InputCall', problems => 'array:CallProblem' },
    sendChatAction                                 => { action => 'ChatAction', topic_id => 'MessageTopic' },
    sendEphemeralMessage                           => { input_message_content => 'InputMessageContent', reply_markup => 'ReplyMarkup', reply_to => 'InputMessageReplyTo', topic_id => 'MessageTopic' },
    sendGift                                       => { owner_id => 'MessageSender', text => 'formattedText' },
    sendGiftPurchaseOffer                          => { owner_id => 'MessageSender', price => 'GiftResalePrice' },
    sendGroupCallMessage                           => { text => 'formattedText' },
    sendInlineQueryResultMessage                   => { options => 'messageSendOptions', reply_to => 'InputMessageReplyTo', topic_id => 'MessageTopic' },
    sendMessage                                    => { input_message_content => 'InputMessageContent', options => 'messageSendOptions', reply_markup => 'ReplyMarkup', reply_to => 'InputMessageReplyTo', topic_id => 'MessageTopic' },
    sendMessageAlbum                               => { input_message_contents => 'array:InputMessageContent', options => 'messageSendOptions', reply_to => 'InputMessageReplyTo', topic_id => 'MessageTopic' },
    sendPassportAuthorizationForm                  => { types => 'array:PassportElementType' },
    sendPaymentForm                                => { credentials => 'InputCredentials', input_invoice => 'InputInvoice' },
    sendPhoneNumberCode                            => { settings => 'phoneNumberAuthenticationSettings', type => 'PhoneNumberCodeType' },
    sendResoldGift                                 => { owner_id => 'MessageSender', price => 'GiftResalePrice' },
    sendRichMessageDraft                           => { message => 'inputRichMessage' },
    sendTextMessageDraft                           => { text => 'formattedText' },
    sentGiftRegular                                => { gift => 'gift' },
    sentGiftUpgraded                               => { gift => 'upgradedGift' },
    session                                        => { device_type => 'SessionDeviceType' },
    sessions                                       => { sessions => 'array:session' },
    setAccountTtl                                  => { ttl => 'accountTtl' },
    setArchiveChatListSettings                     => { settings => 'archiveChatListSettings' },
    setAuthenticationPhoneNumber                   => { settings => 'phoneNumberAuthenticationSettings' },
    setAuthenticationPremiumPurchaseTransaction    => { transaction => 'StoreTransaction' },
    setAutoDownloadSettings                        => { settings => 'autoDownloadSettings', type => 'NetworkType' },
    setAutosaveSettings                            => { scope => 'AutosaveSettingsScope', settings => 'scopeAutosaveSettings' },
    setBirthdate                                   => { birthdate => 'birthdate' },
    setBotProfilePhoto                             => { photo => 'InputChatPhoto' },
    setBusinessAccountGiftSettings                 => { settings => 'giftSettings' },
    setBusinessAccountProfilePhoto                 => { photo => 'InputChatPhoto' },
    setBusinessAwayMessageSettings                 => { away_message_settings => 'businessAwayMessageSettings' },
    setBusinessConnectedBot                        => { bot => 'businessConnectedBot' },
    setBusinessGreetingMessageSettings             => { greeting_message_settings => 'businessGreetingMessageSettings' },
    setBusinessLocation                            => { location => 'businessLocation' },
    setBusinessOpeningHours                        => { opening_hours => 'businessOpeningHours' },
    setBusinessStartPage                           => { start_page => 'inputBusinessStartPage' },
    setChatActiveStoriesList                       => { story_list => 'StoryList' },
    setChatAffiliateProgram                        => { parameters => 'affiliateProgramParameters' },
    setChatAvailableReactions                      => { available_reactions => 'ChatAvailableReactions' },
    setChatBackground                              => { background => 'InputBackground', type => 'BackgroundType' },
    setChatDraftMessage                            => { draft_message => 'draftMessage', topic_id => 'MessageTopic' },
    setChatEmojiStatus                             => { emoji_status => 'emojiStatus' },
    setChatLocation                                => { location => 'chatLocation' },
    setChatMemberStatus                            => { member_id => 'MessageSender', status => 'ChatMemberStatus' },
    setChatMessageSender                           => { message_sender_id => 'MessageSender' },
    setChatNotificationSettings                    => { notification_settings => 'chatNotificationSettings' },
    setChatPermissions                             => { permissions => 'chatPermissions' },
    setChatPhoto                                   => { photo => 'InputChatPhoto' },
    setChatTheme                                   => { theme => 'InputChatTheme' },
    setCommands                                    => { commands => 'array:botCommand', scope => 'BotCommandScope' },
    setCustomLanguagePack                          => { info => 'languagePackInfo', strings => 'array:languagePackString' },
    setCustomLanguagePackString                    => { new_string => 'languagePackString' },
    setDefaultBackground                           => { background => 'InputBackground', type => 'BackgroundType' },
    setDefaultChannelAdministratorRights           => { default_channel_administrator_rights => 'chatAdministratorRights' },
    setDefaultGroupAdministratorRights             => { default_group_administrator_rights => 'chatAdministratorRights' },
    setDefaultMessageAutoDeleteTime                => { message_auto_delete_time => 'messageAutoDeleteTime' },
    setDefaultReactionType                         => { reaction_type => 'ReactionType' },
    setEmojiStatus                                 => { emoji_status => 'emojiStatus' },
    setForumTopicNotificationSettings              => { notification_settings => 'chatNotificationSettings' },
    setGiftCollectionName                          => { owner_id => 'MessageSender' },
    setGiftResalePrice                             => { price => 'GiftResalePrice' },
    setGiftSettings                                => { settings => 'giftSettings' },
    setGroupCallParticipantVolumeLevel             => { participant_id => 'MessageSender' },
    setLiveStoryMessageSender                      => { message_sender_id => 'MessageSender' },
    setLogStream                                   => { log_stream => 'LogStream' },
    setMainProfileTab                              => { main_profile_tab => 'ProfileTab' },
    setManagedBotAccessSettings                    => { settings => 'botAccessSettings' },
    setMenuButton                                  => { menu_button => 'botMenuButton' },
    setMessageFactCheck                            => { text => 'formattedText' },
    setMessageReactions                            => { reaction_types => 'array:ReactionType' },
    setMessageSenderBlockList                      => { block_list => 'BlockList', sender_id => 'MessageSender' },
    setMessageSenderBotVerification                => { verified_id => 'MessageSender' },
    setNetworkType                                 => { type => 'NetworkType' },
    setNewChatPrivacySettings                      => { settings => 'newChatPrivacySettings' },
    setOption                                      => { value => 'OptionValue' },
    setPaidMessageReactionType                     => { type => 'PaidReactionType' },
    setPassportElement                             => { element => 'InputPassportElement' },
    setPassportElementErrors                       => { errors => 'array:inputPassportElementError' },
    setPinnedChats                                 => { chat_list => 'ChatList' },
    setPinnedGifts                                 => { owner_id => 'MessageSender' },
    setProfilePhoto                                => { photo => 'InputChatPhoto' },
    setReactionNotificationSettings                => { notification_settings => 'reactionNotificationSettings' },
    setReadDatePrivacySettings                     => { settings => 'readDatePrivacySettings' },
    setSavedMessagesTagLabel                       => { tag => 'ReactionType' },
    setScopeNotificationSettings                   => { notification_settings => 'scopeNotificationSettings', scope => 'NotificationSettingsScope' },
    setStickerEmojis                               => { sticker => 'InputFile' },
    setStickerKeywords                             => { sticker => 'InputFile' },
    setStickerMaskPosition                         => { mask_position => 'maskPosition', sticker => 'InputFile' },
    setStickerPositionInSet                        => { sticker => 'InputFile' },
    setStickerSetThumbnail                         => { format => 'StickerFormat', thumbnail => 'InputFile' },
    setStoryPrivacySettings                        => { privacy_settings => 'StoryPrivacySettings' },
    setStoryReaction                               => { reaction_type => 'ReactionType' },
    setSupergroupMainProfileTab                    => { main_profile_tab => 'ProfileTab' },
    setUserEmojiStatus                             => { emoji_status => 'emojiStatus' },
    setUserNote                                    => { note => 'formattedText' },
    setUserPersonalProfilePhoto                    => { photo => 'InputChatPhoto' },
    setUserPrivacySettingRules                     => { rules => 'userPrivacySettingRules', setting => 'UserPrivacySetting' },
    setUserSupportInfo                             => { message => 'formattedText' },
    setVideoChatDefaultParticipant                 => { default_participant_id => 'MessageSender' },
    shareChatWithBot                               => { source => 'KeyboardButtonSource' },
    shareUsersWithBot                              => { source => 'KeyboardButtonSource' },
    sharedChat                                     => { photo => 'photo' },
    sharedUser                                     => { photo => 'photo' },
    shippingOption                                 => { price_parts => 'array:labeledPricePart' },
    speechRecognitionResultError                   => { error => 'error' },
    sponsoredChats                                 => { chats => 'array:sponsoredChat' },
    sponsoredMessage                               => { content => 'MessageContent', sponsor => 'advertisementSponsor' },
    sponsoredMessages                              => { messages => 'array:sponsoredMessage' },
    starGiveawayPaymentOption                      => { winner_options => 'array:starGiveawayWinnerOption' },
    starGiveawayPaymentOptions                     => { options => 'array:starGiveawayPaymentOption' },
    starPaymentOptions                             => { options => 'array:starPaymentOption' },
    starRevenueStatistics                          => { revenue_by_day_graph => 'StatisticalGraph', status => 'starRevenueStatus' },
    starRevenueStatus                              => { available_amount => 'starAmount', current_amount => 'starAmount', total_amount => 'starAmount' },
    starSubscription                               => { pricing => 'starSubscriptionPricing', type => 'StarSubscriptionType' },
    starSubscriptionTypeBot                        => { photo => 'photo' },
    starSubscriptions                              => { star_amount => 'starAmount', subscriptions => 'array:starSubscription' },
    starTransaction                                => { star_amount => 'starAmount', type => 'StarTransactionType' },
    starTransactionTypeBotInvoicePurchase          => { product_info => 'productInfo' },
    starTransactionTypeBotInvoiceSale              => { affiliate => 'affiliateInfo', product_info => 'productInfo' },
    starTransactionTypeBotPaidMediaPurchase        => { media => 'array:PaidMedia' },
    starTransactionTypeBotPaidMediaSale            => { affiliate => 'affiliateInfo', media => 'array:PaidMedia' },
    starTransactionTypeBotSubscriptionPurchase     => { product_info => 'productInfo' },
    starTransactionTypeBotSubscriptionSale         => { affiliate => 'affiliateInfo', product_info => 'productInfo' },
    starTransactionTypeChannelPaidMediaPurchase    => { media => 'array:PaidMedia' },
    starTransactionTypeChannelPaidMediaSale        => { media => 'array:PaidMedia' },
    starTransactionTypeFragmentWithdrawal          => { withdrawal_state => 'RevenueWithdrawalState' },
    starTransactionTypeGiftAuctionBid              => { gift => 'gift', owner_id => 'MessageSender' },
    starTransactionTypeGiftOriginalDetailsDrop     => { gift => 'upgradedGift', owner_id => 'MessageSender' },
    starTransactionTypeGiftPurchase                => { gift => 'gift', owner_id => 'MessageSender' },
    starTransactionTypeGiftPurchaseOffer           => { gift => 'upgradedGift' },
    starTransactionTypeGiftSale                    => { gift => 'gift' },
    starTransactionTypeGiftTransfer                => { gift => 'upgradedGift', owner_id => 'MessageSender' },
    starTransactionTypeGiftUpgrade                 => { gift => 'upgradedGift' },
    starTransactionTypeGiftUpgradePurchase         => { gift => 'gift', owner_id => 'MessageSender' },
    starTransactionTypePaidGroupCallMessageReceive => { commission_star_amount => 'starAmount', sender_id => 'MessageSender' },
    starTransactionTypePaidGroupCallReactionReceive => { commission_star_amount => 'starAmount', sender_id => 'MessageSender' },
    starTransactionTypePaidMessageReceive          => { commission_star_amount => 'starAmount', sender_id => 'MessageSender' },
    starTransactionTypePremiumPurchase             => { sticker => 'sticker' },
    starTransactionTypeUpgradedGiftPurchase        => { gift => 'upgradedGift' },
    starTransactionTypeUpgradedGiftSale            => { commission_star_amount => 'starAmount', gift => 'upgradedGift' },
    starTransactionTypeUserDeposit                 => { sticker => 'sticker' },
    starTransactions                               => { star_amount => 'starAmount', transactions => 'array:starTransaction' },
    startLiveStory                                 => { privacy_settings => 'StoryPrivacySettings' },
    startLiveStoryResultFail                       => { error_type => 'CanPostStoryResult' },
    startLiveStoryResultOk                         => { story => 'story' },
    sticker                                        => { format => 'StickerFormat', full_type => 'StickerFullType', sticker => 'file', thumbnail => 'thumbnail' },
    stickerFullTypeMask                            => { mask_position => 'maskPosition' },
    stickerFullTypeRegular                         => { premium_animation => 'file' },
    stickerSet                                     => { emojis => 'array:emojis', sticker_type => 'StickerType', stickers => 'array:sticker', thumbnail => 'thumbnail', thumbnail_outline => 'outline' },
    stickerSetInfo                                 => { covers => 'array:sticker', sticker_type => 'StickerType', thumbnail => 'thumbnail', thumbnail_outline => 'outline' },
    stickerSets                                    => { sets => 'array:stickerSetInfo' },
    stickers                                       => { stickers => 'array:sticker' },
    stopBusinessPoll                               => { reply_markup => 'ReplyMarkup' },
    stopPoll                                       => { reply_markup => 'ReplyMarkup' },
    storageStatistics                              => { by_chat => 'array:storageStatisticsByChat' },
    storageStatisticsByChat                        => { by_file_type => 'array:storageStatisticsByFileType' },
    storageStatisticsByFileType                    => { file_type => 'FileType' },
    storePaymentPurposePremiumGift                 => { text => 'formattedText' },
    storePaymentPurposePremiumGiftCodes            => { text => 'formattedText' },
    storePaymentPurposePremiumGiveaway             => { parameters => 'giveawayParameters' },
    storePaymentPurposeStarGiveaway                => { parameters => 'giveawayParameters' },
    stories                                        => { stories => 'array:story' },
    story                                          => { areas => 'array:storyArea', caption => 'formattedText', chosen_reaction_type => 'ReactionType', content => 'StoryContent', interaction_info => 'storyInteractionInfo', poster_id => 'MessageSender', privacy_settings => 'StoryPrivacySettings', repost_info => 'storyRepostInfo' },
    storyAlbum                                     => { photo_icon => 'photo', video_icon => 'video' },
    storyAlbums                                    => { albums => 'array:storyAlbum' },
    storyArea                                      => { position => 'storyAreaPosition', type => 'StoryAreaType' },
    storyAreaTypeLocation                          => { address => 'locationAddress', location => 'location' },
    storyAreaTypeSuggestedReaction                 => { reaction_type => 'ReactionType' },
    storyAreaTypeVenue                             => { venue => 'venue' },
    storyContentPhoto                              => { photo => 'photo' },
    storyContentVideo                              => { alternative_video => 'storyVideo', video => 'storyVideo' },
    storyInteraction                               => { actor_id => 'MessageSender', block_list => 'BlockList', type => 'StoryInteractionType' },
    storyInteractionTypeForward                    => { message => 'message' },
    storyInteractionTypeRepost                     => { story => 'story' },
    storyInteractionTypeView                       => { chosen_reaction_type => 'ReactionType' },
    storyInteractions                              => { interactions => 'array:storyInteraction' },
    storyRepostInfo                                => { origin => 'StoryOrigin' },
    storyStatistics                                => { story_interaction_graph => 'StatisticalGraph', story_reaction_graph => 'StatisticalGraph' },
    storyVideo                                     => { minithumbnail => 'minithumbnail', thumbnail => 'thumbnail', video => 'file' },
    suggestUserBirthdate                           => { birthdate => 'birthdate' },
    suggestUserProfilePhoto                        => { photo => 'InputChatPhoto' },
    suggestedActionCustom                          => { description => 'formattedText', title => 'formattedText' },
    suggestedPostInfo                              => { price => 'SuggestedPostPrice', state => 'SuggestedPostState' },
    supergroup                                     => { active_story_state => 'ActiveStoryState', restriction_info => 'restrictionInfo', status => 'ChatMemberStatus', usernames => 'usernames', verification_status => 'verificationStatus' },
    supergroupFullInfo                             => { bot_commands => 'array:botCommands', bot_verification => 'botVerification', invite_link => 'chatInviteLink', location => 'chatLocation', main_profile_tab => 'ProfileTab', photo => 'chatPhoto' },
    supergroupMembersFilterMention                 => { topic_id => 'MessageTopic' },
    tMeUrl                                         => { type => 'TMeUrlType' },
    tMeUrlTypeChatInvite                           => { info => 'chatInviteLinkInfo' },
    tMeUrls                                        => { urls => 'array:tMeUrl' },
    targetChatChosen                               => { types => 'targetChatTypes' },
    targetChatInternalLink                         => { link => 'InternalLinkType' },
    telegramPaymentPurposePremiumGift              => { text => 'formattedText' },
    telegramPaymentPurposePremiumGiftCodes         => { text => 'formattedText' },
    telegramPaymentPurposePremiumGiveaway          => { parameters => 'giveawayParameters' },
    telegramPaymentPurposeStarGiveaway             => { parameters => 'giveawayParameters' },
    termsOfService                                 => { text => 'formattedText' },
    testCallVectorIntObject                        => { x => 'array:testInt' },
    testCallVectorStringObject                     => { x => 'array:testString' },
    testProxy                                      => { proxy => 'proxy' },
    testReturnError                                => { error => 'error' },
    testVectorIntObject                            => { value => 'array:testInt' },
    testVectorStringObject                         => { value => 'array:testString' },
    textCompositionStyle                           => { english_example => 'textCompositionStyleExample' },
    textCompositionStyleExample                    => { result_text => 'formattedText', source_text => 'formattedText' },
    textEntities                                   => { entities => 'array:textEntity' },
    textEntity                                     => { type => 'TextEntityType' },
    textEntityTypeDateTime                         => { formatting_type => 'DateTimeFormattingType' },
    textQuote                                      => { text => 'formattedText' },
    themeSettings                                  => { background => 'background', base_theme => 'BuiltInTheme', outgoing_message_fill => 'BackgroundFill' },
    thumbnail                                      => { file => 'file', format => 'ThumbnailFormat' },
    timeZones                                      => { time_zones => 'array:timeZone' },
    toggleChatIsPinned                             => { chat_list => 'ChatList' },
    toggleGroupCallParticipantIsHandRaised         => { participant_id => 'MessageSender' },
    toggleGroupCallParticipantIsMuted              => { participant_id => 'MessageSender' },
    tonTransaction                                 => { type => 'TonTransactionType' },
    tonTransactionTypeFragmentDeposit              => { sticker => 'sticker' },
    tonTransactionTypeFragmentWithdrawal           => { withdrawal_state => 'RevenueWithdrawalState' },
    tonTransactionTypeGiftPurchaseOffer            => { gift => 'upgradedGift' },
    tonTransactionTypeUpgradedGiftPurchase         => { gift => 'upgradedGift' },
    tonTransactionTypeUpgradedGiftSale             => { gift => 'upgradedGift' },
    tonTransactions                                => { transactions => 'array:tonTransaction' },
    transferGift                                   => { new_owner_id => 'MessageSender' },
    translateRichMessage                           => { message => 'inputRichMessage' },
    translateText                                  => { text => 'formattedText' },
    trendingStickerSets                            => { sets => 'array:stickerSetInfo' },
    unconfirmedSession                             => { type => 'SessionType' },
    unreadReaction                                 => { sender_id => 'MessageSender', type => 'ReactionType' },
    updateAccentColors                             => { colors => 'array:accentColor' },
    updateActiveGiftAuctions                       => { states => 'array:giftAuctionState' },
    updateActiveLiveLocationMessages               => { messages => 'array:message' },
    updateActiveNotifications                      => { groups => 'array:notificationGroup' },
    updateAgeVerificationParameters                => { parameters => 'ageVerificationParameters' },
    updateAnimatedEmojiMessageClicked              => { sticker => 'sticker' },
    updateAttachmentMenuBots                       => { bots => 'array:attachmentMenuBot' },
    updateAuthorizationState                       => { authorization_state => 'AuthorizationState' },
    updateAutosaveSettings                         => { scope => 'AutosaveSettingsScope', settings => 'scopeAutosaveSettings' },
    updateBasicGroup                               => { basic_group => 'basicGroup' },
    updateBasicGroupFullInfo                       => { basic_group_full_info => 'basicGroupFullInfo' },
    updateBusinessConnection                       => { connection => 'businessConnection' },
    updateBusinessMessageEdited                    => { message => 'businessMessage' },
    updateCall                                     => { call => 'call' },
    updateChatAccentColors                         => { upgraded_gift_colors => 'upgradedGiftColors' },
    updateChatAction                               => { action => 'ChatAction', sender_id => 'MessageSender', topic_id => 'MessageTopic' },
    updateChatActionBar                            => { action_bar => 'ChatActionBar' },
    updateChatActiveStories                        => { active_stories => 'chatActiveStories' },
    updateChatAddedToList                          => { chat_list => 'ChatList' },
    updateChatAvailableReactions                   => { available_reactions => 'ChatAvailableReactions' },
    updateChatBackground                           => { background => 'chatBackground' },
    updateChatBlockList                            => { block_list => 'BlockList' },
    updateChatBoost                                => { boost => 'chatBoost' },
    updateChatBusinessBotManageBar                 => { business_bot_manage_bar => 'businessBotManageBar' },
    updateChatDraftMessage                         => { draft_message => 'draftMessage', positions => 'array:chatPosition' },
    updateChatEmojiStatus                          => { emoji_status => 'emojiStatus' },
    updateChatFolders                              => { chat_folders => 'array:chatFolderInfo' },
    updateChatJoinResult                           => { result => 'ChatJoinRequestResult' },
    updateChatLastMessage                          => { last_message => 'message', positions => 'array:chatPosition' },
    updateChatMember                               => { invite_link => 'chatInviteLink', new_chat_member => 'chatMember', old_chat_member => 'chatMember' },
    updateChatMessageSender                        => { message_sender_id => 'MessageSender' },
    updateChatNotificationSettings                 => { notification_settings => 'chatNotificationSettings' },
    updateChatPendingJoinRequests                  => { pending_join_requests => 'chatJoinRequestsInfo' },
    updateChatPermissions                          => { permissions => 'chatPermissions' },
    updateChatPhoto                                => { photo => 'chatPhotoInfo' },
    updateChatPosition                             => { position => 'chatPosition' },
    updateChatRemovedFromList                      => { chat_list => 'ChatList' },
    updateChatReplyMarkup                          => { reply_markup_message => 'message' },
    updateChatRevenueAmount                        => { revenue_amount => 'chatRevenueAmount' },
    updateChatTheme                                => { theme => 'ChatTheme' },
    updateChatVideoChat                            => { video_chat => 'videoChat' },
    updateCommunity                                => { community => 'community' },
    updateConnectionState                          => { state => 'ConnectionState' },
    updateContactCloseBirthdays                    => { close_birthday_users => 'array:closeBirthdayUser' },
    updateDefaultBackground                        => { background => 'background' },
    updateDefaultPaidReactionType                  => { type => 'PaidReactionType' },
    updateDefaultReactionType                      => { reaction_type => 'ReactionType' },
    updateDirectMessagesChatTopic                  => { topic => 'directMessagesChatTopic' },
    updateEmojiChatThemes                          => { chat_themes => 'array:emojiChatTheme' },
    updateFile                                     => { file => 'file' },
    updateFileAddedToDownloads                     => { counts => 'downloadedFileCounts', file_download => 'fileDownload' },
    updateFileDownload                             => { counts => 'downloadedFileCounts' },
    updateFileRemovedFromDownloads                 => { counts => 'downloadedFileCounts' },
    updateForumTopic                               => { draft_message => 'draftMessage', notification_settings => 'chatNotificationSettings' },
    updateForumTopicInfo                           => { info => 'forumTopicInfo' },
    updateGiftAuctionState                         => { state => 'giftAuctionState' },
    updateGramRevenueStatus                        => { status => 'gramRevenueStatus' },
    updateGroupCall                                => { group_call => 'groupCall' },
    updateGroupCallMessageLevels                   => { levels => 'array:groupCallMessageLevel' },
    updateGroupCallMessageSendFailed               => { error => 'error' },
    updateGroupCallParticipant                     => { participant => 'groupCallParticipant' },
    updateInstalledStickerSets                     => { sticker_type => 'StickerType' },
    updateLanguagePackStrings                      => { strings => 'array:languagePackString' },
    updateLiveStoryTopDonors                       => { donors => 'liveStoryDonors' },
    updateMessageContent                           => { new_content => 'MessageContent' },
    updateMessageEdited                            => { reply_markup => 'ReplyMarkup' },
    updateMessageFactCheck                         => { fact_check => 'factCheck' },
    updateMessageInteractionInfo                   => { interaction_info => 'messageInteractionInfo' },
    updateMessageReaction                          => { actor_id => 'MessageSender', new_reaction_types => 'array:ReactionType', old_reaction_types => 'array:ReactionType' },
    updateMessageReactions                         => { reactions => 'array:messageReaction' },
    updateMessageSendFailed                        => { error => 'error', message => 'message' },
    updateMessageSendSucceeded                     => { message => 'message' },
    updateMessageSuggestedPostInfo                 => { suggested_post_info => 'suggestedPostInfo' },
    updateMessageUnreadReactions                   => { unread_reactions => 'array:unreadReaction' },
    updateNewBusinessCallbackQuery                 => { message => 'businessMessage', payload => 'CallbackQueryPayload' },
    updateNewBusinessMessage                       => { message => 'businessMessage' },
    updateNewCallbackQuery                         => { payload => 'CallbackQueryPayload' },
    updateNewChat                                  => { chat => 'chat' },
    updateNewChatJoinRequest                       => { invite_link => 'chatInviteLink', request => 'chatJoinRequest' },
    updateNewChosenInlineResult                    => { user_location => 'location' },
    updateNewGroupCallMessage                      => { message => 'groupCallMessage' },
    updateNewGroupCallPaidReaction                 => { sender_id => 'MessageSender' },
    updateNewGuestQuery                            => { message => 'message', reference_messages => 'array:message' },
    updateNewInlineCallbackQuery                   => { payload => 'CallbackQueryPayload' },
    updateNewInlineQuery                           => { chat_type => 'ChatType', user_location => 'location' },
    updateNewMessage                               => { message => 'message' },
    updateNewPreCheckoutQuery                      => { order_info => 'orderInfo' },
    updateNewShippingQuery                         => { shipping_address => 'address' },
    updateNotification                             => { notification => 'notification' },
    updateNotificationGroup                        => { added_notifications => 'array:notification', type => 'NotificationGroupType' },
    updateOption                                   => { value => 'OptionValue' },
    updateOwnedStarCount                           => { star_amount => 'starAmount' },
    updatePendingMessage                           => { content => 'MessageContent' },
    updatePoll                                     => { poll => 'poll' },
    updatePollAnswer                               => { voter_id => 'MessageSender' },
    updateProfileAccentColors                      => { colors => 'array:profileAccentColor' },
    updateQuickReplyShortcut                       => { shortcut => 'quickReplyShortcut' },
    updateQuickReplyShortcutMessages               => { messages => 'array:quickReplyMessage' },
    updateReactionNotificationSettings             => { notification_settings => 'reactionNotificationSettings' },
    updateSavedMessagesTags                        => { tags => 'savedMessagesTags' },
    updateSavedMessagesTopic                       => { topic => 'savedMessagesTopic' },
    updateScopeNotificationSettings                => { notification_settings => 'scopeNotificationSettings', scope => 'NotificationSettingsScope' },
    updateSecretChat                               => { secret_chat => 'secretChat' },
    updateServiceNotification                      => { content => 'MessageContent' },
    updateStakeDiceState                           => { state => 'stakeDiceState' },
    updateStarRevenueStatus                        => { owner_id => 'MessageSender', status => 'starRevenueStatus' },
    updateStickerSet                               => { sticker_set => 'stickerSet' },
    updateStory                                    => { story => 'story' },
    updateStoryListChatCount                       => { story_list => 'StoryList' },
    updateStoryPostFailed                          => { error => 'error', error_type => 'CanPostStoryResult', story => 'story' },
    updateStoryPostSucceeded                       => { story => 'story' },
    updateSuggestedActions                         => { added_actions => 'array:SuggestedAction', removed_actions => 'array:SuggestedAction' },
    updateSupergroup                               => { supergroup => 'supergroup' },
    updateSupergroupFullInfo                       => { supergroup_full_info => 'supergroupFullInfo' },
    updateTermsOfService                           => { terms_of_service => 'termsOfService' },
    updateTextCompositionStyles                    => { styles => 'array:textCompositionStyle' },
    updateTopicMessageCount                        => { topic_id => 'MessageTopic' },
    updateTrendingStickerSets                      => { sticker_sets => 'trendingStickerSets', sticker_type => 'StickerType' },
    updateUnconfirmedSession                       => { session => 'unconfirmedSession' },
    updateUnreadChatCount                          => { chat_list => 'ChatList' },
    updateUnreadMessageCount                       => { chat_list => 'ChatList' },
    updateUser                                     => { user => 'user' },
    updateUserFullInfo                             => { user_full_info => 'userFullInfo' },
    updateUserPrivacySettingRules                  => { rules => 'userPrivacySettingRules', setting => 'UserPrivacySetting' },
    updateUserStatus                               => { status => 'UserStatus' },
    updateWebBrowserSettings                       => { settings => 'webBrowserSettings' },
    updates                                        => { updates => 'array:Update' },
    upgradeGiftResult                              => { gift => 'upgradedGift' },
    upgradedGift                                   => { backdrop => 'upgradedGiftBackdrop', colors => 'upgradedGiftColors', host_id => 'MessageSender', model => 'upgradedGiftModel', original_details => 'upgradedGiftOriginalDetails', owner_id => 'MessageSender', resale_parameters => 'giftResaleParameters', symbol => 'upgradedGiftSymbol' },
    upgradedGiftBackdrop                           => { colors => 'upgradedGiftBackdropColors', rarity => 'UpgradedGiftAttributeRarity' },
    upgradedGiftBackdropCount                      => { backdrop => 'upgradedGiftBackdrop' },
    upgradedGiftModel                              => { rarity => 'UpgradedGiftAttributeRarity', sticker => 'sticker' },
    upgradedGiftModelCount                         => { model => 'upgradedGiftModel' },
    upgradedGiftOriginOffer                        => { price => 'GiftResalePrice' },
    upgradedGiftOriginResale                       => { price => 'GiftResalePrice' },
    upgradedGiftOriginalDetails                    => { receiver_id => 'MessageSender', sender_id => 'MessageSender', text => 'formattedText' },
    upgradedGiftSymbol                             => { rarity => 'UpgradedGiftAttributeRarity', sticker => 'sticker' },
    upgradedGiftSymbolCount                        => { symbol => 'upgradedGiftSymbol' },
    uploadStickerFile                              => { sticker => 'InputFile', sticker_format => 'StickerFormat' },
    user                                           => { active_story_state => 'ActiveStoryState', emoji_status => 'emojiStatus', profile_photo => 'profilePhoto', restriction_info => 'restrictionInfo', status => 'UserStatus', type => 'UserType', upgraded_gift_colors => 'upgradedGiftColors', usernames => 'usernames', verification_status => 'verificationStatus' },
    userAuctionBid                                 => { owner_id => 'MessageSender' },
    userFullInfo                                   => { bio => 'formattedText', birthdate => 'birthdate', block_list => 'BlockList', bot_info => 'botInfo', bot_verification => 'botVerification', business_info => 'businessInfo', first_profile_audio => 'audio', gift_settings => 'giftSettings', main_profile_tab => 'ProfileTab', note => 'formattedText', pending_rating => 'userRating', personal_photo => 'chatPhoto', photo => 'chatPhoto', public_photo => 'chatPhoto', rating => 'userRating' },
    userPrivacySettingRules                        => { rules => 'array:UserPrivacySettingRule' },
    userSupportInfo                                => { message => 'formattedText' },
    validateOrderInfo                              => { input_invoice => 'InputInvoice', order_info => 'orderInfo' },
    validatedOrderInfo                             => { shipping_options => 'array:shippingOption' },
    vectorPathCommandCubicBezierCurve              => { end_control_point => 'point', end_point => 'point', start_control_point => 'point' },
    vectorPathCommandLine                          => { end_point => 'point' },
    venue                                          => { location => 'location' },
    video                                          => { minithumbnail => 'minithumbnail', thumbnail => 'thumbnail', video => 'file' },
    videoChat                                      => { default_participant_id => 'MessageSender' },
    videoMessageAdvertisement                      => { sponsor => 'advertisementSponsor' },
    videoMessageAdvertisements                     => { advertisements => 'array:videoMessageAdvertisement' },
    videoNote                                      => { minithumbnail => 'minithumbnail', speech_recognition_result => 'SpeechRecognitionResult', thumbnail => 'thumbnail', video => 'file' },
    videoStoryboard                                => { map_file => 'file', storyboard_file => 'file' },
    viewMessages                                   => { source => 'MessageSource' },
    viewPremiumFeature                             => { feature => 'PremiumFeature' },
    voiceNote                                      => { speech_recognition_result => 'SpeechRecognitionResult', voice => 'file' },
    webApp                                         => { animation => 'animation', photo => 'photo' },
    webAppInfo                                     => { url => 'webAppUrl' },
    webAppOpenParameters                           => { mode => 'WebAppOpenMode', theme => 'themeParameters' },
    webBrowserSettings                             => { external_exceptions => 'array:webDomainException', in_app_exceptions => 'array:webDomainException' },
    webPageInstantView                             => { blocks => 'array:PageBlock', feedback_link => 'InternalLinkType' },
);

1;
